/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/bckprj.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 SUBROUTINE TO BACK-PROJECT THE DATA GIVEN THE THE ARRAY 'G'
 ONTO THE SQUARE ARRAY 'A'.  G IS A VECTOR OF LENGTH M; A IS AN
 N*N MATRIX.  THE BACK PROJECTION IS PERFORMED USING THE
 INTERPOLATION METHOD GIVEN BY 'INTERP' PARALLEL TO THE UNIT
 VECTOR GIVEN BY (CTH,STH).


 This subroutine is modified so that it is not necessary to
 call qintp for linear iteration (two-point Lagrange).
 The exacution is done within this subroutine.
 July 23, 1989
 */

#include <cstdio>

#include "blkdta.h"
#include "qintp.h"

#include "bckprj.h"

void bckprj(REAL* a, INTEGER n, REAL* g, INTEGER m, REAL sth, REAL cth,
		REAL spac, INTEGER interp)
{
	INTEGER epos;
	REAL nmid, mmid;

	INTEGER ind;
	REAL yinc;
	REAL xinc;
	REAL ypos;
	REAL pos;
	INTEGER j;
	INTEGER i;
	REAL er;
	REAL qintp1;

	nmid = (n - 1) / 2;
	mmid = (m + 1) / 2;
	ind = 0;
	yinc = cth * spac;
	xinc = -sth * spac;
	ypos = nmid * yinc + mmid;

	if (interp == 2)
	{
		for (i = 0; i < n; i++)
		{
			pos = -nmid * xinc + ypos;
			for (j = 0; j < n; j++)
			{
				epos = (INTEGER) pos;
				er = pos - epos;
				qintp1 = g[epos - 1] + er * (g[epos] - g[epos - 1]);
				a[ind++] += qintp1;
				pos += xinc;
			}
			ypos -= yinc;
		}
	}
	else
	{
		for (i = 0; i < n; i++)
		{
			pos = -nmid * xinc + ypos;
			for (j = 0; j < n; j++)
			{
				a[ind++] += qintp(pos, g, m, interp);
				pos += xinc;
			}
			ypos -= yinc;
		}
	}
	return;
}
