/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/errfac.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS ROUTINE ESTIMATES THE VARIANCE IN THE PROJECTION DATA
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "err.h"

#include "errfac.h"

REAL errfac(REAL prdta, REAL rinc, REAL rinc2)
{
	REAL errfac_tmp;
	REAL temp;

	if (Err.errtyp == 0)
	{
		return Err.minprd;
	}

	switch (Err.errtyp)
	{

	// ADDITIVE NOISE ONLY
	case 1:
		errfac_tmp = Err.afactr;
		break;

		// MULTIPLICATIVE NOISE ONLY
	case 2:
		errfac_tmp = Err.mfactr * prdta * prdta;
		break;

		// ADDITIVE AND MULTIPLICATIVE NOISE
	case 3:
		errfac_tmp = Err.mfactr * prdta * prdta + Err.afactr;
		break;

		// QUANTUM NOISE (LINE)
	case 4:
		errfac_tmp = Err.af + Err.bf * (REAL) exp(prdta);
		break;

		// QUANTUM (LINE) AND ADDITIVE NOISE
	case 5:
		errfac_tmp = Err.af + Err.bf * (REAL) exp(prdta) + Err.afactr;
		break;

		// QUANTUM (LINE) AND MULTIPLICATIVE NOISE
	case 6:
		errfac_tmp = Err.af + Err.bf * (REAL) exp(prdta);
		errfac_tmp = errfac_tmp + Err.mfactr * prdta * prdta;
		break;

		// QUANTUM (LINE) MULTIPLICATIVE AND ADDITIVE NOISE
	case 7:
		errfac_tmp = Err.af + Err.bf * (REAL) exp(prdta);
		errfac_tmp = errfac_tmp + Err.mfactr * prdta * prdta + Err.afactr;
		break;

		// QUANTUM (STRIP) NOISE
	case 8:
		prdta = prdta / rinc;
		errfac_tmp = (Err.af + Err.bf * (REAL) exp(prdta)) * rinc2;
		break;

		// QUANTUM (STRIP) AND ADDITIVE NOISE
	case 9:
		prdta = prdta / rinc;
		errfac_tmp = (Err.af + Err.bf * (REAL) exp(prdta)) * rinc2 + Err.afactr;
		break;

		// QUANTUM (STRIP) AND MULTIPLICATIVE NOISE
	case 10:
		temp = prdta / rinc;
		errfac_tmp = (Err.af + Err.bf * (REAL) exp(temp)) * rinc2
				+ Err.mfactr * prdta * prdta;
		break;

		// QUANTUM (STRIP) MULTIPLICATIVE AND ADDITIVE NOISE
	case 11:
		temp = prdta / rinc;
		errfac_tmp = (Err.af + Err.bf * (REAL) exp(temp)) * rinc2
				+ Err.mfactr * prdta * prdta + Err.afactr;
		break;
	}

	return MAX0(errfac_tmp, Err.minprd);;
}
