/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/eval.cpp $
 $LastChangedRevision: 179 $
 $Date: 2015-08-03 20:13:24 -0400 (Mon, 03 Aug 2015) $
 $Author: zye $
 ***********************************************************

 EVAL IS THE POST-PROCESSING ROUTINE WHICH DRIVES THE TWO
 RECONSTRUCTION EVALUATION PROCEDURES, DIST AND POINT.
 THE PARAMETER "TYPE" DETERMINES WHICH OF THESE ROUTINES TO
 PERFORM.
 IF TYPE=1 THEN DO DIST ONLY.
 IF TYPE=2 THEN DO POINT ONLY.
 OTHERWISE, DO BOTH DIST AND POINT.
 MODIFIED TO COMPUTE THE RELATIVE ERROR AS DEFINED BY RAMA+LAK.
 ALSO, MODIFIED SO THAT THE RELATIVE ERROR AND DISTANCE
 MEASURES ARE NOT NORMALIZED IF ABSSUM AND TSTD ARE
 TOO SMALL. ...4/27/76

 Modified 4/1/88 to enable evaluation of subregions, either
 through boundary selection by geomtric descriptions OR (OR BOTH)
 pixel selection through density windowing wrt to phantom.
 Only pixels within the specified regions and density window
 will be used in computing the metrics.
 A switch ISW is added to indicate whether the phantom is passed through
 the first array, a(), or the second array, b(), so that density
 selection can always be referenced wrt the phantom.
 */

#include <cstdlib>       
#include <cstdio>
#include "blkdta.h"
#include "consts.h"
#include "uiod.h"
#include "bldlst.h"
#include "int2str.h"
#include "recfile.h"
#include "projfile.h"
#include "infile.h"
#include "eval.h"   
#include "blob.h"

Eval_class Eval;

void Eval_class::eval_1()
{
	BOOLEAN temp;
	FILE* eval1;

	CHAR prjnam[81];
	CHAR phnnam[81];
	CHAR recnam[81];
	CHAR evalnam[81];

	int i, j;

	static const CHAR * objname[] =	{ "wholepic", "rect", "elip" };

	REAL *ave;
	REAL *dis;
	REAL *rel;
	REAL *var;
	REAL *std;
	REAL *tstd;
	REAL *tvar;
	REAL *abssum;
	REAL resid;
	REAL klds;
	REAL wsqd;

	INTEGER nerr;
	REAL epsil[16];
	INTEGER flag[51];
	CHAR algn[5];
	unsigned INTEGER count;
	unsigned INTEGER iter;
	INTEGER flag0;
	REAL* Phantom;
	REAL* Recon;
	INTEGER* list = NULL;
	REAL* weight = NULL;
	INTEGER type;
	INTEGER word;
	BOOLEAN eol, linef, linef_set = false;
	BOOLEAN fresid, fklds, fwsqd, fulimg, flagdw;

	int n;

	static bool called = false;

	static const INTEGER eval_codes[9] =
	{
		CHAR2INT('r', 'e', 's', 'o'),
		CHAR2INT('p', 'o', 'i', 'n'),
		CHAR2INT('l', 'i', 'n', 'e'),
		CHAR2INT('s', 't', 'r', 'i'),
		CHAR2INT('w', 'h', 'o', 'l'),
		CHAR2INT('s', 'e', 'l', 'e'),
		CHAR2INT('e', 'l', 'i',	'p'),
		CHAR2INT('r', 'e', 'c', 't'),
		CHAR2INT('l', 'a', 's', 't')
	};

	// Parse input
	// Read evaluation type specifications

	word = InFile.getwrd(FALSE, &eol, eval_codes, 4);
	if (eol)
	{
		type = 0;
	}
	else
	{
		if (word == eval_codes[0] || word == eval_codes[1])
		{
			type = (word == eval_codes[0]) ? 1 : 2;
			word = InFile.getwrd(FALSE, &eol, &(eval_codes[2]), 2);
		}
		else
		{
			type = 0;
		}

		if (word == eval_codes[2])
		{
			linef = TRUE;
			linef_set = true;
		};
		if (word == eval_codes[3])
		{
			linef = FALSE;
			linef_set = true;
		};
	};

	// Read evaluation name
	InFile.GetComment(evalnam);
	fprintf(output, "\n         %s", evalnam);

	// Read evaluation region specifications
	word = InFile.getwrd(TRUE, &eol, &(eval_codes[4]), 2);

	if (eol)
	{
		fprintf(output, "\n **** no region (WHOLEPIC or SELECTIVE) specified");
		fprintf(output, "\n **** evaluation aborted\n");
		return;
	};

	if (word == eval_codes[4])
	{
		// Whole picture is specified
		ObjN = 1; // set number of objects to 1
		Obj = new Object[ObjN];
		getden(&flagdw, 0); // get densities
		Obj[0].objcod = 0; // set object type
		fulimg = TRUE; // set full immage flag
	}
	else
	{
		//  Selective is specified

		ObjN = 0;
		fulimg = FALSE;

		eval_Object_Node *f_object, *t_object;
		f_object = new eval_Object_Node();
		t_object = f_object;

		while(true)
		{
			word = InFile.getwrd(TRUE, &eol, &(eval_codes[6]), 3);

			if (eol)
			{
				fprintf(output, "\n **** missing keyword elip, rect or last");
				fprintf(output, "\n **** program aborted\n");
				exit(999);
			};

			if (word == eval_codes[8])
			{
				// keyword last found
				if (ObjN > 0) break;

				fprintf(output, "\n **** no region specified for SELECTIVE");
				fprintf(output, "\n **** program aborted\n");
				exit(999);
			};

			t_object->data.objcod = (word == eval_codes[7]) ? 1 : 2;

			// get cx
			t_object->data.cx = InFile.getnum(FALSE, &eol);
			if (eol)
			{
				fprintf(output, "\n **** parameter cx of object %s is missing",int2str(word));
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			};

			// get cy
			t_object->data.cy = InFile.getnum(FALSE, &eol);
			if (eol)
			{
				fprintf(output, "\n **** parameter cy of object %s is missing",int2str(word));
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			};

			// get u
			t_object->data.u = InFile.getnum(FALSE, &eol);
			if (eol)
			{
				fprintf(output, "\n **** parameter u of object %s is missing",int2str(word));
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			};

			// get v
			t_object->data.v = InFile.getnum(FALSE, &eol);
			if (eol)
			{
				fprintf(output, "\n **** parameter v of object %s is missing",int2str(word));
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			};

			// get ang
			t_object->data.ang = InFile.getnum(FALSE, &eol);
			if (eol)
			{
				fprintf(output, "\n **** parameter ang of object %s is missing",int2str(word));
				fprintf(output, "\n **** program aborted\n");
				exit(-1);
			};

			getdens(&flagdw, t_object);

			t_object->next = new eval_Object_Node();
			t_object = t_object->next;

			ObjN++;
		}

		// copy list elements to array Obj
		if (Obj != NULL)
		{
			delete[] Obj;
			delete[] objdon;
		}
		Obj = new Object[ObjN];
		objdon = new BOOLEAN[ObjN];

		t_object = f_object;

		for (i = 0; i < ObjN; i++)
		{
			Obj[i].cx = t_object->data.cx;
			Obj[i].cy = t_object->data.cy;
			Obj[i].u = t_object->data.u;
			Obj[i].v = t_object->data.v;
			Obj[i].ang = t_object->data.ang;
			Obj[i].objcod = t_object->data.objcod;
			Obj[i].t1 = t_object->data.t1;
			Obj[i].t2 = t_object->data.t2;

			t_object = t_object->next;
			delete f_object;
			f_object = t_object;
		};

		delete f_object;
	}

	// create arrays
	ave = new REAL[ObjN];
	dis = new REAL[ObjN];
	rel = new REAL[ObjN];
	var = new REAL[ObjN];
	std = new REAL[ObjN];
	tstd = new REAL[ObjN];
	tvar = new REAL[ObjN];
	abssum = new REAL[ObjN];

	//  Echo region specifications

	fprintf(output,"\n Region        cx        cy         u         v        ang      t1        t2\n");

	if (Obj[0].objcod == 0)
	{
		fprintf(output,"\n %s                                                     %9.2g %9.2g",objname[0], Obj[0].t1, Obj[0].t2);
	}
	else
	{
		for (j = 0; j < ObjN; j++)
		{
			fprintf(output,"\n %2i %s  %9.2f %9.2f %9.2f %9.2g %9.2g   %9.2g %9.2g", j,objname[Obj[j].objcod], Obj[j].cx, Obj[j].cy, Obj[j].u,Obj[j].v, Obj[j].ang, Obj[j].t1, Obj[j].t2);
		}
	}

	// READ AND ECHO FLAGS
	InFile.listit(flag);
	fresid = FALSE;
	fklds = FALSE;
	fwsqd = FALSE;

	for(i=1;i<51;i++){
		if(flag[0]==flag[i]||flag[i]==0);
		else{
                        fprintf(output, "\n **** The value of fl[0] is %d, but the value of fl[%d] is %d ",flag[0],i,flag[i]);
                        fprintf(output, "\n **** program aborted\n");
                        exit(-1);
		}
	}
	if(flag[0]==2){fresid = TRUE;}
	if(flag[0]==3){fklds = TRUE; fwsqd = TRUE; fresid = TRUE;}

	/* for (i = 0; i < 51; i++)
	{
		if (flag[i] >= 3)
		{
			fresid = TRUE;
			fklds = TRUE;
			fwsqd = TRUE;
			break;
		}
		else if (flag[i] >= 2)
		{
			fresid = TRUE;
			break;
		}
	} */

	// RESIDUAL CANNOT AND WILL NOT BE CALCULATED IF SOME PIXELS
	// ARE EXCLUDED BY DENSITY WINDOWING OR REGION SELECTION.
	// RESET RESIDUAL FLAG
	if (!fulimg)
	{
		fresid = FALSE;
		fklds = FALSE;
		fwsqd = FALSE;
	}
	else if (flagdw)
	{
		fresid = FALSE;
		fklds = FALSE;
		fwsqd = FALSE;
	}

	flag0 = (fresid) ? 2 : 1;

	////////////////////
	// START evaluate //
	////////////////////

	// check if called before
	if (called == false)
	{
		eval1 = fopen("eval", "w"); // open new file
		called = true;
	}
	else
	{
		eval1 = fopen("eval", "a+"); // open existing file for appending
	}

	//  print evaluation name
	fprintf(eval1, "\n\nevaluation name: %s\n", evalnam);

	// open recfile
	if (RecFile.Open("recfil") != 0)
	{
		fprintf(output, "\n **** unable to open recfil");
		fprintf(output, "\n **** EVAL execution aborted\n");
		fclose(eval1);
		return;
	}

	// Get projection name
	if (RecFile.GetProjName(prjnam) != 0)
	{
		;
	}

	// get picture dimension
	if (RecFile.GetNelem(&(GeoPar.nelem)) != 0)
	{
		;
	}

	// get picture dimension
	if (RecFile.GetPixSize(&(GeoPar.pixsiz)) != 0)
	{
		;
	}

	// calculate picture area
	GeoPar.area = GeoPar.nelem * GeoPar.nelem;
	GeoPar.midpix = (GeoPar.nelem + 1) / 2;

	// allocate phantom and reconstruction arrays
	Phantom = new REAL[GeoPar.area];
	Recon = new REAL[GeoPar.area];

	// read phantom
	if (RecFile.ReadPhan(phnnam, Phantom) != 0)
	{
		fprintf(output, "\n **** phantom not present");
		fprintf(output, "\n **** EVAL execution aborted\n");
		fclose(eval1);
		return;
	}

	if (type != 2)
	{
		// calculate resolution

		// open prjfil only if residual is calculated
		if (fresid)
		{
			if (ProjFile.Open("prjfil") != 0)
			{
				fprintf(output, "\n **** unable to open prjfil");
				fprintf(output, "\n **** EVAL execution aborted\n");
				fclose(eval1);
				return;
			};

			ProjFile.ReadHeader(prjnam, &NoisePar, &Spctrm, &Anglst);

			if (!linef_set) linef = GeoPar.line;

			bldlst(&list, &weight);
		};

		// COMPUTE AVERAGE, VARIANCE AND STANDARD DEVIATION OF TEST PATTERN

		for (j = 0; j < GeoPar.area; j++)
			Recon[j] = 0.0;
		temp=Blob.pix_basis;
		Blob.pix_basis = TRUE;
		dist(1, Recon, Phantom, linef, list, weight, flag0, ave, dis, abssum, tvar, tstd, &resid, fresid, &klds, fklds, &wsqd, fwsqd);
		Blob.pix_basis=temp;

		fprintf(eval1, "\n  global resolution measures\n");

		fprintf(eval1, "\n    phantom name:    %s", phnnam);

		fprintf(eval1, "\n    projection name: %s", prjnam);

		if (fresid)
		{
			if (linef)
			{
				fprintf(output,"\n    residual calculation using line integration");

			}
			else
			{
				fprintf(output,"\n    residual calculation using strip integration");
			}
		}

		fprintf(eval1, "\n\n    metrics for test phantom");

		if (fresid && (fklds || fwsqd))
		{
			fprintf(eval1,"\n    region    area   average  distance   rel err  variance   std dev   residual         klds         wsqd");
		}
		else if (fresid)
		{
			fprintf(eval1,"\n    region    area   average  distance   rel err  variance   std dev   residual");
		}
		else
		{
			fprintf(eval1,"\n    region    area   average  distance   rel err  variance   std dev");
		}

		for (j = 0; j < ObjN; j++)
		{
			Obj[j].avep = ave[j];

			if (fresid && (fklds || fwsqd))
			{
				fprintf(eval1,"\n    %6d %7d %9.4f                     %9.4f %9.4f %10.4f %12.4f %12.4f",j, Obj[j].npt, Obj[j].avep, tvar[j], tstd[j], resid, klds, wsqd);
			}
			else if (fresid)
			{
				fprintf(eval1,"\n    %6d %7d %9.4f                     %9.4f %9.4f %10.4f",j, Obj[j].npt, Obj[j].avep, tvar[j], tstd[j], resid);
			}
			else
			{
				fprintf(eval1,"\n    %6d %7d %9.4f                     %9.4f %9.4f",j, Obj[j].npt, Obj[j].avep, tvar[j], tstd[j]);
			}
		}

		while (RecFile.ReadRec(recnam, algn, &count, &iter, Recon) == 0)
		{

			// if first iteration of algorithm print header
			if (count == 1)
			{
				fprintf(eval1, "\n\n    execution name: %s", recnam);

				if (fulimg)
				{
					fprintf(eval1, "\n    metrics for algorithm %s", algn);

					if (fresid && (fklds || fwsqd))
					{
						fprintf(eval1,"\n    iter    area   average  distance   rel err  variance   std dev   residual         klds         wsqd");
					}
					else if (fresid)
					{
						fprintf(eval1,"\n    iter    area   average  distance   rel err  variance   std dev   residual");
					}
					else
					{
						fprintf(eval1,"\n    iter    area   average  distance   rel err  variance   std dev");
					}
				}
			}

			// if iteration not selected continue
			if (flag[iter - 1] == 0)
			{
				continue;
			}

			if (!fulimg)
			{
				fprintf(eval1, "\n    metrics for algorithm %4s iteration %4d",algn, count);

				fprintf(eval1, "\n    region    area   average  distance   rel err  variance   std dev");
			}
			temp = Blob.pix_basis;
			Blob.pix_basis = TRUE;
			dist(0, Phantom, Recon, linef, list, weight, flag[iter - 1], ave, dis, rel, var, std, &resid, fresid, &klds, fklds, &wsqd, fwsqd);
			Blob.pix_basis = temp;
			// Loop for each regions
			for (j = 0; j < ObjN; j++)
			{
				if (abssum[j] > Consts.zero)
				{
					rel[j] /= abssum[j];
				}

				if (tstd[j] > Consts.zero)
				{
					dis[j] /= tstd[j];
				}

				if (((flag[iter - 1] == 1) || (flag[iter - 1] > 1)) && (!fresid))
				{
					if (fulimg)
					{
						fprintf(eval1,"\n    %4d %7d %9.4f %9.4f %9.4f %9.4f %10.4f", count, Obj[j].npt, ave[j], dis[j], rel[j], var[j], std[j]);
					}
					else
					{
						fprintf(eval1,"\n    %6d %7d %9.4f %9.4f %9.4f %9.4f %10.4f", j, Obj[j].npt, ave[j], dis[j], rel[j], var[j], std[j]);
					}
				}

				if ((flag[iter - 1] > 1) && fresid && (fklds || fwsqd))
				{
					fprintf(eval1,"\n    %4d %7d %9.4f %9.4f %9.4f %9.4f %9.4f %10.4f %12.4f %12.4f", count, Obj[j].npt, ave[j], dis[j], rel[j], var[j], std[j], resid, klds, wsqd);
				}
				else if ((flag[iter - 1] > 1) && fresid)
				{
					fprintf(eval1,"\n    %4d %7d %9.4f %9.4f %9.4f %9.4f %9.4f %10.4f", count, Obj[j].npt, ave[j], dis[j], rel[j], var[j], std[j], resid);
				}
			}

			if (ObjN > 1)
			{
				fprintf(eval1, "\n");
			}
		};

		if (fresid)
		{
			ProjFile.Close();
		}
	};

	// calculate point measurements
	// if type was 1 we are done
	if (type == 1 || !fulimg || (fulimg && flagdw))
	{

		if (type != 1)
		{

			if (!fulimg)
			{
				fprintf(output,"\n **** Selective was specified, point measures will not be computed.\n");
			}
			else
			{
				fprintf(output,"\n **** Density range was specified, point measures will not be computed.\n");
			}
		}
	}
	else
	{ // if type != 1 -> type == 0 (both) or type = 2

		fprintf(eval1, "\n  point by point resolution measures\n");

		fprintf(eval1, "\n    phantom name:    %s", phnnam);

		if (fresid || (type != 2))
		{
			fprintf(eval1, "\n    projection name: %s", prjnam);
		}

		// select first reconstruction in file
		RecFile.SelectRec(0);

		// loop though receonstructions
		while (RecFile.ReadRec(recnam, algn, &count, &iter, Recon) == 0)
		{

			BOOLEAN headfl;

			// if first iteration of algorithm print header
			if (count == 1)
			{
				headfl = true;
			}

			// if iteration not selected continue
			if (flag[iter - 1] == 0)
			{
				continue;
			}

			point(Phantom, Recon, Recon, GeoPar.nelem, &nerr, epsil);

			if (headfl)
			{
				fprintf(eval1, "\n\n    execution name:  %s", recnam);

				fprintf(eval1, "\n    algn    iter"); // only print out 'e(n)' as many times as there
				for (n = 0; n < nerr; n++)
				{ // are values in 'epsil[]' (i.e. 'nerr' times)
					fprintf(eval1, "      e(%d)", n);
				}

				headfl = false;
			}

			fprintf(eval1, "\n    %4s  %6d", algn, count);
			for (n = 0; n < nerr; n++)
			{
				fprintf(eval1, " %9.4f", epsil[n]);
			}
		}
	}

	delete[] Obj;
	Obj = NULL;
	delete[] ave;
	ave = NULL;
	delete[] dis;
	dis = NULL;
	delete[] rel;
	rel = NULL;
	delete[] var;
	var = NULL;
	delete[] std;
	std = NULL;
	delete[] tstd;
	tstd = NULL;
	delete[] tvar;
	tvar = NULL;
	delete[] abssum;
	abssum = NULL;

	fprintf(eval1, "\n ");
	fclose(eval1);

	RecFile.Close();

	delete[] Phantom;
	Phantom = NULL;
	delete[] Recon;
	Recon = NULL;
	if (list != NULL) delete[] list;
	list = NULL;
	if (weight != NULL) delete[] weight;
	weight = NULL;
}
