/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_vcase2.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ------------------------------------------------------------------

 VCASE2: VARIABLE SPACING CASE 2
 PURPOSE: FOR VARIABLE SPACING, ICASE=2 (THE TWO LINES STADLE
 THE -45 DEGREE LINE) FIND ALL POINTS THAT LIE
 BETWEEN THE TWO LINES AND INTERPOLATE
 PJ1 AND PJ2 ARE THE ARRAYS  HOLDING THE TRANSFORMED PROJS
 ANG1 AND ANG2 ARE THE CORRESPONDING ANGLES

 ------------------------------------------------------------------
 */
// bug79 - problem with fourier algorithm
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "fourie.h"
#include "consts.h"

#include "foru.h"

#define  cot(z)  (cos(z)/sin(z));

void foru_class::vcase2(REAL ang1, REAL ang2, REAL* pj1, REAL* pj2)
{
	REAL xx[4];
	REAL yy[4];

	REAL uleft;
	REAL uhigh;
	INTEGER kh;
	INTEGER j;
	INTEGER iy;
	REAL aleft;
	REAL ahigh;
	INTEGER indl;
	INTEGER indh;
	REAL a;
	INTEGER low;
	INTEGER indmin;
	INTEGER indmax;
	REAL hdis;
	REAL vdis;
	REAL distan;
	REAL yg;
	INTEGER ix;
	REAL xg;
	REAL gr;
	REAL gi;
	REAL amid;
	INTEGER ind;
	INTEGER x;
	REAL t;
	REAL pr;
	REAL qr;
	REAL px;
	REAL qi;
	REAL s;
	INTEGER y;

	//tan(z)=sin(z)/cos(z)
	///cot(z)=cos(z)/sin(z)

	uleft = (REAL) cot(ang1)
	;
	uhigh = (REAL) tan(ang2);
	kh = Fourie.nsize2 / 2 - 1;

	for (j = 1; j <= kh; j++)
	{     // bug79 - swr 2/11/05

		iy = -j;
		aleft = uleft * iy;
		ahigh = uhigh * j;
		indl = (INTEGER) ceil(aleft - Fourie.epsln); // bug82 - changed to ceil - swr - 2/20/05
		indh = j - 1;
		a = j * Fourie.delp / Fourie.deld;
		low = (INTEGER) a;
		indmin = 2 * low;     // bug79 - swr 2/11/05
		indmax = indmin + 2;
		hdis = j - aleft;
		vdis = ahigh - (-j);
		distan = hdis + vdis;

		switch (Fourie.interp)
		{

		// INTERP = 1 NEAREST NEIGHBOR IN CARTESIAN SENSE
		// INTERP = 4 F=(F1/D1+F2/D2+F3/D3+F4/D4)/(1/D1+1/D2+1/D3+1/D4)
		case 1:
		case 4:
			yy[1] = (REAL) -low;
			yy[2] = (REAL) -low - 1;
			xx[1] = yy[1] * uleft;
			xx[2] = yy[2] * uleft;
			xx[3] = (REAL) low;
			xx[4] = (REAL) low + 1;
			yy[3] = xx[3] * uhigh;
			yy[4] = xx[4] * uhigh;
			yg = -a;
			if (indl <= indh)
			{
				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					xg = ix * Fourie.delp / Fourie.deld;
					vint14(xx, yy, xg, yg, pj1, pj2, indmin, indmax, &gr, &gi); // changed to pass address of "gr" and "gi". Lajos, Feb 10, 2005
					store(ix, iy, gr, gi);
				}
			}
			break;

			// INTERP = 2 NEAREST NEIGHBOR IN RADIAL SENSE
		case 2:

			amid = aleft + distan / (REAL) 2.0;
			ind = 2 * (lround(a)) + 1; // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			if (indl <= indh)
			{
				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					x = ix;
					if (x <= amid)
					{     // bug79 - swr 2/15/05
						gr = pj1[ind - 1];
						gi = pj1[ind];
					}
					else
					{
						gr = pj2[ind - 1];
						gi = pj2[ind];
					}
					store(ix, iy, gr, gi);
				}
			}
			break;

			// INTERP = 3 LINEAR INTERPOLATION
		case 3:
			t = a - low;
			if (t <= Consts.zero)
				t = 0.0;
			if ((1. - t) <= Consts.zero)
				t = 1.0;
			pr = pj1[indmin] * ((REAL) 1.0 - t) + pj1[indmax] * t;
			qr = pj2[indmin] * ((REAL) 1.0 - t) + pj2[indmax] * t;
			px = pj1[indmin + 1] * ((REAL) 1.0 - t) + pj1[indmax + 1] * t;
			qi = pj2[indmin + 1] * ((REAL) 1.0 - t) + pj2[indmax + 1] * t;
			if (indl <= indh)
			{
				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					s = (ix - aleft) / distan;
					if (s <= Consts.zero)
						s = 0.0;
					if ((1. - s) <= Consts.zero)
						s = 1.0;
					gr = pr * ((REAL) 1.0 - s) + qr * s;
					gi = px * ((REAL) 1.0 - s) + qi * s;
					store(ix, iy, gr, gi);
				}
			}
			break;

		}

		ix = j;
		indl = -j;
		indh = (INTEGER) floor(ahigh + Fourie.epsln); // bug82 - changed to floor - swr - 2/20/05

		if (indl <= indh)
		{

			switch (Fourie.interp)
			{

			// INTERP = 1 NEAREST NEIGHBOR IN CARTESIAN SENSE
			// INTERP = 4 F=(F1/D1+F2/D2+F3/D3+F4/D4)/(1/D1+1/D2+1/D3+1/D4)
			case 1:
			case 4:
				xg = a;
				for (iy = indl; iy <= indh; iy++)
				{     // bug79 - swr 2/11/05
					yg = iy * Fourie.delp / Fourie.deld;
					vint14(xx, yy, xg, yg, pj1, pj2, indmin, indmax, &gr, &gi); // changed to pass address of "gr" and "gi". Lajos, Feb 10, 2005
					store(ix, iy, gr, gi);
				}
				break;

				// INTERP = 2 NEAREST NEIGHBOR IN RADIAL SENSE
			case 2:
				amid = ahigh - distan / (REAL) 2.0;
				for (iy = indl; iy <= indh; iy++)
				{     // bug79 - swr 2/11/05
					y = iy;
					if (y <= amid)
					{     // bug79 - swr 2/15/05
						gr = pj1[ind - 1];
						gi = pj1[ind];
					}
					else
					{
						gr = pj2[ind - 1];
						gi = pj2[ind];
					}
					store(ix, iy, gr, gi);
				}
				break;

				// INTERP = 3 LINEAR INTERPOLATION
			case 3:
				for (iy = indl; iy <= indh; iy++)
				{     // bug79 - swr 2/11/05
					s = (hdis + iy - (-j)) / distan;
					if (s <= Consts.zero)
						s = 0.0;
					if ((1. - s) <= Consts.zero)
						s = 1.0;
					gr = pr * ((REAL) 1.0 - s) + qr * s;
					gi = px * ((REAL) 1.0 - s) + qi * s;
					store(ix, iy, gr, gi);
				}
				break;
			}
		}
	}
}
