/* 
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/lino.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ******************************************************************

 - LINOGRAM RECONSTRUCTION ALGORITHM -

 THIS PROGRAM IMPLEMENTS AN IMAGE RECONSTRUCTION TECHNIQUE
 DERIVED BY PAUL EDHOLM AND GABOR T. HERMAN.  DATA IS COLLECTED
 IN A SPECIAL GEOMETRY,AND THE IMAGE IS THEN RECONSTRUCTED
 (ESSENTIALLY) BY USING 3 ONE-DIMENSIONAL FOURIER TRANSFORMS.
 
 VERSION 1.0 :     LYLE BERKOWITZ  AUGUST, 1986
 1.1 :     DAVID A. ROBERTS   MAY, 1987
 :     RESTRUCTURED PROGRAM
 ADDED GRIDDING ALGORITHM
 1.2 :     DAVID A. ROBERTS  JUNE, 1987
 :     USE OF RTFORT TO IMPROVE SPEED
 1.3 :     DAVID A. ROBERTS  JULY, 1987
 :     USE OF CHIRP-Z ALGORITHM
 TO COMPUTE SECOND DFT
 1.4 :     D. ODHNER  JAN, 1989
 ******************************************************************
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"
#include "int2str.h"
#include "anglst.h"
#include "rtfort.h"
#include "projfile.h"
#include "infile.h"
#include "qfilt.h"

#include "lino.h"

BOOLEAN lino_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{

	static const REAL padwgt = 3.0;

	//     NEW VARIABLES

	INTEGER i, j, k, r1, ii, iii, iflag, i3, kk;
	INTEGER b, word, filter, n3[3], np, np1, np2;
	INTEGER nnew;
	REAL* img;
	INTEGER glnum, gl1num, imgnum;
	REAL w, r, theta, sinth, costh, dold, dnew, dist, rinc;
	REAL sum, diff, c, cutoff, factr1, factr2, factr3;
	BOOLEAN eol;

	REAL* g;
	REAL* gl;
	REAL *gl1;
	REAL* otemp;
	REAL* temp;

	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 13, 2004
	static const INTEGER fname[3] =
	{ CHAR2INT('b', 'a', 'n', 'd'), CHAR2INT('s', 'i', 'n', 'c'), CHAR2INT('c',
			'o', 's', 'i') };

	INTEGER intflag = 2;

	//commented out since it is never used. Lajos, Dec 16, 2004
	//static const INTEGER pad = CHAR2INT('p','a','d',' ');

	int n;

//-------------------------------------------------------------------
//...              START MAIN PROGRAM
//------------------------------------------------------------------

	// INITIALIZATION

	factr2 = 1.0;

	// READ IN FILTER NAME FROM USER INPUT FILE

	word = InFile.getwrd(TRUE, &eol, fname, 3);
	for (filter = 0; filter < 3; filter++)
	{
		if (word == fname[filter])
			goto L0003;
	}

	fprintf(output, "\n **** %4s is not a valid filter name", int2str(word));
	fprintf(output, "\n **** LINOGRAM execution aborted\n");
	return TRUE;

	L0003: cutoff = InFile.getnum(FALSE, &eol);

	if (eol)
	{
		fprintf(output, "\n **** not enough parameters");
		fprintf(output, "\n **** LINOGRAM execution aborted\n");
		return TRUE;
	}

	if (cutoff <= 0)
		cutoff = (REAL) 0.0001;

	fprintf(output, "\n         linogram reconstruction algorithm");
	fprintf(output, "\n          using filter %4s", int2str(word));
	fprintf(output, "\n          cutoff = %9.4f", cutoff);

	// INITIALIZE LOOP INVARIANTS

	nnew = (GeoPar.nelem - 1) / 2;
	n3[0] = 1;
	n3[2] = 1;

	// Decide whether to pad
	k = 0;

	while ((1 << k) < (GeoPar.nelem + 1))
	{
		k++;
	}

	if (((1 << k) + padwgt * 2 * k)
			<= (GeoPar.nelem + 1 + padwgt * sumfac(GeoPar.nelem + 1)))
	{
		n3[1] = 2;

		while (n3[1] < (GeoPar.nelem + 1))
		{
			n3[1] *= 2;
		}

	}
	else
	{
		n3[1] = GeoPar.nelem + 1;
	}

	dold = (REAL) 2.0 / ((REAL) (GeoPar.usrays));
	dist = 1 / (2 * n3[1] * GeoPar.pixsiz);
	gl1num = 2 * n3[1] + 2;
	glnum = gl1num * GeoPar.usrays;
	imgnum = GeoPar.nelem * gl1num;

	// G IS THE BASE OF PROJECTION DATA

	g = new REAL[GeoPar.usrays + 2];

	//******************************************************C
	//...  BEGIN THE RECONSTRUCTION
	//******************************************************C

	// FIRST LOOP ON SETS OF PROJECTION ANGLES
	//   IFLAG = 0 FOR THE  FIRST SET OF ANGLES (T)
	//   IFLAG = 1 FOR THE SECOND SET OF ANGLES (C)

	for (iflag = 0; iflag <= 1; iflag++)
	{
		// GET THE FIRST AND LAST PROJECTION NUMBERS

		if (iflag == 0)
		{
			np1 = 0;
			np2 = GeoPar.usrays;
		}
		else
		{
			np1 = GeoPar.usrays;
			np2 = 2 * GeoPar.usrays;
		}

		img = new REAL[imgnum];
		for (ii = 0; ii < imgnum; ii++)
		{
			img[ii] = 0.0;
		}

		gl = new REAL[glnum];
		for (ii = 0; ii < glnum; ii++)
		{
			gl[ii] = 0.0;
		}

		gl1 = new REAL[gl1num];
		for (ii = 0; ii < gl1num; ii++)
		{
			gl1[ii] = 0.0;
		}

		b = 2;
		for (np = np1; np < np2; np++)
		{
			for (ii = 0; ii < GeoPar.usrays + 2; ii++)
			{
				g[ii] = 0.0;
			}

			ProjFile.ReadProj(np, g, GeoPar.usrays);

			Anglst.getang(np, &theta, &sinth, &costh);

			//-------------------------------------------------------------------C
			//...  1) READ IN THE PROJECTION DATA BACKWARDS
			//...  2) MAKE SURE GL1 ARRAY IS IN PROPER FORMAT BEFORE FFT
			//...     (I.E. WITH THE ZERO VALUE AT THE BEGINNING)
			//-------------------------------------------------------------------C

			i3 = 0;
			for (i = GeoPar.nelem; i >= 0; i--)
			{
				gl1[i3] = g[i];
				i3++;
			}

			for (i = GeoPar.usrays + 1; i <= 2 * n3[1]; i++)
			{
				gl1[i3] = 0.0;
				i3++;
			}

			for (i = GeoPar.usrays; i >= GeoPar.nelem + 2; i--)
			{
				gl1[i3] = g[i - 1];
				i3++;
			};

			//------------------------------------------------------------C
			//...  TAKE FIRST 1-D FOURIER TRANSFORM; USRAY-DIRECTION
			//...  ** N3(2) IS ASSIGNED HALF THE NUMBER OF ELEMENTS **
			//------------------------------------------------------------C

			rtfort(gl1, n3, 1);

			//------------------------------------------------------------C
			//...  MULTIPLY BY FILTER AND WEIGHT (AND SAMPLE SPACING)
			//------------------------------------------------------------C

			w = (REAL) fabs(sinth);
			if (iflag == 1)
				w = (REAL) fabs(costh);
			c = cutoff / (2 * GeoPar.pinc * w);
			j = 0;
			rinc = dist / w;
			r = 0.0;
			for (r1 = 0; r1 <= n3[1]; r1++)
			{
				if (r <= Consts.zero)
				{
					gl1[j] = 0.0;
					gl1[j + 1] = 0.0;
				}
				else
				{
					factr1 = GeoPar.pinc * w * w;
					if (intflag == 1)
						factr2 = qfilt(r, c, 1) / r;
					if (intflag == 2)
					{
						factr2 = qfilt(r, c, 1) / r;
						factr2 = factr2 * factr2;
					}
					factr3 = qfilt(r, c, filter);
					gl1[j] = factr1 * factr2 * factr3 * gl1[j];
					gl1[j + 1] = factr1 * factr2 * factr3 * gl1[j + 1];
				}

				//-------------------------------------------------------------------C
				//...  STORE DATA IN NEW ARRAY (2-DIMENSIONAL): [NELEM+2,4NELEM+2]
				//-------------------------------------------------------------------C

				gl[(r1 + 1) * 2 * GeoPar.usrays - b] = gl1[j];
				gl[(r1 + 1) * 2 * GeoPar.usrays - b + 1] = gl1[j + 1];
				j += 2;
				r += rinc;
			}
			b += 2;
		}

		//************************************C
		//  END OF LOOP ON PROJECTIONS
		//************************************C

		//------------------------------------------------------------------
		//...  PERFORM GRIDDING (INCORPORATES A 1-D FFT IN ANGULAR VARIABLE)
		//...  STORE OUTPUT OF GRIDDING IN TWO DIMENSIONAL ARRAY:
		//...  [NELEM+1,2*NELEM]
		//-------------------------------------------------------------------

		otemp = new REAL[2 * GeoPar.nelem];

		iii = 0;
		for (n = 0; n < n3[1] + 1; n++)
		{
			if (n == 0)
			{

				// SPECIAL CASE WHEN n = 1
				// DUE TO NATURE OF FILTERING, DC VALUE WILL ALWAYS BE ZERO.

				for (kk = 0; kk < 2 * GeoPar.nelem; kk++)
				{
					otemp[kk] = 0.0;
				}
			}
			else
			{

				// ****  CHANGED DNEW FOR RTFORT!     ****
				//   (DUE TO PADDING WITH AN EXTRA ZERO)

				dnew = ((REAL) (n)) / (2 * n3[1]);

				// ****  USE CHIRP-Z ****

				temp = &(gl[(n) * 2 * GeoPar.usrays]);
				czt(temp, otemp, dold, dnew, GeoPar.nelem, nnew);
			}

			//-------------------------------------------------------------------
			//...  WRITE 'OTEMP' INTO THE 2D ARRAY 'IMG' ON A COLUMN-BY-COLUMN BASIS
			//-------------------------------------------------------------------

			ii = 0;
			for (k = 0; k < GeoPar.nelem; k++)
			{
				img[k * gl1num + iii] = otemp[ii];
				img[k * gl1num + iii + 1] = otemp[ii + 1];
				ii += 2;
			}
			iii += 2;
		}
		//-----------------------------------------------------------------*
		//...  PERFORM AN INVERSE FFT ALONG ROWS IN THE ARRAY 'IMG'.
		//...  THE FOURIER SAMPLES ARE ARRANGED IN EACH ROW STARTING FROM
		//...  0 (DC).
		//-----------------------------------------------------------------*

		// LOOP ON ROWS OF 'IMG' (SHOULD BE 'NELEM' ROWS)

		for (n = 0; n < GeoPar.nelem; n++)
		{
			rtfort(&(img[n * gl1num]), n3, -1);

			//----------------------------------------------C
			//     MULTIPLY BY SAMPLE SPACING AFTER FFT:
			//----------------------------------------------C

			for (k = 0; k < 2 * n3[1]; k++)
			{
				img[(n) * gl1num + k] /= (GeoPar.pixsiz);
			}
		}

		//-----------------------------------------------------------------*
		//...  WE NOW HAVE THE VALUES ALONG ROWS/COLUMNS IN THE IMAGE
		//...  IN THE ROWS OF 'IMG'.  ALL WE HAVE TO DO NOW IS OUTPUT
		//...  THESE VALUES IN THE PROPER ORDER INTO THE ARRAY 'RECON'.
		//-----------------------------------------------------------------*

		// LOOP ON ROWS OF 'IMG' (SHOULD BE 'NELEM' ROWS)

		iii = 0;
		for (n = GeoPar.nelem - 1; n >= 0; n--)
		{

			// LOOP ON LAST 'N' COLUMNS IN EACH ROW [N = (NELEM-1)/2]

			ii = 0;
			for (k = 2 * n3[1] - nnew; k < 2 * n3[1]; k++)
			{
				if (iflag == 0)
				{
					recon[(n) * GeoPar.nelem + ii] += img[(iii) * gl1num + k];
				}
				else
				{
					recon[(GeoPar.nelem - ii - 1) * GeoPar.nelem + n] +=
							img[(iii) * gl1num + k];
				}
				ii++;
			}

			// LOOP ON FIRST 'N+1' COLUMNS IN EACH ROW
			// (USING THE SAME INDEX, 'ii', TO INDEX RECON ARRAY)

			for (k = 0; k < nnew + 1; k++)
			{
				if (iflag == 0)
				{
					recon[(n) * GeoPar.nelem + ii] += img[(iii) * gl1num + k];
				}
				else
				{
					recon[(GeoPar.nelem - ii - 1) * GeoPar.nelem + n] +=
							img[(iii) * gl1num + k];
				}
				ii++;
			}
			iii++;
		}
		delete[] otemp;
		delete[] img;
		delete[] gl;
		delete[] gl1;
	}
	delete[] g;

	//*************************************************C
	//  END OF LOOP ON SETS OF PROJECTION ANGLES
	//*************************************************C

	sum = 0.0;


	for (n = 0; n < GeoPar.area; n++)
	{
		sum += recon[n];
	}
	sum /= (REAL) (GeoPar.nelem * GeoPar.nelem);

	diff = (REAL) fabs(GeoPar.aveden - sum);
	for (n = 0; n < GeoPar.area; n++)
	{
		recon[n] += diff;
	}
	return FALSE;
}
