/*
 ***********************************************************
 $SNARK_Header$
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/nav1.cpp $
 $LastChangedRevision: 153 $
 $Date: 2014-07-27 21:27:06 -0500 (Sun, 27 Jul 2014) $
 $Author: olangthaler $
 ***********************************************************

**************************************************************************************
Non-ascending vector for secondary optimization criterion "total variation" (sec_cri1)
**************************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <iostream>
#include "nav1.h"
#include "math.h"
#include "anglst.h"
#include "blob.h"

void nav1_class::Run(REAL* xkn, REAL* vkn)
{
 int pixelcount = GeoPar.area;
  
 REAL gradientSquareNorm = 0;
 calcGradient(xkn, vkn);
 for(INTEGER i = 0; i < pixelcount; i++)
     gradientSquareNorm += pow(vkn[i], 2);
  
 REAL gradientNorm = sqrt(gradientSquareNorm);
 if(gradientNorm > pow(10,-15))
    for(INTEGER i = 0; i < pixelcount; i++)
        vkn[i] = (-1.0) * vkn[i] / gradientNorm;
 else
    for(INTEGER i = 0; i < pixelcount; i++)
        vkn[i] = 0.0;
}

void nav1_class::calcGradient(REAL* x, REAL* grad)
{
 // read image dimensions
 int nelem = GeoPar.nelem;
 int pixelcount = GeoPar.area;
  
 // define minimum value for divisions
 const double min = pow(10,-15);
  
 // set gradient starting value
 memset(grad,0,pixelcount*sizeof(REAL));
  
 // formula 1
 // (2*x[i]-x[i+1]-x[i+nelem])/sqrt(pow(x[i]-x[i+1],2)+pow(x[i]-x[i+nelem],2))

 // formula 2
 // (x[i]-x[i-1])/sqrt(pow(x[i-1]-x[i],2)+pow(x[i-1]-x[i-1+nelem],2))

 // formula 3
 // (x[i]-x[i-nelem])/sqrt(pow(x[i-nelem]-x[i+1-nelem],2)+pow(x[i-nelem]-x[i],2))
 /*
  __________________________
 |1|       1 & 2         |2|
 |-|---------------------|-|
 | |                     | |
 | |                     | |
 |1|                     | |
 | |                     | |
 |&|      1 & 2 & 3      |2|
 | |                     | |
 |3|                     | |
 | |                     | |
 | |                     | |
 |_|_____________________|_|
 |3|          3          |0|
  -------------------------
 */
 // top left pixel: only formula 1 applies
 double denx = pow(x[0]-x[1],2)+pow(x[0]-x[nelem],2);
 if(denx>min){
    grad[0]=(2*x[0]-x[1]-x[nelem])/sqrt(denx);
   }
  
 double deny;
 // (rest of) top row: formulas 1 and 2 apply
 for(int i=1; i<nelem; i++){
     deny = pow(x[i-1]-x[i],2)+pow(x[i-1]-x[i-1+nelem],2);
      
     if((i+1)==nelem){
        if(deny>min)
           grad[i] = ((x[i]-x[i-1])/sqrt(deny));
       }
     else{ 
        denx = (x[i]-x[i+1])*(x[i]-x[i+1]) + (x[i]-x[i+nelem])*(x[i]-x[i+nelem]);
         
        if(denx>min && deny>min)
           grad[i]=((2*x[i]-x[i+1]-x[i+nelem])/sqrt(denx)) + ((x[i]-x[i-1])/sqrt(deny));
       }
 }
  
 double denz;
 // rest of image except left and right column and bottom row: all 3 formulas apply
 for(int i=nelem; i<pixelcount-nelem; i++){
     if(!(i%nelem)){ // first column: only cases 1 & 3
        denx = pow(x[i]-x[i+1],2) + pow(x[i]-x[i+nelem],2);
        denz = pow(x[i-nelem]-x[i-nelem+1],2) + pow(x[i-nelem]-x[i],2);
        if(denx>min && denz>min){
           grad[i]=((2*x[i]-x[i+1]-x[i+nelem])/sqrt(denx)) + ((x[i]-x[i-nelem])/sqrt(denz));
          }
       }
     else{
        if(!((i+1)%nelem)){ // last column: only case 2 applies
           deny = pow(x[i-1]-x[i],2) + pow(x[i-1]-x[i-1+nelem],2);
           if(deny>min){
              grad[i]=(x[i]-x[i-1])/sqrt(deny);
             }
          }
        else{
           denx = pow(x[i]-x[i+1],2)+pow(x[i]-x[i+nelem],2);
           deny = pow(x[i-1]-x[i],2)+pow(x[i-1]-x[i-1+nelem],2);
           denz = pow(x[i-nelem]-x[i-nelem+1],2)+pow(x[i-nelem]-x[i],2);
           if(denx>min && deny>min && denz>min){
              grad[i]=(2*x[i]-x[i+1]-x[i+nelem])/sqrt(denx) +
                      (x[i]-x[i-1])/sqrt(deny) +
                      (x[i]-x[i-nelem])/sqrt(denz);
             }
          }
       }
    }
 // Bottom row without last pixel: only third case applies
 for(int i=pixelcount-nelem; i<(pixelcount-1); i++){
     denz = pow(x[i-nelem]-x[i-nelem+1],2)+pow(x[i-nelem]-x[i],2);
     // left column: formulas 1 and 3 apply
     if(denz>min){
        grad[i] = ((x[i]-x[i-nelem])/sqrt(denz));
       }
    }

 // failsafe
 for(int i=0;i<(pixelcount-1);i++){
     if(isnan(grad[i]))
        grad[i]=0.0;
    }
}
