/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/prdta.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 PRDTA RETURNS AS ITS VALUE THE RAYSUM OF THE NR-TH RAY IN THE
 NP-TH PROJECTION.
 IT BRINGS INTO CORE THE NRAYS VALUES FOR THE PROJECTION IF THEY
 ARE NOT ALREADY THERE
 */

#include <cstdlib>
#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "anglst.h"
#include "uiod.h"

#include "projfile.h"

REAL anglst_class::prdta(INTEGER np, INTEGER nr)
{
	REAL raysum;

// IF NP IS NOT A VALID PROJECTION  TERMINATE WITH ERROR MESSAGE

	if ((np < 0) || (np >= GeoPar.prjnum))
	{
		// ARGUMENTS OUT OF RANGE
		fprintf(output, "\n **** np = %d is out of range (0..%d)", np, GeoPar.prjnum - 1);
		fprintf(output, "\n **** program aborted\n");
		exit(777);
	}

	raysum = 0.0;

	// TEST IF PROJECTION IS ALREADY IN CORE, SAME AS LAST ONE READ

	if (np != Anglst.incore)
	{

		// GET PROJECTION DATA FROM RANDOM FILE
		Anglst.incore = np;
		ProjFile.ReadProj(np, Anglst.pbase, GeoPar.nrays);
	}

	// IF  NR  IS NOT A VALID RAY NUMBER TERMINATE WITH ERROR MESSAGE

	if ((nr < 0) || (nr >= GeoPar.nrays))
	{
		fprintf(output, "\n **** nr = %d is out of range (0..%d)", nr, GeoPar.nrays - 1);
		fprintf(output, "\n **** program aborted\n");
		exit(777);
	}

	raysum = Anglst.pbase[nr];

	if (trace > 5)
	{
		fprintf(output, "\n          prdta   np = %5i  nr = %5i  raysum = %9.4f", np, nr, raysum);
	}
	return raysum;
}

