/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/projfile.cpp $
 $LastChangedRevision: 118 $
 $Date: 2014-07-09 14:22:29 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdlib>
#include <cstring>
#include <cmath>

#include "uiod.h"

#include "projfile.h"

SnarkPrjFile ProjFile;

SnarkPrjFile::SnarkPrjFile()
{
	AnglesBuff = NULL;
}

INTEGER SnarkPrjFile::Open(const char* FileName)
{

	if (DIGFileSnarkProj::Open(FileName) != 0)
	{
		return -1;
	}

	Angles = 1;
	return 0;
}

INTEGER SnarkPrjFile::ReadHeader(char* name, Noise_class* Noise,
		spctrm_class* Spct, anglst_class* Anglst)
{
	const char* tname;
	double treal1, treal2;
	int tint;
	int i;
	unsigned int tuint;

	//////////////
	// get Name //
	//////////////

	GetTitle(&tname);
	strncpy(name, tname, 80);
	name[80] = 0;

	/////////////////////////////
	// get GeoPar's attributes //
	/////////////////////////////

	GetNoOfProjs(&tuint);
	NumberOfProjections = GeoPar.prjnum = tuint;

	double tDIGSamp;
	GetSampling(&tDIGSamp);
	GeoPar.pinc = tDIGSamp;

	unsigned int tDIGDim;
	GetNoOfRays(&tDIGDim);
	NumberOfRays = GeoPar.usrays = tDIGDim;

	GeometryTypeEnum tGeomType;
	GetGeometryType(&tGeomType);

	DetectorTypeEnum tDetType;
	ProjTypeEnum tProjType;

	if (tGeomType == GT_DIVERGENT)
	{
		GeoPar.div = true;
		GeoPar.par = false;
		GeoPar.strip = false; // bug 207 - swr - 3/3/06
		GeoPar.line = true;

		GetGeometry(&treal1, &treal2, &tDetType);

		if (tDetType == DT_TANGENT)
		{
			GeoPar.tang = true;
			GeoPar.arc = false;
		}
		else
		{
			GeoPar.tang = false;
			GeoPar.arc = true;
		};

		GeoPar.radius = treal1;
		GeoPar.stod = treal2;
	}
	else
	{
		GeoPar.div = false;
		GeoPar.par = true;

		GetGeometry(&tDetType, &tProjType);

		if (tDetType == DT_UNIFORM)
		{
			GeoPar.uni = true;
			GeoPar.vri = false;
		}
		else
		{
			GeoPar.uni = false;
			GeoPar.vri = true;
		};

		if (tProjType == PT_STRIP)
		{
			GeoPar.strip = true;
			GeoPar.line = false;
		}
		else
		{
			GeoPar.strip = false;
			GeoPar.line = true;
		};
	};

	////////////////////////////
	// get Noise's attributes //
	////////////////////////////

	//bug221 start - swr - 03/03/07
	if (GetNoiseQuantum(&treal1, &treal2, &tint) == 0)
	{
		Noise->setQuantumNoise(tint, treal1, treal2);
	}
	else
	{
		Noise->setQuantumNoise(0, 1.0, 1.0);
	}

	if (GetNoiseScatter(&treal1, &treal2) == 0)
	{
		Noise->setScatter(treal1, treal2);
	};

	if (GetNoiseAdditive(&treal1, &treal2) == 0)
	{
		Noise->setAdditiveNoise(treal1, treal2);
	};

	if (GetNoiseMultiplicative(&treal1, &treal2) == 0)
	{
		Noise->setMultiplicativeNoise(treal1, treal2);
	};
	//bug221 end - swr - 03/03/07

	///////////////////////////
	// get Spct's attributes //
	///////////////////////////

	GetNoOfEnergies(&tuint);
	Spct->nergy = tuint;

	for (i = 0; i < tuint; i++)
	{
		GetSpectrum(i, &tint, &treal1, &treal2);

		Spct->energy[i] = tint;
		Spct->engwt[i] = treal1;
		Spct->backgr[i] = treal2;
	};

	///////////////////////////////////
	// calculate GeoPar's attributes //
	///////////////////////////////////

	GeoPar.snrays = GeoPar.numray(GeoPar.nelem, GeoPar.pixsiz);

	if (GeoPar.usrays > GeoPar.snrays)
	{
		GeoPar.nrays = GeoPar.usrays;
	}
	else
	{ // bug194 - swr - 1/6/06
		GeoPar.nrays = GeoPar.snrays;
	}


	GeoPar.midray = GeoPar.nrays / 2;
	GeoPar.fsnray = (GeoPar.nrays - GeoPar.snrays) / 2;
	GeoPar.lsnray = GeoPar.fsnray + GeoPar.snrays - 1;
	GeoPar.fusray = (GeoPar.nrays - GeoPar.usrays) / 2;
	GeoPar.lusray = GeoPar.fusray + GeoPar.usrays - 1;

	//////////////////////////////
	// get Anglst's  attributes //
	//////////////////////////////

	Anglst->Init(GeoPar.prjnum);

	for (i = 0; i < GeoPar.prjnum; i++)
	{

		if (SelectProj(i) != 0)
		{
			return -1;
		}

		if (GetProjAngle(&(Anglst->bth[i])) != 0)
		{
			return -1;
		}

		Anglst->bsin[i] = (REAL) sin(Anglst->bth[i]);
		Anglst->bcos[i] = (REAL) cos(Anglst->bth[i]);
	};

	Anglst->pbase = new REAL[GeoPar.nrays];
	Anglst->incore = -1;

	if (GeoPar.div)
	{
		Anglst->genphi();
	}

	return 0;
}

INTEGER SnarkPrjFile::ReadProj(INTEGER np, REAL* data, unsigned INTEGER nsize)
{
	unsigned int i;
	unsigned int tNum;
	INTEGER tSkip;
	REAL *tArray;

	if (SelectProj(np) != 0)
	{
		return -1;
	}

	GetNoOfRays(&tNum);
	tArray = new REAL[tNum];
	GetProjData(tArray);

	if (nsize <= NumberOfRays)
	{

		// NSIZE IS FEWER THAN OR EQUAL TO USRAYS
		tSkip = (NumberOfRays - nsize) / 2;
		for (i = 0; i < nsize; i++)
		{
			data[i] = tArray[i + tSkip];
		}
	}
	else
	{

		// NSIZE IS GREATER THAN USRAYS
		for (i = 0; i < nsize; i++)
		{
			data[i] = 0.0;
		}

		tSkip = (nsize - NumberOfRays) / 2;

		for (i = 0; i < NumberOfRays; i++)
		{
			data[i + tSkip] = tArray[i];
		}
	};

	delete[] tArray;

	return 0;
}

INTEGER SnarkPrjFile::Open(const char* FileName, INTEGER usrays, INTEGER prjnum)
{
	NumberOfRays = usrays;
	NumberOfProjections = prjnum;

	AnglesBuff = new REAL[prjnum];

	if (DIGFileSnarkProj::Open(FileName, "unknown", usrays, GeoPar.pinc,
			"This projection file has been created by snark14") != 0)
		return -1;

	Angles = 0;

	return 0;
}

INTEGER SnarkPrjFile::WriteHeader(char* name, Geometry_class* Geo,
		Noise_class* Noise, spctrm_class* Spct)
{
	//////////////
	// set Name //
	//////////////

	SetTitle(name);

	/////////////////////////////
	// set GeoPar's attributes //
	/////////////////////////////

	if (Geo->div)
	{
		if (Geo->tang)
		{
			SetGeometry(Geo->radius, Geo->stod, DT_TANGENT);
		}
		else
		{
			SetGeometry(Geo->radius, Geo->stod, DT_ARC);
		}
	};

	if (Geo->par)
	{
		if (Geo->uni)
		{
			if (Geo->strip)
			{
				SetGeometry(DT_UNIFORM, PT_STRIP);
			}
			else
			{
				SetGeometry(DT_UNIFORM, PT_LINE);
			}
		}
		else
		{
			if (Geo->strip)
			{
				SetGeometry(DT_VARIABLE, PT_STRIP);
			}
			else
			{
				SetGeometry(DT_VARIABLE, PT_LINE);
			}
		};
	};

	////////////////////////////
	// set Noise's attributes //
	////////////////////////////

	if (Noise->quanin > 0)
	{
		SetNoiseQuantum(Noise->quanmn, Noise->quancm, Noise->quanin);
	}

	if (Noise->sctnfl == true)
	{
		SetNoiseScatter(Noise->sctnpk, Noise->sctnwd);
	}

	if (Noise->addnfl == true)
	{
		SetNoiseAdditive(Noise->addnmn, Noise->addnsd);
	}

	if (Noise->ultnfl == true)
	{
		SetNoiseMultiplicative(Noise->ultnmn, Noise->ultnsd);
	}

	///////////////////////////
	// set Spct's attributes //
	///////////////////////////

	int i;
	double* treal1 = new double[Spct->nergy];
	double* treal2 = new double[Spct->nergy];

	for (i = 0; i < Spct->nergy; i++)
	{
		treal1[i] = Spct->engwt[i];
		treal2[i] = Spct->backgr[i];
	};

	SetSpectrum(Spct->nergy, Spct->energy, treal1, treal2);

	delete[] treal1;
	delete[] treal2;

	return 0;
}

INTEGER SnarkPrjFile::WriteAngles(REAL* bth)
{

	unsigned int i;


	// if angles already stored report error
	if (Angles == 1)
	{
		return -1;
	}

	// store angles into buffer
	for (i = 0; i < NumberOfProjections; i++)
	{
		AnglesBuff[i] = bth[i];
	};

	Angles = 1;
	return 0;
}

INTEGER SnarkPrjFile::WriteProj(INTEGER np, REAL* iprjec)
{

	if (Angles == 0)
	{
		fprintf(output, "\n **** PRJFile error: projection can not "
				"be stored while angles are not given !!!\n");
		return -1;
	};

	if (AppendProj(np, AnglesBuff[np], "Projection created by snark14", iprjec)
			!= 0)
	{
		return -1;
	}

	return 0;
}

INTEGER SnarkPrjFile::Close()
{

	if (AnglesBuff != NULL)
	{
		delete[] AnglesBuff; // bug 92 - Lajos - 03/02/2005
		AnglesBuff = NULL;
	}

	// closes the file

	if (DIGFileSnarkProj::Close() != 0)
	{
		return 1;
	}

	return 0;

}
