/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                              S N A R K   1 4                              *
 *                                                                           *
 *                     A PICTURE RECONSTRUCTION PROGRAM                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

 quad.cpp,v 1.6 2009/06/01 03:33:26 jklukowska Exp

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **
 *                                                                *
 *      QUADRATIC ITERATIVE RECONSTRUCTION ALGORITHMS             *
 *                                                                *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * **


    PROGRAM BY EHUD ARTZY 1977

     INTRODUCTION
     ============

     THIS PROGRAM IS SET TO MINIMIZE THE FUNCTION

     K(X)=SA*(P-M*X)#*A*(P-M*X)+(X-XBAR)#*(SB*B**L+SC*C**(-1)*(X-XBAR)

     WHERE

     M IS AN MM BY NN MATRIX
     P AN MM-DIMENSIONAL VECTOR
     A IS A NONNEGATIVE DIAGONAL MATRIX OF RANK MM
     B,C ARE POSITIVE DEFINITE SYMETRIC NN BY NN MATRICES
     SA,SB,SC ARE NONNEGATIVE REAL NUMBERS
     XBAR IS AN NN-DIMENSIONAL VECTOR
     L IS A NONNEGATIVE INTEGER
     # STANDS FOR TRANSPOSE

     IF SB = SC = 0 THEN AN ADDITIONAL DIAGONAL MATRIX D OF RANK NN IS
     INTRODUCED WHICH, FOR ALL X WHICH MINIMIZE K(X), WILL CHOOSE THE
     ONE WHICH MINIMIZES THE NORM OF D**(-1)*(X-XBAR)

     IF SB + SC GREATER THAN 0 THEN

        LET

   (1)  Q = C**(T/2)*((SA*M#*A*M +SB*B**L)*C**(T/2) +(SC*C**(T/2 - 1))

        F = C**(T/2)*SA*M#*A*(P - M*XBAR)

        IF Y IS A SOLUTION OF Q*Y = F THEN  X = C**(T/2)*Y + XBAR
        MINIMIZES K(X)

     ELSE LET

   (2)   Q = D*M#*A*M*D

         F = D*M#*A*P

         IF Y IS A SOLUTION OF Q*Y = F THEN X = D*Y IS A
         MINIMUM POINT OF K(X)

     FOR FURTHER DETAILS SEE:
           QUADRATIC OPTIMIZATION FOR IMAGE RECONSTRUCTION.
           GABOR T. HERMAN AND ARNOLD LENT
           COMPUTER GRAPHICS AND IMAGE PROCESSING 5,319-332 (1976)

      THE FOLLOWING ALGORITHM IS USED TO FIND THE SOLUTION OF Q*Y = F

      G(0) = R(0) = Q*Y(0) - F
      Y(K+1) = Y(K) - DELTA(K)*G(K)
      R(K+1) = Q*Y(K+1) - F
      G(K+1) = R(K+1) + GMA(K)*G(K)

      AN ALTERNATIVE WAY TO COMPUTE R(K+1) IS GIVEN BY
      R(K+1) = R(K) - DELTA(K)*Q*G(K)

      THE FORMER IS REFFERED TO AS DIRECT METHOD AND THE LATER INDIRECT
      METHOD

      THE CHOICE OF DELTA(K) AND GMA(K) DETERMINE THE PARTICULAR KIND
      OF ALGORITHM USED. WHEN GMA(K) = 0 FOR EVERY K WE HAVE A ONE STEP
      METHOD (OSM) OTHERWISE WE HAVE TWO STEPS METHOD (TSS)

      FOR USING THE PROGRAM THE USER MUST SPECIFY THE FOLLOWING
      PARAMETERS:

     CARD 1 - AOPT,BOPT,COPT,MINZ,PERIOD,TOLER,DELTA,GMA

      AOPT  : 1 - OSM WITH DIRECT COMPUTATION OF GRADIENT
              2 - OSM WITH INDIRECT COMPUTATION OF GRADIENT
              3 - TSS WITH DIRECT COMPUTATION OF GRADIENT
              4 - TSS WITH INDIRECT COMPUTATION OF GRADIENT

      BOPT  : 0 - THIS IS A STATIONARY METHOD FOR OSM USER SUPPLIES
                  DELTA FOR TSS USER SUPPLIES DELTA AND GMA
              1 - THIS IS A STATIONARY METHOD WITH DELTA COMPUTED BY
                 THE PROGRAM TO BE

                         DELTA = 2.0/(LMD + ESMIN)
                 WHERE
                   LMD AND ESMIN ARE ESTIMATES OF LARGEST AND SMALLEST
                   POSITIVE EIGENVALUES OF THE MATRIX Q.
             -1 -  USER SUPPLY LMD AND ESMIN FOR BOPT 1
              2 -  FOR OSM THIS IS RICHARDSON METHOD OF LENGTH PERIOD
                   FOR TSS THIS IS SEMI ITERATIVE METHOD BASED ON
                   RICHARDSON STATIONARY METHOD. LMD AND ESMIN(SEE 1)
                   ARE COMPUTED BY THE PROGRAM FOR REFERENCE SEE:
                   DAVID M. YOUNG
                   ITERATIVE SOLUTION OF LARGE LINEAR SYSTEMS
                   ACADEMIC PRESS 1971
             -2 -  USER SUPPLY LMD AND ESMIN FOR BOPT 2
              3 -  FOR OSM THIS IS THE STEEPEST DESCENT METHOD WHERE
                   DELTA(K) = R(K)#*R(K)/(R(K)#*Q*R(K))
                   FOR TSS THIS IS THE CONJUGATE GRADIENT METHOD WHERE
                   DELTA(K) = R(K)#*R(K)/(G(K)#*Q*G(K))
                   GMA(K) = R(K)#*R(K)/(R(K-1)#*R(K-1))

      COPT  : 1 -  EQUATION 2 IS USED WITH GOITEIN D MATRIX
                   FOR REFERENCE SEE:
                   M. GOITEIN
                   THREE-DIMENSIONAL DENSITY RECONSTRUCTION FROM A
                   SERIES OF TWO-DIMENSIONAL PROJECTIONS
                   NUCL. INSTR. METHODS 101, 509-518 (1972)
              2 -  EQUATION 2 IS USED WITH LSIRT D MATRIX
                   FOR REFERENCE SEE:
                   A. V. LAKSHMINARAYNAN AND A. LENT
                   THE SIMULTANEOUS ITERATIVE RECONSTRUCTION TECHNIQUE
                   AS LEAST SQUARES METHOD
                   SPIE. VOL. 96 OPTICAL INSTRUMENTATION IN
                   MEDICINE V (1976)
              3 -  THE USER SUPPLY HIS OWN DIAGONAL D MATRIX. THE
                   VALUES OF THE DIAGONAL ELEMENTS ARE COMPUTED BY
                   DSET SQUARED AND STORED IN A VECTOR OF LENGTH
                   AREA (SEE SNARK MANUAL). THE CALLING SEQUENCE
                   FOR DSET IS
                   CT(D,LIST,WEIGHT)
                   WHERE
                   D     -CALCULATED VECTOR
                   LIST  -SEE THE SNARK MANUAL
                   WEIGHT-SEE THE SNARK MANUAL
              4 -  EQUATION 1 IS USED CARD 3-5 MUST BE PRESENTED

      MINZ  : 0 -  FOR BOPT 1 OR 2 ESMIN IS ASSUMED TO BE 0
              1 -  FOR BOPT 1 OR 2 ESMIN IS CALCULATED

      PERIOD: N -  N IS THE LENGTH OF THE PERIOD FOR THE RICHARDSON OSM
                   N MUST BE IN THE RANGE 1-32

      TOLER :   -  THIS IS THE TOLERANCE FOR WHICH LMD AND ESMIN IS
                   COMPUTED FOR REFERENCE SEE:
                   A COMPUTER INPLEMENTATION OF BAYESIAN ANALYSIS
                   OF IMAGE RECONSTRUCTION
                   G. T. HERMAN AND A. LENT
                   INFORMATION AND CONTROL 31,364-384 (1976)

      DELTA :   -  FOR BOPT 0 THIS IS CONSTANT DELTA
                   FOR BOPT -1,-2 THIS IS USER SUPPLY ESMIN

      GMA   :   -  FOR BOPT 0 AND TSS THIS IS CONSTANT GMA
                   FOR BOPT -1,-2 THIS IS USER SUPPLY LMD

     CARD 2 - DOPT,EOPT,FOPT,GOPT,HOPT,POPT,ERROR,MINPRD,INTERP

      DOPT  : 1 -  NATURAL ORDER FOR PICKING DELTA FOR RICHARDSON OSM
                   (AOPT LE 2 BOPT 2)
              2 -  USER SUPPLY ORDER OF PICKING DELTA CARD 6 MUST BE
                   PRESENTED
              3 - LEBEDTV-FINOGENOV ORDERING FOR PICKING DELTA
                  FOR REFERENCE SEE:
                  R.S. ANDERSSEN AND G.H. GOLUB
                  RICHARDSONS NON-STATIONARY MATRIX ITERATIVE PROCEDURE
                  STAN-CS-72-307 (1972)
                  STANFORD UNIVERSITY
      EOPT  : 1 - A IS THE IDENTITY MATRIX TIMES THE CONSTANT ERROR
              2 - THE VALUE OF THE MATRIX A IS COMPUTED BY ESTIMATING
                  THE VARIANCE IN THE MEASUREMENTS. THIS OPTION IS
                  AVAILABLE ONLY FOR NOISY DATA (SEE SNARK MANUAL FOR
                  NOISE ESTIMATION).
              3 - USER SUPPLY AN ERROR ROUTINE. THE CALLING SEQUENCE
                  IS
                    PUNCTION UERROR(NP,NR,PD)
                    WHERE
                    NP - IS A PROJECTION NUMBER
                    NR - IS A RAY NUMBER IN PROJECTION NP
                    PD - IS THE VALUE OF THE (NP,NR) PROJECTION

      FOPT  : 1 - XBAR (EXPECTED VALUE) IS THE ZERO VECTOR
              2 - XBAR IS AVERAGE DENSITY AS APROXIMATE BY SNARK
              3 - XBAR IS NORMALIZE BACK PROJECTION
              4 - XBAR IS CONTINUOUS BACK PROJECTION
              5 - XBAR IS STARTING POINT
      GOPT  : 0 - NO SPLITTING
              1 - DIAGONAL SPLITTING
              2 - USER SUPPLY SPLITTING MATRIX
                    CALL SSET(S,LIST,WEIGHT)
            WHERE
            S IS A DIAGONAL SPLITTING MATRIX
            LIST SEE THE SNARK05 MANUAL
            WEIGHT SEE THE SNARK05 MANUAL

      HOPT  : 1 - RECONSTRUCTED PICTURE IS CORRECTED SO THAT AVERAGE
                  DENSITY WILL BE EQUAL TO THAT OF AVERAGE DENSITY
                  COMPUTED BY SNARK
              0 - NO CORRECTION

      POPT  : 0 - NO REPORTS
              1 - THE FOLLOWING VALUES ARE REPORTED
                  GRADIENT (ONLY FOR AOPT GE 2 OR BOPT 3)
                  NORM OF CORRECTION,DELTA,NORM OF CORRECTING VECTOR
              2 - IN ADDITION COST FUNCTION IF POSSIBLE

      ERROR :   - FOR EOPT 1 THE VALUE OF THE DIAGONAL ELEMENT
      MINPRD:   - IF MULTIPLICATIVE NOISE IS PRESENTED AND EOPT 2
                  A CONSTANT TO USE WHEN MEASURMENT IS ZERO MUST BE
                  SPECIFIED

      INTERP:   - FOR FOPT 4 THIS IS THE INTERPOLATION INDEX
               (SEE THE SNARK05 MANUAL)

      THE FOLLOWING CARD(S) IS(ARE) READ ONLY IF BOPT = 2

            ORDER(I), I = 1,PERIOD
      ORDER :  PERIOD NUMBERS IN THE RANGE 1 TO PERIOD

      THE FOLLOWING 3 CARDS ARE READ ONLY IF COPT GE 4

     CARD 3 - SA,SB,SC

      SA    :   CONSTANT (SEE EQUATION 1)
      SB    :   CONSTANT (SEE EQUATION 1)
      SC    :   CONSTANT (SEE EQUATION 1)

     CARD 4 - L,BW1,BW2,BBCON          *** MATRIX B ***

      L     : POWER OF MATRIX B
      BW1   : VARIANCE OF DENSITY IN AN ARBITRARY PIXEL
      BW2   : COVARIANCE OF DENSITIES IN NEIGHBORING PIXELS
      BW3   : COVARIANCE OF DENSITIES IN DIAGONALY NEIGHBORING PIXELS
      BBCON : KAMI - GENERALIZED KASHYAP-MITTAL SMOOTING (L MUST BE 1)
                     FOR REFERENCE SEE:
                     R.L. KASHYAP AND M.C. MITTLE
                     PICTURE RECONSTRUCTION FROM PROJECTIONS
                     1EEE TRANS. COMPUTERS, C-27,915-923 (1975)
              SAJZ - SMOTH AS IN SNAR WITHOUT NORMALIZATION
              SAJC - SAME AS SAJZ EXCEPT THAT BOARDER PIXELS ASSUME
                     THE SAME DENSITY OUTSIDE THE PICTURE
              SNAR - SMOTH AS IN SNARK(SEE SNARK MANUAL) FOR SNAR
                     B IS NOT A SYMETRIC MATRIX

     CARD 5 - T,CW1,CW2,CW3,CBCON      *** MATRIX C ***

      T     : T/2 IS THE POWER OF MATRIX C IN EQUATION 1
      CW1   : SEE BW1
      CW2   : SEE BW2
      CW3   : SEE BW3
      CBCON : SEE BBCON



***********************************************************************
*/

#include <cstdlib>
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "spctrm.h"
#include "fourie.h"
#include "anglst.h"
#include "bckprj.h"
#include "int2str.h"
#include "modefl.h"
#include "consts.h"
#include "uiod.h"
#include "projfile.h"
#include "infile.h"
#include "errpar.h"
#include "err.h"
#include "blob.h"

#include "quad.h"

INTEGER quad_class::Init()
{

  cg_ = NULL;
  g_ = NULL;
  v_ = NULL;
  w_ = NULL;
  r_ = NULL;
  f_ = NULL;
  fb_ = NULL;
  s_ = NULL;
  d_ = NULL;
  m = NULL;
  n = NULL;

  del = NULL;               //bug 174, wei, 10/2005
  order = NULL;

  return 0;                      //wei, 3/2005
}

INTEGER quad_class::Reset()
{
  if(m != NULL&& m != n) delete[] m;
  if(n != NULL&& n != w_) delete[] n;                 //bug 191, wei, 12/2005
  if(cg_ != NULL&&cg_!=g_) delete[] cg_;
  if(g_ != NULL&&g_!=r_) delete[] g_;
  if(v_ != NULL&&v_!=r_) delete[] v_;
  if(w_ != NULL&&w_!=r_) delete[] w_;
  if(r_ != NULL) delete[] r_;
  if(f_ != NULL) delete[] f_;
  if(fb_ != NULL) delete[] fb_;
  if(s_ != NULL) delete[] s_;
  if(d_ != NULL) delete[] d_;
  if(del != NULL) delete[] del;   //bug 174, wei, 10/2005
  if(order != NULL) delete[] order;

    

  return 0;                    //wei, 3/2005
}

BOOLEAN quad_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{
  static INTEGER minz;

  static REAL delta;
  REAL toler;
  static REAL gma;//set static. hstau see deltac()  below
  //INTEGER popt;  poot is global, why is it declared here? hstau
  INTEGER ir;
  //REAL ratio;
  //INTEGER iratio;
  INTEGER i;

  ///INTEGER* order;
  //REAL error;
  //REAL sa;
  //REAL sb;
  //REAL sc;
  //INTEGER l;
  //INTEGER t;
  //REAL bw1;
  //REAL bw2;
  //REAL bw3;
  //INTEGER bbcon;
  //REAL cw1;
  //REAL cw2;
  //REAL cw3;
  //INTEGER cbcon;
  //REAL xbar;
  ///INTEGER iterx;
  //INTEGER aopt;
  //INTEGER bopt;
  //INTEGER copt;
  //INTEGER period;
  //INTEGER dopt;
  //INTEGER eopt;
  //INTEGER fopt;
  //INTEGER gopt;
  //INTEGER  hopt;
  
  INTEGER k;
  REAL dumy;
  REAL dum = 0.0; // this is dummy variable
  //INTEGER js;
  static REAL esmin; // needed to be set static? hstau
  INTEGER j;

  INTEGER np;
  REAL theta;
  REAL sinth;
  REAL costh;
  REAL spac;


  BOOLEAN xalg8_tmp;
  
/*
  static REAL* cg_;//set static. hstau see deltad()  below
  static REAL* g_;//set static. hstau see deltad()  below
  static REAL* v_;
  static REAL* w_;  //set static. hstau
  static REAL* r_;
  static REAL* f_;  //set static. hstau see resedu() and prpict()  below
  static REAL* fb_;  //for blob case;
  static REAL* s_; //set static. hstau
  static REAL* d_; //set static. hstau                     //moved to class quad,wei,3/2005
*/                               

  static REAL lmd; // needed to be set static? hstau
  ///dimension init(20),dmart(6),let(8)
  INTEGER headng;
  static BOOLEAN alg;//set static. hstau
  BOOLEAN eol;

  /*
  //data let / 1ha,1hb,1hc,1hd,1he,1hf,1hg,1hh /
   static const INTEGER let[] = {// needed to be set static? hstau
     CHAR2INT(' ',' ',' ',' '),   //added to match Fortran indexing .hstau
     CHAR2INT('a',' ',' ',' '),
     CHAR2INT('b',' ',' ',' '),
     CHAR2INT('c',' ',' ',' '),
     CHAR2INT('d',' ',' ',' '),
     CHAR2INT('e',' ',' ',' '),
     CHAR2INT('f',' ',' ',' '),
     CHAR2INT('g',' ',' ',' '),
     CHAR2INT('h',' ',' ',' ')
   };
   */
  const char *let[] = {
    "algorithm", 
    "delta_flag",
    "d_flag",
    "order",
    "error_type",
    "exp_value",
    "split", 
    "normalize",
    "period",
    "toler",
    "print",
    "error",
    "minv",
    "interp"};

  //data dmart /4hgoit,4hein ,4hsirt,4h    ,4huser,4h    /
//  static const INTEGER dmart[] = {// needed to be set static? hstau
//    CHAR2INT(' ',' ',' ',' '),   //added to match Fortran indexing .hstau
//    CHAR2INT('g','o','i','t'),
//    CHAR2INT('e','i','n',' '),
//    CHAR2INT('s','i','r','t'),
//    CHAR2INT(' ',' ',' ',' '),
//    CHAR2INT('u','s','e','r'),
//    CHAR2INT(' ',' ',' ',' ')
//  };

  ///data init / 4hzero,4h vec,4htor ,4h    ,4haver,4hage ,4hdens,
  ///            4hity ,4hback,4h pro,4hject,4hion , 4hcont,4h bac,
  ///            4hk pr,4hojs ,4hinit,4hial ,4hvect,4hor  /

  static const INTEGER init[] = {// needed to be set static? hstau
    CHAR2INT(' ',' ',' ',' '),   //added to match Fortran indexing .hstau
    CHAR2INT('z','e','r','o'),
    CHAR2INT(' ','v','e','c'),
    CHAR2INT('t','o','r',' '),
    CHAR2INT(' ',' ',' ',' '),
    CHAR2INT('a','v','e','r'),
    CHAR2INT('a','g','e',' '),
    CHAR2INT('d','e','n','s'),
    CHAR2INT('i','t','y',' '),
    CHAR2INT('b','a','c','k'),
    CHAR2INT(' ','p','r','o'),
    CHAR2INT('j','e','c','t'),
    CHAR2INT('i','o','n',' '), 
    CHAR2INT('c','o','n','t'),
    CHAR2INT(' ','b','a','c'),
    CHAR2INT('k',' ','p','r'),
    CHAR2INT('o','j','s',' '),
    CHAR2INT('i','n','i','t'),
    CHAR2INT('i','a','l',' '),
    CHAR2INT('v','e','c','t'),
    CHAR2INT('o','r',' ',' ')
  };

  static const INTEGER hbb = CHAR2INT(' ',' ',' ',' ');// needed to be set static? hstau
  static const INTEGER hin = CHAR2INT(' ',' ','i','n');// needed to be set static? corr. move i n to the last two positions hstau

  // modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 16, 2004
  static const INTEGER quad_blurck_codes[4] =
  {
    CHAR2INT('s','n','a','r'),
    CHAR2INT('k','a','m','i'),
    CHAR2INT('s','a','j','z'),
    CHAR2INT('s','a','j','c')
  };


  INTEGER area;


  iterx = iter;

  
  if(Blob.pix_basis) {
    area = GeoPar.area;
  }
  else {
    area = Blob.area;
  }
    
 ///tracer
  //   for(i = 0; i < area; i++) {
  //  fprintf(output,"\nrecon=%f", recon[i]);
  //}
      ////tracer
 
// THE FOLLOWING SECTION IS EXECUTED ONLY AT FIRST ITERATION

  if(!(iter != 1)) {

    // INITILIZE DUMMY VARIABLE

    xalg8_tmp = FALSE;
    alg = FALSE;
    ///  d = 1;
    ///  f = 1;
    ///  s = 1;

    // READ INPUT

    ///aopt = getint(input, &eol);
    ///bopt = getint(NULL, &eol);
    ///copt = getint(NULL, &eol);
    ///minz = getint(NULL, &eol);
    ///period = getint(NULL, &eol);
    ///toler = getnum(NULL, &eol);
    ///delta = getnum(NULL, &eol);
    ///gma = getnum(NULL, &eol);
    ///dopt = getint(input, &eol);
    ///eopt = getint(NULL, &eol);
    ///fopt = getint(NULL, &eol);
    ///gopt = getint(NULL, &eol);
    ///hopt = getint(NULL, &eol);
    ///popt = getint(NULL, &eol);
    ///error_1 = getnum(NULL, &eol);
    ///Err.minprd = getnum(NULL, &eol);
    ///Fourie.interp = getint(NULL, &eol);

    aopt = InFile.getint(TRUE, &eol);
    bopt = InFile.getint(FALSE, &eol);
    copt = InFile.getint(FALSE, &eol);
    minz = InFile.getint(FALSE, &eol);
    period = InFile.getint(FALSE, &eol);
    toler = InFile.getnum(FALSE, &eol);
    delta = InFile.getnum(FALSE, &eol);
    gma = InFile.getnum(FALSE, &eol);

    if(eol) {
      fprintf(output, "\n         *** insufficient data item - end of line encountered ***");
      xalg8_tmp = TRUE;
      return xalg8_tmp;
    }

    dopt = InFile.getint(TRUE, &eol);
    eopt = InFile.getint(FALSE, &eol);
    fopt = InFile.getint(FALSE, &eol);
    gopt = InFile.getint(FALSE, &eol);
    hopt = InFile.getint(FALSE, &eol);
    popt = InFile.getint(FALSE, &eol);
    error_1 = InFile.getnum(FALSE, &eol);
    Err.minprd = InFile.getnum(FALSE, &eol);
    Fourie.interp = InFile.getint(FALSE, &eol);
    //tracer
    //fprintf(output,"dopt=%d eopt=%d fopt=%d  gopt=%d hopt=%d popt=%d error=%f minv=%f interp=%d", dopt, eopt, fopt, gopt, hopt, popt, error_1, Err.minprd, Fourie.interp);  
    //tracer
    if(eol) {
      fprintf(output, "\n         *** insufficient data item - end of line encountered ***");
      xalg8_tmp = TRUE;
      return xalg8_tmp;
    }

    // CHECK RANGE OF OPTIONS

    // Add a general checking regardless whether the parameter will be used or not,
    // reflecting the table summrizing all the restrictions on the QUAD variables in the manual
    // bug 191, wei, 12/2006
    
    if(!((aopt>=1 ) &&(aopt<=4)))  
    	{ fprintf(output, "\n         *** option algorithm is out of range ***");
      xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((bopt>= -2) &&(bopt<=3)))  
    	{fprintf(output, "\n         *** option delta-flga is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((copt>= 1) &&(copt<=4)))  
    	{fprintf(output, "\n         *** option d-flag is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!(period>=1 ))                     
    	{fprintf(output, "\n         *** option period is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!(toler> Consts.zero))            
    	{fprintf(output, "\n         *** option toler is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((dopt>=1 ) &&(dopt<=3)))  
    	{fprintf(output, "\n         *** option order is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((eopt>=1 ) &&(eopt<=3)))  
    	{fprintf(output, "\n         *** option error-type is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((fopt>= 1) &&(fopt<=5)))  
    	{fprintf(output, "\n         *** option exp-valueis out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((gopt>=0 ) &&(gopt<=2)))  
    	{fprintf(output, "\n         *** option split is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((hopt>=0 ) &&(hopt<=1)))  
    	{fprintf(output, "\n         *** option normalize is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((popt>=0 ) &&(popt<=2)))  
    	{fprintf(output, "\n         *** option print is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!(error_1> Consts.zero ))       
    	{fprintf(output, "\n         *** option error is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!(Err.minprd> Consts.zero ))  
    	{fprintf(output, "\n         *** option minv is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
    if(!((Fourie.interp>= -1) &&(Fourie.interp<=6)))  
    	{fprintf(output, "\n         *** option interp is out of range ***");
          xalg8_tmp = TRUE;
      return xalg8_tmp;}
      
        
    ir = 0;
    if(((aopt == 3) || (aopt == 4)) && ((bopt == 1) || (bopt == -1))) ir = 3;   // corr. and bug in Fortran hstau
    if((aopt < 1) || (aopt > 4)) ir = 1;
    if((bopt < -2) || (bopt > 3)) ir += 2;  //bug in Fortran hstau
    if((copt < 1) || (copt > 4)) ir += 4;

    if(!((aopt > 2) || (abs(bopt))  !=  2)) {

      if((dopt < 1) || (dopt > 3)) ir += 8;
    }

    if((eopt < 1) || (eopt > 3)) ir += 16;
    if(copt  >  3)  {

      if((fopt < 1) || (fopt > 5)) ir += 32;//bug in Fortran hstau
    }
    if((gopt < 0) || (gopt > 2)) ir += 64;
    if((hopt < 0) || (hopt > 1)) ir += 128;
    if(period < 1)               ir += 256;  // added hstau
    if(toler < Consts.zero)      ir += 512;  // added hstau
    if( popt < 0  ||  popt > 2 ) ir += 1024;  // added hstau
    if(error_1 < Consts.zero)    ir += 2048;  // added hstau
    
    if(eopt == 2) {
      if(Err.minprd < Consts.zero) ir += 4096;  // added hstau
    }
    
    if(fopt == 4) {
      if(Fourie.interp < -1 ||  Fourie.interp > 6 ) ir += 8192; // added hstau
    }
   
    if(aopt > 2 && abs(bopt) == 2) {  //eleminate this quad algorithm from the old manual. hstau 1/04
      fprintf(output, "\n\n      ***delta_flag cannot be 2 or -2 for algorithm = 3 or 4***");
      xalg8_tmp = TRUE;
      return xalg8_tmp;
    }

      // OPTION OUT OF RANGE
    if(ir > 0) {
      for(i = 0; i < 14; i++) {
        if(ir != ((ir/2)*2)) {
          //fprintf(output, "\n\n      *** %c opt out of range, program stops***", let[i]);
          fprintf(output, "\n\n      ***option %s out of range, program stops***", let[i]); //hstau
        }

        ir = ir/2;
      }
      xalg8_tmp = TRUE;
      return xalg8_tmp;

    }

    // PRINT HEADING

    if((aopt >= 3) && (bopt == 3)) {
      fprintf(output, "\n           the conjugate gradient algorithm");
    }

    if((aopt <= 2) && (bopt == 3)) {
      fprintf(output, "\n           the steepest descent algorithm");
    }

    if((aopt >= 3) && (abs(bopt) == 2)) {
      fprintf(output, "\n           richardson semi iterative method");
    }

    if((aopt <= 2) && (abs(bopt) == 2)) {
      fprintf(output, "\n           richardson one step method");
    }

    if((aopt >= 3) && (abs(bopt)  <=  1)) {
      fprintf(output, "\n           a stationary two steps method");
    }

    if((aopt <= 2) && (abs(bopt)  <=  1)) {
      fprintf(output, "\n           a stationary one steps method");
    }

    headng = hbb;
    if(aopt == (aopt/2)*2) headng = hin;
    fprintf(output, "\n\n            %2sdirect gradient computation", int2str(headng)); //corr. reduce space betw %2s and direct hstau
    fprintf(output, "\n          =================================");

    if(gopt == 1) {
      fprintf(output, "\n\n              diagonal scaling is used");
    }

    if(gopt == 2) {
      fprintf(output, "\n\n              with user supplied scalling");
    }
  
    //bug 154 this checking is ignored hstau. 8 2005
    /*
    // CHECK RATIO OF PIXSIZ AND RINC (APPLY ONLY FOR STRIP MODE) affect blobs? hstau

    ratio = GeoPar.pixsiz/GeoPar.pinc;
    iratio = (INTEGER)(ratio + 0.5);
    if((GeoPar.strip) && ((iratio < 1) || (fabs(iratio-ratio) >= 0.05))) {
      fprintf(output, "\n\n      ***warning pinc is not an integer multiplication of pixsiz");
    }
    */
  
    // READ USER SUPLIED ORDER IF CALLED FOR

    if(!((aopt > 2) || (abs(bopt) != 2))) {
      order = new INTEGER[period];

      if(dopt == 2) {

        order[0] = InFile.getint(TRUE, &eol);

      if(eol || order[0] < 1 || order[0] > period){
        fprintf(output, "\n\n       insufficient data item, user specified order required \n   or illegal order number, order should be integer in [1, period]");
        xalg8_tmp = TRUE;
        return xalg8_tmp;}

        for(i = 1; i < period; i++) {   //corr.hstau
          //alg = eol;
          ///if(alg) order[i] = getint(input, &eol);
          //if(alg) order[i] = InFile.getint(TRUE, &eol);
          ///if(!alg) order[i] = getint(NULL, &eol);
          //if(!alg) order[i] = InFile.getint(FALSE, &eol);
           order[i] = InFile.getint(FALSE, &eol);

      if(eol || order[0] < 1 || order[0] > period){
        fprintf(output, "\n\n       insufficient data item, user specified order required \n   or illegal order number, order should be integer in [1, period] \n");
        xalg8_tmp = TRUE;
        return xalg8_tmp;}                                //wei, bug 174, 10/2005
        }
        //alg = FALSE;
      }
    }

    // FOR  COPT .GT. 3 READ NEXT CARDS

    if(copt <= 3) {

      // SET UNUSED VARIABLES

      sa = 1.0;
      sb = 0.0;
      sc = 0.0;
      l = 0;
      t = 0;
    }
    else {
      ///sa = getnum(input, &eol);
      ///sb = getnum(NULL, &eol);
      ///sc = getnum(NULL, &eol);
      ///l = getint(input, &eol);
      ///bw1 = getnum(NULL, &eol);
      ///bw2 = getnum(NULL, &eol);
      ///bw3 = getnum(NULL, &eol);
      ///bbcon = getwrd(NULL, &eol);
      ///t = getint(input, &eol);
      ///cw1 = getnum(NULL, &eol);
      ///cw2 = getnum(NULL, &eol);
      ///cw3 = getnum(NULL, &eol);
      ///cbcon = getwrd(NULL, &eol);

      sa = InFile.getnum(TRUE, &eol);
      sb = InFile.getnum(FALSE, &eol);
      sc = InFile.getnum(FALSE, &eol);

      if(eol){
        fprintf(output, "\n\n       insufficient data item, a, b, c missing, check the manual \n");
        xalg8_tmp = TRUE;
        return xalg8_tmp;}

      l = InFile.getint(TRUE, &eol);   
      bw1 = InFile.getnum(FALSE, &eol);
      bw2 = InFile.getnum(FALSE, &eol);
      bw3 = InFile.getnum(FALSE, &eol);
      bbcon = InFile.getwrd(FALSE, &eol, quad_blurck_codes, 4);

     if(eol){
        fprintf(output, "\n\n       insufficient data item, matrix B missing, check the manual \n");
        xalg8_tmp = TRUE;
        return xalg8_tmp;}
        
      t = InFile.getint(TRUE, &eol);
      cw1 = InFile.getnum(FALSE, &eol);
      cw2 = InFile.getnum(FALSE, &eol);
      cw3 = InFile.getnum(FALSE, &eol);
      cbcon = InFile.getwrd(FALSE, &eol, quad_blurck_codes, 4);

      if(eol){
        fprintf(output, "\n\n       insufficient data item \n, matrix C missing, check the manual \n");
        xalg8_tmp = TRUE;
        return xalg8_tmp;}
        
      //tracer
      //      fprintf(output,"t=%d", t);
      //tracer
    
      // CHECK INPUT

      if(t == 0) copt = 5;  // should it not be a 4?? hstau
      fprintf(output, "\n\n      the weights are: sa= %8.4f sb= %8.4f sc= %8.4f", sa, sb, sc);

      if(!((sa >= 0.0) && (sb >= 0.0) && (sc >= 0.0))) { // there was a bug in this line (see Fortran version). hstau 6/2/2003
        fprintf(output, "\n\n      ***negative weight, program stops***");

        xalg8_tmp = TRUE;
      }

      if(!((sb == 0.0) || (l == 0))) {
        fprintf(output, "\n\n      matrix b is raised to the power %4i", l);


        if(l <= 0) {
          fprintf(output, "\n\n      ***negative power, program stops***");
          xalg8_tmp = TRUE;
        }
        blurck(bw1, bw2, bw3, bbcon, l, &alg); 
       
        xalg8_tmp = alg;
        if((Blob.pix_basis && GeoPar.nelem < 3) || (!Blob.pix_basis && Blob.H < 5) ) {
          fprintf(output, "\n\n      ***nelem (or the number of blobs in one direction) must be greater than 2 (4) for smoothing");
          fprintf(output, "\n      program stops***");
          xalg8_tmp = TRUE;
        }
      }
      k = t;
      if(copt != 5) {

        fprintf(output, "\n\n      t value for the matrix c is %4i", t);

        if(t <= 0) {
          //fprintf(output, "\n\n      ***t value %4i is not allowed*** program stops", t);
          fprintf(output, "\n\n      ***t value %4i is not allowed, program stops***", t); //hstau

          xalg8_tmp = TRUE;
        }
        else {
          if(!((aopt == 1) && (bopt  <=  2) && (gopt == 0))) {

            // SET T ACORDING TO METHOD

            t = k / 2;
            if(k != t*2) {
              //fprintf(output, "\n\n      ***with either aopt .gt. 1 .or. bopt .gt. 2 t .or. splitting must be even, program stops***");  //Fortran code like output?? hstau
              fprintf(output, "\n\n      ***with either aopt > 1 or bopt > 2, t or splitting must be even, program stops***");  //Fortran code like output?? hstau

              xalg8_tmp = TRUE;
              goto L7;
            }
          }
          blurck(cw1, cw2, cw3, cbcon, t, &alg);
          xalg8_tmp = alg;
          if((Blob.pix_basis && GeoPar.nelem < 3) || (!Blob.pix_basis && Blob.H < 5) ) {
            fprintf(output, "\n\n      ***nelem (or the number of blobs in one direction) must be greater than 2 (4) for smoothing");
            fprintf(output, "\n      program stops***");

            xalg8_tmp = TRUE;
          }
        }
      }
    }
    // ERROR CHECK

L7:
    if((eopt - 2) <= 0) {
      if((eopt - 2) < 0) {

       //bug 154 this report is removed hstau. 8 2005
	/*
        fprintf(output, "\n\n      error is identity matrix times the constant %9.4f", error_1);
	*/
      
        if(fabs(error_1) > Consts.zero) goto L11;
        fprintf(output, "\n\n      ***abs(error) must be greater than ZERO*** program stops");

        xalg8_tmp = TRUE;
        goto L11;
      }
      else {
        fprintf(output, "\n\n      error is calculated by program");

        fprintf(output, "\n\n      minimum measurement is %9.4f", Err.minprd);

        if(Err.minprd <= Consts.zero) {
          xalg8_tmp = TRUE;
          //fprintf(output, "\n\n      *** min variance is less than zero, program stops***");
          fprintf(output, "\n\n      *** minv is less than ZERO, program stops***");  //corr. hstau

          goto L12;
        }
        errpar(alg);
         if(Err.errtyp == 0) error_1 = Err.minprd;
         if(Err.errtyp == 0) eopt = 1;  //suspicious logic. hstau
        xalg8_tmp = alg;
        
        goto L11;
      }
    }

    fprintf(output, "\n\n      user error routine");

    dumy = uerror(1, 1, dum, &alg);
    xalg8_tmp = alg;


        
L11:
    if(!xalg8_tmp) { 
      if(copt <= 3) {
      
	 //bug 154 this report is removed hstau. 8 2005

	/*
        fprintf(output, "\n\n      second criterion matrix is %4s", int2str(dmart[2*copt-1])); 
        //fprintf(output, "%4s matrix", int2str(dmart[2*copt-1]));
        fprintf(output, "%4s matrix", int2str(dmart[2*copt])); //corr.hstau
	*/

        d_ = new REAL[area];
      
        if(copt  <=  2) deset(d_, list, weight, &alg);  // check this.hstau ok
        if(copt == 3) dset(d_, list, weight, &alg);
        xalg8_tmp = alg; 
         
        if(Blob.pix_basis) {
          prpict(GeoPar.nelem, d_, 1, "dset", "d");
        }
        else {
          bprpict(Blob.M, Blob.H, d_, 1, "dset", "d");
        }
      }
      // SPLIT IF CALLED FOR

      if(!xalg8_tmp) {
        if(gopt > 0) {
          s_ = new REAL[area];
        }
        if(gopt == 1) seset(s_, list, weight, &alg);
        if(gopt == 2) sset(s_, list, weight, &alg);
        xalg8_tmp = alg;
        if(gopt > 0) {
          if(Blob.pix_basis) {
            prpict(GeoPar.nelem, s_, 1, "set ", "s");
          }
          else {
            bprpict(Blob.M, Blob.H, s_, 1, "set", "s");
          }
        }
        // FOR COPT .LE. 3 AND SPLITTING COMBINE THE VECTORS
      
        if(!((copt > 3) || (gopt == 0))) {
          //js = s - 1;
          for(i = 0; i < area; i++) {
            d_[i] *= s_[i];
          }
          delete[] s_;  // bug 92 - Lajos - 03/02/2005
          s_ = NULL;    //bug 174, wei, 10/2005
          ///  s = 1; !!!              //check this.hstau
        }
      }
    }
L12:

    if(bopt != 3) {
      if((bopt == 0) && (aopt >= 3)) {
        fprintf(output, "\n\n      user supplied delta is %9.4f", delta);
        fprintf(output, "\n      user supplied gma   is %9.4f", gma);
      }
      if((bopt == 0) && (aopt <= 2)) {
        fprintf(output, "\n\n      user supplied delta is %9.4f", delta);
      }
      if(bopt != 0) {
        if(bopt <= 0) {
          //fprintf(output, "\n\n      user supplied smallest eigenvalue %14.6e", delta); 
          fprintf(output, "\n\n      user supplied smallest eigenvalue %14.6e", gma);  // bug in Fortran .hstau
          //fprintf(output, "\n      user supplied largest eigenvalue  %14.6e", gma);
          fprintf(output, "\n      user supplied largest eigenvalue  %14.6e", delta);   //bug in Fortran .hstau
          if((delta+gma) <= Consts.zero) {
            fprintf(output, "\n\n      ***largest eigenvalue must be positive number");
            fprintf(output, "\n      greater than smallest eigenvalue, program stops***");

            xalg8_tmp = TRUE;
            goto L13;
          }

          //esmin = delta;
          //lmd = gma;
          esmin = gma; // bug in Fortran.hstau
          lmd = delta;  // bug in Fortran. hstau
          bopt = - bopt;
        }
        else {
          if(minz == 0) {
            fprintf(output, "\n\n      smallest eigenvalue is set to 0");
          }
          fprintf(output, "\n\n      eigenvalue is calculated to tolerance %9.4f", toler);

          if(toler <= Consts.zero) {
            fprintf(output, "\n\n      ***tolerance too small, program stops***");

            xalg8_tmp = TRUE;
            goto L13;
          }
	
          if(xalg8_tmp) {

            goto L13;
          }

          r_ = new REAL[area];
          //eigval(r, (minz == 0), d, list, weight, toler, esmin, lmd, s);
          eigval(r_, (minz == 0), d_, list, weight, toler, &esmin, &lmd, s_); //corr. hstau
          delete[] r_;  // bug 92 - Lajos - 03/02/2005
          r_ = NULL; //bug 147, wei, 2005/10

          if((esmin+lmd) <= Consts.zero) {
            xalg8_tmp = TRUE;
            fprintf(output, "\n\n      ***largest eigenvalue must be positive number");
            fprintf(output, "\n\n      greater than smallest eigenvalue, program stops***");

            goto L13;
          }
        }   
 //tracer
	//	fprintf(output,"\n quad: bopt=%d esmin=%f lmd=%f ",bopt,esmin, lmd);
        
      //tracer

        if(bopt == 1) delta = ((REAL) 2.0) / (esmin + lmd);
        if(bopt == 1) {
          fprintf(output, "\n\n      delta computed by program is %14.4e", delta);
        }
        if((bopt == 2) && (aopt  <=  2)) {
          fprintf(output, "\n\n      the period for richardson method is %2i", period);
        } 
        if(bopt == 2) prsemi(esmin, lmd, &alg, &delta);//affected by blobs? hstau. no
        xalg8_tmp = alg;
      
      }
    }

L13:
    j = 4*(fopt - 1);
    if(copt > 3) {
      fprintf(output, "\n\n      expected value is %4s", int2str(init[j+1]));
      fprintf(output, "%4s", int2str(init[j+2]));  // corr. hstau
      fprintf(output, "%4s", int2str(init[j+3]));
      fprintf(output, "%4s", int2str(init[j+4]));
    }

    if(Modefl.lofl) {
      fprintf(output, "\n\n      lower value of pixels is set to %9.4f", Modefl.lower);
    }
    if(Modefl.upfl) {
      fprintf(output, "\n\n      upper value of pixels is set to %9.4f", Modefl.upper);
    }
    if(hopt >= 1) {
      fprintf(output, "\n\n      average density is corrected after each iteration");
    }

    // RETURN IF THERE ARE ERRORS IN INPUT

    if(xalg8_tmp) {


      return xalg8_tmp;
    }

    // INITILIZE EXPECTED VECTOR                   

    xbar = 0.0;
    if(fopt == 2) xbar = GeoPar.aveden;
    if(fopt > 2) {
      f_ = new REAL[GeoPar.area];   // f always in pixel basis. hstau
      if(fopt == 4) {
        if(GeoPar.div) {
          fprintf(output, "\n\n      warning   continuous back projection is not intended for divergent data");
        }
        Fourie.rinc = GeoPar.pinc;
        r_ = new REAL[GeoPar.nrays];

        for(i = 0; i < GeoPar.area; i++) {
          f_[i] = 0.0;
        }
      
        for(np = 0; np < GeoPar.prjnum; np++) {
          ProjFile.ReadProj(np, r_, GeoPar.nrays);
          Anglst.getang(np, &theta, &sinth, &costh);
          if(GeoPar.vri) Fourie.rinc = (REAL) (GeoPar.pinc * MAX0(fabs(costh), fabs(sinth)));
          spac = GeoPar.pixsiz/Fourie.rinc;
          bckprj(f_, GeoPar.nelem, r_, GeoPar.nrays, sinth, costh, spac, Fourie.interp);
        }

        delete[] r_;  // bug 92 - Lajos - 03/02/2005
        r_ = NULL;//bug 147, wei, 2005/10
        
        for(i = 0; i < GeoPar.area; i++) {
          f_[i] /= ((REAL)(GeoPar.prjnum));
        }
      }
      else {
        if(fopt == 3) back(f_, list, weight);
        if(fopt == 5) { 
           if(Blob.pix_basis) { 
              for(i = 0; i < GeoPar.area; i++) { 
                 f_[i] = recon[i];   
	      }
	   }
           else {
              fb_ = new REAL[Blob.area];
              for(i = 0; i < Blob.area; i++) { 
                 fb_[i] = recon[i];   
	      }
	   }
	}
      }
    }

    if(!Blob.pix_basis && fopt > 2 && fopt !=5) {  
       fb_ = new REAL[Blob.area];
       //tracer
       //fprintf(output,"\nf=%f %f %f area=%d", f[0],f[1],f[2],Blob.area);fflush(output);
       //tracer
       Blob.pix2blob(f_,fb_);
       delete[] f_;  // bug 92 - Lajos - 03/02/2005
       f_ = NULL; //bug 147, wei, 2005/10
       
       //f = fb;
    }
  
    // ALLOCATE AREA

    r_ = new REAL[area];
    g_ = r_;
    v_ = r_;
    w_ = r_;
    if((aopt == 2) || (aopt == 4)) {
      v_ = new REAL[area];
    }
    if(aopt >= 3) {
      g_ = new REAL[area];
    }
  
    cg_ = g_;
//tracer
    //fprintf(output,"\n quad: cg=%f",cg[0]);
	///tracer
  
    if((sc != 0.0) || ((sb !=  0.0) && (l > 0))) {
      w_ = new REAL[area];
    }


    if((((copt !=  5) || (gopt > 0))  &&   // corr. why so many () hstau
       ((aopt !=  1) || (bopt > 2))) ||
       ((aopt == 1) && (bopt > 2))
       ) {

      cg_ = new REAL[area];
    }
    // FIRST TIME AROUND CALL RESEDU FOR ALL METHODS

  }  // if iter == 1
  else {

  // FLOW CONTROL


  // AOPT EQUAL 2 OR AOPT EQUAL 4

    if(aopt == (aopt/2)*2) {
      nextr(r_, v_, delta, area);        
      if(Blob.pix_basis) {  
        prpict(GeoPar.nelem, r_, iter, "next", "r");
      }
      else {
        bprpict(Blob.M, Blob.H, r_, iter, "next", "r");
      }
      goto L22;
    }
  }

  // AOPT EQUAL 1 OR AOPT EQUAL 3

  if(Blob.pix_basis) {  
    resedu(recon, r_, w_, d_, list, weight, f_, s_); 
//tracer
    //fprintf(output,"\n quad2: r=%f, cg=%f",r[0],cg[0]);
	///tracer
   
    prpict(GeoPar.nelem, r_, iter, "rese", "r");
  }
  else {
    resedu(recon, r_, w_, d_, list, weight, fb_, s_);  
  
    bprpict(Blob.M, Blob.H, r_, iter, "rese", "r");
  }

  // FOR BOPT = 2 GET DELTA AND GMA

L22:
  if(bopt == 2) semi(&delta, &gma, &alg);
      
  xalg8_tmp = alg;
  if(xalg8_tmp) return xalg8_tmp;
  
  // FOR CONSTANT DELTA AND AOPT EQUAL 1 CALCULATE NEXT PICTURE

  if(!((bopt  <=  2) && (aopt == 1))) {

    // SECOND CRITERIA

    if(copt <= 3) {

      //deltad(r, g, v, cg, d, delta, gma, list, weight, &alg);
      deltad(r_, g_, v_, cg_, d_, &delta, &gma, list, weight, &alg); //corr.hstau

      if(Blob.pix_basis) {  
        prpict(GeoPar.nelem, r_, iter, "deld", "r");
        prpict(GeoPar.nelem, cg_,iter, "deld", "cg");
      }
      else {
        bprpict(Blob.M, Blob.H, r_, iter, "deld", "r");
        bprpict(Blob.M, Blob.H, cg_,iter, "deld", "cg");
      }   
      xalg8_tmp = alg;
      if(alg) return xalg8_tmp;
    }
    else {

      //deltac(r, g, v, cg, w, delta, gma, list, weight, &alg, s);
      deltac(r_, g_, v_, cg_, w_, &delta, &gma, list, weight, &alg, s_);// corr.hstau


      if(Blob.pix_basis) {  
        prpict(GeoPar.nelem, r_, iter, "delc", "r");
        prpict(GeoPar.nelem, g_, iter, "delc", "g");
        prpict(GeoPar.nelem, v_, iter, "delc", "v");
        prpict(GeoPar.nelem, cg_, iter, "delc", "cg");
      }
      else {
        bprpict(Blob.M, Blob.H, r_, iter, "delc", "r");
        bprpict(Blob.M, Blob.H, g_, iter, "delc", "g");
        bprpict(Blob.M, Blob.H, v_, iter, "delc", "v");
        bprpict(Blob.M, Blob.H, cg_, iter, "delc", "cg");
      }

      xalg8_tmp = alg;
      if(alg) return xalg8_tmp;
    }
  }

  // COMPUTE NEXT PICTURE

  nxtpct(recon, cg_, delta);

  // IF POPT .EQ. 2 AND POSSIBLE COMPUTE THE COST FUNCTION

  if(popt >= 2) {
    if(Blob.pix_basis) {  
      costfn(recon, r_, g_, cg_, w_, list, weight, f_);
    }
    else {
      costfn(recon, r_, g_, cg_, w_, list, weight, fb_);
    }
  }
  return xalg8_tmp;
}

