/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/rdpict.cpp $
 $LastChangedRevision: 91 $
 $Date: 2014-07-02 17:34:27 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 READ A TEST PICTURE FROM FILE 'FILE11' AND CONVERT TO SNARK
 INTERNAL REPRESENTATION
 */

#include <cstdlib>       
#include <cstdio>

#include "blkdta.h"
#include "creacm.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"

#include "creaer.h"
#include "recfile.h"

#include "file11.h"
#include "infile.h"

#include "rdpict.h"

//CHAR pictf[20];

void rdpict()
{
	static BOOLEAN rdpict_called = false;

	BOOLEAN eol;

	INTEGER tword;
	static const INTEGER pict_codes[2] =
	{ CHAR2INT('r', 'e', 'c', 'o'), CHAR2INT('t', 'e', 's', 't') };

	// TEST FOR MULTIPLE CALLS AND ISSUE WARNING
	if (rdpict_called)
		creaer(1);

	rdpict_called = TRUE;
	Creacm.test = NULL;

	// DETERMINE WHETHER PICTURE TEST OR RECON.

	tword = InFile.getwrd(FALSE, &eol, pict_codes, 2);

	if (eol)
	{
		fprintf(output, "\n **** TEST or RECONSTRUCTION must be specified");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	};

	if (tword == pict_codes[0])
	{
		// FOR PICT RECON GET NELEM, PIXSIZ, WRITE ON RECFIL QUIT
		GeoPar.nelem = InFile.getint(FALSE, &eol);
		// The following line added for SNARK89 Version 1.01 7/13/90
		if ((GeoPar.nelem <= 0) || (GeoPar.nelem / 2 * 2 == GeoPar.nelem))
			creaer(9);
		GeoPar.area = GeoPar.nelem * GeoPar.nelem;
		GeoPar.midpix = (GeoPar.nelem + 1) / 2;
		GeoPar.pixsiz = InFile.getnum(FALSE, &eol);
		// The following line added for SNARK89 Version 1.01 7/13/90
		if (GeoPar.pixsiz < Consts.zero)
			creaer(10);

		// OPEN RECONSTRUCTION FILE FOR PICTURE RECONSTRUCTION CASE

		if (RecFile.Open("recfil", GeoPar.nelem, GeoPar.pixsiz) != 0)
		{
			fprintf(output, "\n **** unable to open recfil");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}
	}
	else
	{
		// READ PICTURE HEADER INFORMATION

		if (File11.Open(FALSE) != 0)
		{
			fprintf(output, "\n **** unable to open file11 for reading");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		};
		File11.isOpen = TRUE;

		File11.CreaInPicture();

		if (Creacm.erflag)
			exit(444);

		// WRITE HEADER ON RECFIL
		// OPEN RECONSTRUCTION FILE FOR PICTURE TEST CASE

		if (RecFile.Open("recfil", GeoPar.nelem, GeoPar.pixsiz) != 0)
		{
			fprintf(output, "\n **** unable to open recfil");
			fprintf(output, "\n **** program aborted\n");
			exit(-1);
		}

		// ALLOCATE SPACE FOR THE PICTURE
		Creacm.test = new REAL[GeoPar.area];

		// READ IN THE TEST PICTURE INTO THE WORK AREA

		File11.ReadPicture(Creacm.test, GeoPar.nelem);

		// WRITE TEST PICTURE ON RECFIL
		RecFile.WritePhan(Creacm.name, Creacm.test);

		fprintf(output, "\n         test picture read");
		fprintf(output, "\n         %s", Creacm.name);
	}

	return;
}
