/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/read_eval_phantom1.c $
 $LastChangedRevision: 92 $
 $Date: 2014-07-02 17:46:48 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

/* ------------------------- read_eval_phantom1.c -------------------------

 This function reads from the eval file the abnormality index for all
 the structures in the phantom.

 INPUTS: None.

 OUTPUTS:
 numstr - number of structures in phantom.
 phantom - array of length numstr containing the average pixel values (i.e the
 abnormality index) for each structure in the phantom.
 regions - array of length numstr containing the structure numbers
 strarea - array of length numstr containing the area (i.e number of pixels)
 for each structure.
 */

#include <malloc.h>
#include "experimenter.h"
#include "errorc.h"
#include "read_eval_phantom1.h"

void read_eval_phantom1(int** regions, int* numstr, double** phantom, int** strarea)
{
	int reg, area, count = 0;
	double average;
	char string[MAXLINESIZE];
	FILE *evalfile;

	if ((evalfile = fopen("eval", "r")) == NULL)
	{
		errorc("in read_eval_phantom1.c: error in opening file ", "eval");
	}

	while (1)
	{ /* Read eval file until first line of numbers is reached
	 then break from while loop */

		fgets(string, sizeof(string), evalfile);
		if (sscanf(string, "%d %d %lf", &reg, &area, &average) == 3)
			break;
		else
			;
	}
	/* Allocate memory for the arrays phantom, strarea and
	 regions. Store abnormality index, area and structure
	 number for the first line (from previous while loop).
	 Count is a counter for the number of structures read
	 in. */

	*phantom = (double *) malloc(sizeof(double));
	*regions = (int *) malloc(sizeof(int));
	*strarea = (int *) malloc(sizeof(int));
	if ((*phantom == NULL) || (*regions == NULL))
		errorc("memory allocation failed in ", "read_eval_phantom.c");
	(*phantom)[count] = average;
	(*strarea)[count] = area;
	(*regions)[count] = reg;
	count++;

	while (1)
	{ /* Read in abnormality indexes, areas and structure numbers
	 and allocate more memory for the new values. Break from
	 the while loop when finished */

		fgets(string, sizeof(string), evalfile);
		if (sscanf(string, "%d %d %lf", &reg, &area, &average) == 3)
		{

			*phantom = (double *) realloc(*phantom, (count + 1) * sizeof(double));
			*strarea = (int *) realloc(*strarea, (count + 1) * sizeof(int));
			*regions = (int *) realloc(*regions, (count + 1) * sizeof(int));
			if (((*phantom) == NULL) || ((*regions) == NULL) || ((*strarea) == NULL))
			{
				errorc("memory reallocation failed in ", "read_eval_phantom.c");
			}

			(*phantom)[count] = average;
			(*strarea)[count] = area;
			(*regions)[count] = reg;
			count++;
		}
		else
		{
			break;
		}
	}
	*numstr = count;

	fclose(evalfile);
}
