/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/read_eval_recon3.c $
 $LastChangedRevision: 92 $
 $Date: 2014-07-02 17:46:48 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include <string.h>
#include <strings.h>

#include "experimenter.h"
#include "errorc.h"
#include "read_eval_recon3.h"

/* ----------------------------- read_eval_recon1.c ----------------------

 This function reads from the eval file the variance for
 all the structures in a reconstruction.

 INPUTS:
 iter - iteration number
 keywrd - keyword which defines the algorithm whose reconstruction is
 to be read in.
 numstr - number of structures in reconstruction.

 OUTPUT:
 recon - array of length numstr containing the variance for each
 structure in the reconstruction.
 */

void read_eval_recon3(int iter, char* keywrd, int numstr, double* recon)
{
	int reg, area, i;
	double average, dist, rel_err, variance;
	char string[MAXLINESIZE], iter_text[MAXLINESIZE], test[MAXLINESIZE];
	FILE *evalfile;

	/* convert iteration number to a string */
	sprintf(iter_text, "%d", iter);

	if ((evalfile = fopen("eval", "r")) == NULL)
	{
		errorc("in read_eval_recon1.c: error in opening file ", "eval");
	}

	while (1)
	{
		/* Read eval file until a line which contains the string
		 'execution name' is reached. If this line contains the
		 keyword break out of the while loop. If end of file is
		 reached then keyword was not found so terminate program.  */

		fgets(string, sizeof(string), evalfile);
		if (strstr(string, "execution name") != NULL)
		{
			sscanf(string, "%*s%*s %s", test);
			if (strncasecmp(test, keywrd, 4) == 0)
			{
				break;
			}
		}
		if (feof(evalfile))
		{
			sprintf(string, "Error in read_eval_recon1.c: keyword %s not found in file", keywrd);
			errorc(string, " eval");
		}
	}

	while (1)
	{
		/* Continue reading eval file until a line which contains
		 the string 'Metrics for algorithm' is reached. If this
		 line contains the string 'iteration' and the required
		 iteration number break out of the while loop. If end
		 of file is reached then the appropriate string was
		 not found so print error message and exit */

		fgets(string, sizeof(string), evalfile);
		if ((strstr(string, "metrics for algorithm") != NULL)
				&& // changed "m" in "metrics for algorithm" to lowercase. Lajos, Dec 16, 2004
				(strstr(string, iter_text) != NULL)
				&& (strstr(string, "iteration") != NULL))
		{
			break;
		}

		if (feof(evalfile))
		{
			sprintf(string, "Error in read_eval_recon1.c: %s ---> iteration %d not found in", keywrd, iter);
			errorc(string, " eval");
		}
	}

	while (1)
	{
		/* Continue reading eval file until a line of numbers is
		 reached then break out of while loop. Terminate program
		 if end of file is reached */

		fgets(string, sizeof(string), evalfile);
		if (sscanf(string, "%d %d %lf %lf %lf %lf", &reg, &area, &average, &dist, &rel_err, &variance) == 6)
		{
			break;
		}
	}

	if (feof(evalfile))
	{
		errorc("Error in read_eval_recon1.c: end of file reached in file ", "eval");
	}

	/* this is the variance of the first structure
	 in the reconstruction */
	recon[0] = variance;

	/* now read in the abnormality index for the remaining
	 structures */

	for (i = 1; i < numstr; i++)
	{
		fgets(string, sizeof(string), evalfile);
		sscanf(string, "%d %d %lf %lf %lf %lf", &reg, &area, &average, &dist, &rel_err, &variance);
		recon[i] = variance;
	}

	fclose(evalfile); /* close eval file */
}
