/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/read_fomfile.c $
 $LastChangedRevision: 94 $
 $Date: 2014-07-02 18:53:30 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include <string.h>

#include "experimenter.h"
#include "errorc.h"
#include "read_fomfile.h"

/* ----------------------------- read_fomfile.c -------------------------

 This function reads the fom values stored in fomfile.

 INPUTS:
 iexp - experiment number
 irun - run number
 compare - counter for the number of COMPAREs read in so far
 niters - number of iterations to be compared
 fom - character string denoting the figure of merit

 OUTPUTS:
 fom1 - array of length niters containing the fom for the first algorithm.
 fom2 - array of length niters containing the fom for the second algorithm.

 */

void read_fomfile(int iexp, int irun, int compare, int niters, char* fom, double* fom1, double* fom2)
{
	int itr_a, itr_b, rexp, rrun, rcompare, i;
	double fom_a, fom_b;
	char string[MAXLINESIZE];
	FILE *fomfile;

	strcpy(string, "fomfil");
	sprintf(string, "%s.%d", string, iexp);

	if ((fomfile = fopen(string, "r")) == NULL)
	{
		errorc("in read_fomfile.c: error in opening file ", string);
	}

	while (1)
	{
		/* Read fomfil until a line which contains the strings
		 'run','experiment', and 'compare' is reached. If this
		 line contains the correct combination of iexp, irun
		 and compare break out of while loop  */

		fgets(string, sizeof(string), fomfile);
		if (strstr(string, "experiment") != NULL)
		{
			sscanf(string, "%*s %d %*s %d %*s %d", &rexp, &rrun, &rcompare);
			if ((rexp == iexp) && (rrun == irun) && (rcompare == compare))
			{
				break;
			}
		}
	}
	while (1)
	{
		/* Continue reading fomfil until a line which contains
		 either the string '*****' or the fom is reached
		 then break out of while loop */

		fgets(string, sizeof(string), fomfile);
		if (strstr(string, fom) != NULL)
		{
			break;
		}

		if (strstr(string, "*****") != NULL)
		{
			/* the string '*****' means no paired structures */
			for (i = 0; i < niters; i++)
			{
				fom1[i] = 0.0;
				fom2[i] = 0.0;
			}
			fclose(fomfile);
			return;
		}
	}

	/* now read in the iteration numbers and the foms */

	for (i = 0; i < niters; i++)
	{
		fgets(string, sizeof(string), fomfile);
		sscanf(string, "%d %lf %d %lf", &itr_a, &fom_a, &itr_b, &fom_b);
		fom1[i] = fom_a;
		fom2[i] = fom_b;
	}

	fclose(fomfile);
}
