/*      
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/select.cpp $
 $LastChangedRevision: 96 $
 $Date: 2014-07-02 20:06:42 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 PROCESS THE SELECT COMMAND AND SET THE VARIABLES IN /RAYSEL/ FOR
 USE BY PICK
 */

#include <cstdlib>
#include <cstdio>

#include "blkdta.h"
#include "raysel.h"
#include "uiod.h"

#include "geom.h" 
#include "effpick.h" 
#include "int2str.h"
#include "infile.h"

#include "select.h"

void select(BOOLEAN isdefault)                    //add isdefault option, by wei
{

	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 16, 2004
	static const INTEGER sel_type[] =
	{
			CHAR2INT('r', 'a', 'y', 's'), CHAR2INT('p', 'r', 'o', 'j'),
			CHAR2INT('r', 'a', 'n', 'd'), CHAR2INT('e', 'f', 'f', 'i') // adding efficient mode
	};

	static const INTEGER user = CHAR2INT('u', 's', 'e', 'r');
	static const INTEGER snark = CHAR2INT('s', 'n', 'a', 'r');

	// modified calls to getwrd() to use the 4-parameter version. Lajos, Dec 16, 2004
	static const INTEGER select_codes[3] =
	{ CHAR2INT('u', 's', 'e', 'r'), CHAR2INT('s', 'n', 'a', 'r'), CHAR2INT('s', 't', 'e', 'p') };

//  static const INTEGER step = 35;  // commented out since it is unused. Lajos, Jan 13, 2005

	int i;
	int NumRay;  //for efficient mode
	INTEGER word;
	BOOLEAN eol;

	if (isdefault)
		goto L40;
	//default select efficient, by wei

	word = InFile.getwrd(FALSE, &eol, select_codes, 2);
	if ((word != user) && (word != snark))
	{
		fprintf(output, "\n **** select must be followed by either USER or SNARK");
		fprintf(output, "\n **** program aborted\n");
		exit(666);
	}

	RaySel.useray = (word == user);
	word = InFile.getwrd(FALSE, &eol, sel_type, 4);
	for (i = 0; i < 3; i++)
	{
		if (word == sel_type[i])
			goto L20;
	}

	if (word == sel_type[3])
	{ //efficient way

		word = InFile.getwrd(FALSE, &eol, sel_type, 4);
		if (eol)
		{
			RaySel.stype = 7;
			goto L40;
		}
		else
		{
			fprintf(output, "\n **** unknown selection type %s", int2str(word));
			fprintf(output, "\n **** program aborted\n");
			exit(666);
		}
	}

	if (!eol)
	{
// ERROR
		fprintf(output, "\n **** unknown selection type %s", int2str(word));
		fprintf(output, "\n **** program aborted\n");
		exit(666);
	}

//RaySel.stype = 0;
	RaySel.stype = 7;
	goto L40;  			// efficient mode by default wei 1/2005

	L20: RaySel.stype = i;
	word = InFile.getint(FALSE, &eol);
	if (eol)
		goto L30;
	if (word < 0)
	{
		fprintf(output, "\n **** illegal selection : %s %s, the number after a selection type must be greater than zero", int2str(sel_type[RaySel.stype]), int2str(word));
		fprintf(output, "\n **** program aborted\n");
		exit(666);
	}

	RaySel.rn1 = word;
	word = InFile.getwrd(FALSE, &eol, sel_type, 3);
	if (eol)
	{
		// ERROR
		fprintf(output, "\n **** illegal selection : %s, %i", int2str(sel_type[RaySel.stype]), RaySel.rn1);
		fprintf(output, "\n **** program aborted\n");
		exit(666);
	}

	for (i = 0; i < 3; i++)
	{
		if (word == sel_type[i])
			goto L25;
	}
// ERROR
	fprintf(output, "\n **** unknown selection type %s", int2str(word));
	fprintf(output, "\n **** program aborted\n");
	exit(666);

	L25: if ((RaySel.stype == 0) && (i == 2))
		RaySel.stype = 5;
	if ((RaySel.stype == 1) && (i == 2))
		RaySel.stype = 6;
	if ((RaySel.stype == 2) && (i == 0))
		RaySel.stype = 3;
	if ((RaySel.stype == 2) && (i == 1))
		RaySel.stype = 4;

	if (!((RaySel.stype != 0) && (RaySel.stype != 1) && (RaySel.stype != 2)))
	{

		// ERROR
		fprintf(output, "\n **** illegal select combination : %s, %s", int2str(sel_type[RaySel.stype]), int2str(sel_type[i]));
		fprintf(output, "\n **** program aborted\n");
		exit(666);
	}

	word = InFile.getint(FALSE, &eol);
	if ((!eol) && (word <= 0))
	{
		fprintf(output, "\n **** illegal selection : %s %s, the number after a selection type must be greater than zero", int2str(sel_type[RaySel.stype]), int2str(word));
		fprintf(output, "\n **** program aborted\n");
		exit(666);
	}

	RaySel.rn2 = word;

	L30: if (RaySel.stype != 2)
	{

		///if(getwrd(input, &eol) != keywd[step]) {
		if (InFile.getwrd(TRUE, &eol, &(select_codes[2]), 1) != CHAR2INT('s','t','e','p'))
		{
			fprintf(output, "\n **** keyword STEP missing");
			fprintf(output, "\n **** program aborted\n");
			exit(666);
		}

		RaySel.prjinc = InFile.getint(FALSE, &eol);
		RaySel.rayinc = InFile.getint(FALSE, &eol);
		if ((RaySel.prjinc <= 0) || (RaySel.rayinc <= 0))
		{
			fprintf(output, "\n **** both mod1 and mod2 must be positive.");
			fprintf(output, "\n      mod1 = %5i, mod2 = %5i", RaySel.prjinc, RaySel.rayinc);
			fprintf(output, "\n **** program aborted\n");
			exit(666);
		}

	}

	// ECHO INPUT

	if (RaySel.stype == 3 && eol)
	{
		fprintf(output, "\n         a fixed number of %5i rays at random, followed by sequential ray selection", RaySel.rn1);
		fprintf(output, "\n         with ray-step %5i and projection-step %5i", RaySel.rayinc, RaySel.prjinc);
		return;
	}

	if (RaySel.stype == 4 && eol)
	{
		fprintf(output, "\n         a fixed number of %5i rays at random followed by sequential projection selection", RaySel.rn1);
		fprintf(output, "\n         with projection-step %5i and ray-step %5i", RaySel.prjinc, RaySel.rayinc);
		return;
	}

	if (RaySel.stype == 5 && eol)
	{
		fprintf(output, "\n         a fixed number of, %5i sequential rays, with ray-step, %5i and projection-step %5i", RaySel.rn1, RaySel.rayinc, RaySel.prjinc);
		fprintf(output, "\n         followed by random selection of rays and projections");
		return;
	}

	if (RaySel.stype == 6 && eol)
	{
		fprintf(output, "\n         a fixed number of, %5i sequential rays, with ray-step, %5i and projection-step %5i", RaySel.rn1, RaySel.prjinc, RaySel.rayinc);
		fprintf(output, "\n         followed by random selection of rays and projections");
		return;
	}

	L40: switch (RaySel.stype)
	{
	case 0:
		fprintf(output, "\n         sequential ray selection with ray-step %5i and projection-step %5i", RaySel.rayinc, RaySel.prjinc);
		return;

	case 1:
		fprintf(output, "\n         sequential projection selection with projection-step %5i and ray-step %5i", RaySel.prjinc, RaySel.rayinc);
		return;

	case 2:
		fprintf(output, "\n         random selection of rays and projections");
		return;

	case 3:
		fprintf(output, "\n         cycles of %5i rays at random", RaySel.rn1);
		fprintf(output, "\n         followed by a fixed number of %5i sequential rays with ray-step %5i and projection-step %5i", RaySel.rn2, RaySel.rayinc, RaySel.prjinc);
		return;

	case 4:
		fprintf(output, "\n         cycles of %5i rays at random", RaySel.rn1);
		fprintf(output, "\n         followed by a fixed number of %5i sequential projection selection with projection-step %5i and ray-step %5i", RaySel.rn2, RaySel.prjinc, RaySel.rayinc);
		return;

	case 5:
		fprintf(output, "\n         cycles of %5i sequential rays with ray-step %5i and projection-step %5i", RaySel.rn1, RaySel.rayinc, RaySel.prjinc);
		fprintf(output, "\n         followed by %5i rays at random", RaySel.rn2);
		return;

	case 6:
		fprintf(output, "\n         cycles of %5i sequential projection selection with projection-step %5i and ray-step %5i", RaySel.rn1, RaySel.prjinc, RaySel.rayinc);
		fprintf(output, "\n         followed by %5i rays at random", RaySel.rn2);

		return;

	case 7:
		if (!isdefault)
			fprintf(output, "\n         efficient data access");

		if (!RaySel.useray)
			NumRay = GeoPar.snrays;
		else
			NumRay = GeoPar.usrays;

		factorization(GeoPar.prjnum, &RaySel.decompprj, &RaySel.num_digprj);
		factorization(NumRay, &RaySel.decompray, &RaySel.num_digray);
		//factorization(30, &RaySel.decompray, &RaySel.num_digray);

		get_base(RaySel.decompprj, RaySel.num_digprj, &RaySel.baseprj);
		get_base(RaySel.decompray, RaySel.num_digray, &RaySel.baseray);

		if (RaySel.old_numray != NULL)
			delete[] RaySel.old_numray;
		if (RaySel.old_numprj != NULL)
			delete[] RaySel.old_numprj;
		if (RaySel.new_numray != NULL)
			delete[] RaySel.new_numray;
		if (RaySel.new_numprj != NULL)
			delete[] RaySel.new_numprj; //added to avoid memory leak, by wei 1/2005

		RaySel.old_numray = new INTEGER[RaySel.num_digray];
		RaySel.old_numprj = new INTEGER[RaySel.num_digprj];
		RaySel.new_numray = new INTEGER[RaySel.num_digray];
		RaySel.new_numprj = new INTEGER[RaySel.num_digprj];

		RaySel.old_numprj[0] = -1;

		return;
	}

}
