/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/trm3.cpp $
 $LastChangedRevision: 122 $
 $Date: 2014-07-09 18:18:59 -0400 (Wed, 09 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 trm3.cpp,v 1.5 2009/06/01 03:33:26 jklukowska Exp

 */

#include <math.h>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"
#include "consts.h"  // added. hstau
#include "trm3.h"
#include "blob.h"   //wei, 9/05

// bug220 - allow user args for termination tests - swr - 1/30/07

void trm3_class::Init()
{
	BOOLEAN eol;

	epsiln = InFile.getnum(FALSE, &eol);
	// bug 128 - added check for epsilon - swr - 7/6/05
	if (epsiln <= Consts_class::zero)
	{
		fprintf(output, "\n **** epsilon must be specified and greater than ZERO");
		fprintf(output, "\n **** program aborted\n");
		exit(-1);
	}
	fprintf(output, "\n         epsilon = %10.6f", epsiln);
}

BOOLEAN trm3_class::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter)
{

	static REAL mean;
	static REAL var1;
	static REAL var2;
	static REAL chgvar;
	REAL area;                       //wei, 9/05

	int i;

	REAL sum = 0.0;
	REAL sqsum = 0.0;

	INTEGER lhf, lhl, lmf, lml, h, m, h1, m1, ind;    //wei, 9/05

	if (!Blob.pix_basis)
	{

		lhf = (INTEGER) (-1 * (INTEGER) (Blob.H / 2));
		lhl = (INTEGER) ((Blob.H - 1) / 2);

		lmf = -1 * (INTEGER) (Blob.M / 2);
		lml = (INTEGER) ((Blob.M - 1) / 2);

		area = Blob.area / 2;

		for (h = lhl; h >= lhf; h--)
		{
			for (m = lmf; m <= lml; m++)
			{ // m and h must have the same parity!!
				if ((m + h) % 2 == 0)
				{
					m1 = m + Blob.M2;
					h1 = h + Blob.H2;
					ind = h1 * Blob.M + m1;
					sum += recon[ind];
					sqsum += recon[ind] * recon[ind];
				}
			}
		}

	} //wei, 9/05

	else
	{
		area = GeoPar.area;

		for (i = 0; i < area; i++)
		{
			sum += recon[i];
			sqsum += recon[i] * recon[i];
		}
	}     //wei, 9/05

	mean = sum / area;
	var2 = sqsum / area - mean * mean;   //wei, 9/05

	if (!(iter == 1))
	{
		if (fabs(var1) < Consts.zero)
		{  // added for robustness. hstau
			fprintf(output,
					"\n          the variance in the previous iteration is less than ZERO");
			return TRUE;
		}
		else
		{             //added for robustness. hsta
			chgvar = ((REAL) fabs(var2 - var1)) / var1;
			if (chgvar < epsiln)
			{
				fprintf(output,
						"\n          iterative process stops at iteration %5i",
						iter);
				fprintf(output,
						"\n          the change in variance is less than %10.6f of the variance",
						chgvar);
				return TRUE;
			}
		}
	}
	var1 = var2;

	return FALSE;
}
