/** @file displayprojection.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
        
    modification: Joanna Klukowska 1/13/2009
	    added choice of grayScale display for plot
	    in addition to color display
 */

#include "variables.hpp"
#include "verbosity.hpp"
#include "displaywidget.hpp"
#include "displayprojection.hpp"
#include "lines.hpp"
#include "SnarkDisplay.hpp"
#include "line_real_t.hpp"
#include "plot_t.hpp"
#include "displaylines.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpen.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qcolor.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qfont.h>
#include <qregexp.h>
#include <qstring.h>
#include <qscrollview.h>

#ifndef MIN
#define MIN(a, b) (((a < b) ? (a) : (b)))
#endif

extern Snarkimage **imageset;
QImage projection;
bool convproj;

/* 
 *  Constructs a displayprojection which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
displayprojection::displayprojection(QWidget* parent, const char* name, bool modal, WFlags fl, int x, int y)
: QDialog(parent, name, modal, fl) {
    unsigned int i, j, minvalrange, maxvalrange;
    QPixmap disp;
    QPoint topleft;

    windowname.sprintf("Sinogram: %s", name);
    windowname = windowname.simplifyWhiteSpace();
    imagename.sprintf("%s", name);
    imagename = imagename.simplifyWhiteSpace();
    setName(windowname);
    setCaption(windowname);

    topleft.setX(x + 30);
    topleft.setY(y + 30);
    this->move(topleft);
    //    projfil->readProj(projformat);
    projfil->readProj();
    maxzoom = 2 * maxSizex / prjnum;
    if (maxzoom <= 1) maxzoom = 2;
    proj = projfil->getProj();
    lowthreshold = minval = projfil->getMin();
    highthreshold = maxval = projfil->getMax();
    range = highthreshold - lowthreshold;

    setMinimumSize(QSize(190 + prjnum + 10, 600));
    resize(190 + prjnum + 10, usrays + 20);
    //    if((10+(usrays*maxzoom)+10)>600)
    //      setMaximumSize( QSize( 190+(prjnum*maxzoom)+10, 10+(usrays*maxzoom)+10 ) );
    //    else 
    //      setMaximumSize( QSize( 190+(prjnum*maxzoom)+10, 600 ) );

    projection = QImage(prjnum, usrays, 8, 256);
    for (i = 0; i < 256; i++)
        projection.setColor(i, qRgb(i, i, i));
    for (i = 0; i < usrays; i++)
        for (j = 0; j < prjnum; j++)
            *(projection.scanLine(i) + j) = (unsigned char) (((proj[j][i] - lowthreshold) / range)*255);

    disp = QPixmap(prjnum, usrays);
    convproj = disp.convertFromImage(projection, QPixmap::Auto);

    displaywidget = new displayWidget(this, "displaywidget");
    displaywidget->setGeometry(QRect(190, 10, prjnum, usrays));
    displaywidget->setBackgroundPixmap(disp);
    //displaywidget->setScaledContents( TRUE );
    QObject::connect(displaywidget, SIGNAL(clicked(int, int)), this, SLOT(updateClickedPixel(int, int)));

    maxvalrange = (int) (maxval * 1000);
    minvalrange = (int) (minval * 1000);

    zoomlabel = new QLabel(this, "zoomlabel");
    zoomlabel->setGeometry(QRect(60, 5, 45, 20));
    zoomlabel->setText(tr("Zoom"));

    zoomslider = new QSlider(this, "zoomslider");
    zoomslider->setGeometry(QRect(5, 30, 170, 20));
    zoomslider->setOrientation(QSlider::Horizontal);
    zoomslider->setRange(1, maxzoom);
    zoomslider->setValue(1);
    zoomslider->setLineStep(1);
    zoomslider->setPageStep(2);
    QObject::connect(zoomslider, SIGNAL(valueChanged(int)), this, SLOT(updateZoom()));
    zoomval = 1;

    zoom = new QLineEdit(this, "zoom");
    zoom->setGeometry(QRect(5, 55, 80, 30));
    zoom->setText("1");

    setzoombutton = new QPushButton(this, "setzoombutton");
    setzoombutton->setGeometry(QRect(90, 50, 90, 40));
    setzoombutton->setText(tr("Set Value"));
    QObject::connect(setzoombutton, SIGNAL(clicked()), this, SLOT(updateZoomslider()));

    globalthresholdbutton = new QRadioButton(this, "globalthresholdbutton");
    globalthresholdbutton->setGeometry(QRect(10, 100, 160, 25));
    globalthresholdbutton->setText(tr("Global Thresholds"));
    globalthresholdbutton->setEnabled(false);

    differencebutton = new QRadioButton(this, "differencebutton");
    differencebutton->setGeometry(QRect(10, 135, 160, 25));
    differencebutton->setText(tr("Difference"));
    differencebutton->setEnabled(false);

    intensitybutton = new QRadioButton(this, "intensitybutton");
    intensitybutton->setGeometry(QRect(10, 170, 160, 25));
    intensitybutton->setText(tr("Intensity"));
    QObject::connect(intensitybutton, SIGNAL(clicked()), this, SLOT(intensity()));

    lowthreshlabel = new QLabel(this, "lowthreshlabel");
    lowthreshlabel->setGeometry(QRect(30, 205, 110, 20));
    lowthreshlabel->setText(tr("Low Threshold"));

    lowthreshslider = new QSlider(this, "Slider4_2");
    lowthreshslider->setGeometry(QRect(5, 230, 170, 20));
    lowthreshslider->setOrientation(QSlider::Horizontal);
    lowthreshslider->setRange(minvalrange, maxvalrange);
    lowthreshslider->setValue(minvalrange);
    QObject::connect(lowthreshslider, SIGNAL(valueChanged(int)), this, SLOT(updateLowthresh()));

    lowthresh = new QLineEdit(this, "lowthresh");
    lowthresh->setGeometry(QRect(5, 260, 80, 30));
    char s[16];
    snprintf(s,sizeof(s)-1,"%7.4f", minval);
    lowthresh->setText(s);

    setlowthreshbutton = new QPushButton(this, "setlowthreshbutton");
    setlowthreshbutton->setGeometry(QRect(90, 255, 90, 40));
    setlowthreshbutton->setText(tr("Set Value"));
    QObject::connect(setlowthreshbutton, SIGNAL(clicked()), this, SLOT(updateLowthreshslider()));

    highthreshlabel = new QLabel(this, "highthreshlabel");
    highthreshlabel->setGeometry(QRect(30, 300, 115, 20));
    highthreshlabel->setText(tr("High Threshold"));

    highthreshslider = new QSlider(this, "Slider4_3");
    highthreshslider->setGeometry(QRect(5, 325, 170, 20));
    highthreshslider->setOrientation(QSlider::Horizontal);
    highthreshslider->setRange(minvalrange, maxvalrange);
    highthreshslider->setValue(maxvalrange);
    QObject::connect(highthreshslider, SIGNAL(valueChanged(int)), this, SLOT(updateHighthresh()));

    highthresh = new QLineEdit(this, "highthresh");
    highthresh->setGeometry(QRect(5, 355, 80, 30));
    sprintf(s, "%7.4f", maxval);
    highthresh->setText(s);

    sethighthreshbutton = new QPushButton(this, "sethighthreshbutton");
    sethighthreshbutton->setGeometry(QRect(90, 350, 90, 40));
    sethighthreshbutton->setText(tr("Set Value"));
    QObject::connect(sethighthreshbutton, SIGNAL(clicked()), this, SLOT(updateHighthreshslider()));

    pixelposlabel = new QLabel(this, "pixelposlabel");
    pixelposlabel->setGeometry(QRect(30, 395, 45, 20));
    pixelposlabel->setText(tr("Pixel"));

    pixelpos = new QLineEdit(this, "pixelpos");
    pixelpos->setGeometry(QRect(5, 420, 80, 30));

    pixelvaluelabel = new QLabel(this, "pixelvaluelabel");
    pixelvaluelabel->setGeometry(QRect(110, 395, 45, 20));
    pixelvaluelabel->setText(tr("Value"));

    pixelvalue = new QLineEdit(this, "pixelvalue");
    pixelvalue->setGeometry(QRect(95, 420, 80, 30));

    savebutton = new QPushButton(this, "savebutton");
    savebutton->setGeometry(QRect(15, 460, 150, 40));
    savebutton->setText(tr("Save Image"));
    QObject::connect(savebutton, SIGNAL(clicked()), this, SLOT(saveImage()));

    linesbutton = new QPushButton(this, "linesbutton");
    linesbutton->setGeometry(QRect(15, 505, 150, 40));
    linesbutton->setText(tr("Display Lines"));
    // QObject::connect(linesbutton,SIGNAL(clicked()),this,SLOT(callLines_old()));
    QObject::connect(linesbutton, SIGNAL(clicked()), this, SLOT(callLines()));

    closebutton = new QPushButton(this, "closebutton");
    closebutton->setGeometry(QRect(15, 550, 150, 40));
    closebutton->setText(tr("Close Window"));
    QObject::connect(closebutton, SIGNAL(clicked()), this, SLOT(accept()));

    scrollv = new QScrollView(this, "qscrollview");
    scrollv->viewport()->setBackgroundColor(colorGroup().background());
    scrollv->setFrameShape(QFrame::NoFrame);
    scrollv->setGeometry(190, 10, maxSizex - 210, maxSizey - 110);
    scrollv->addChild(displaywidget);
}

void displayprojection::resetscroll() {
    scrollv->resize(MIN(maxSizex - 210, 20 + (prjnum * zoomval)),
            MIN(maxSizey - 110, 20 + (usrays * zoomval)));
    scrollv->resizeContents(prjnum * zoomval + 10, usrays * zoomval + 10);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
displayprojection::~displayprojection() {
    // no need to delete child widgets, Qt does it all for us
}

void displayprojection::updateClickedPixel(int cx, int cy) {
    int ix, iy;
    char s[12];

    ix = cx / zoomval;
    iy = cy / zoomval;
    sprintf(s, "(%d,%d)", ix, iy);
    pixelpos->setText(s);
    sprintf(s, "%7.4f", proj[ix][iy]);
    pixelvalue->setText(s);
}

void displayprojection::updateZoom() {
    char s[4];
    int v = zoomslider->value();
    sprintf(s, "%d", v);
    zoom->setText(s);
    zoomval = v;
    displaywidget->resize(prjnum*zoomval, usrays * zoomval);
    resize(MIN(maxSizex, 210 + (prjnum * zoomval)),
            MIN(maxSizey, 20 + (usrays * zoomval)));
    resetImage();
    resetscroll();
}

void displayprojection::updateZoomslider() {
    int v;
    bool ok = false;
    v = zoom->text().toInt(&ok, 10);
    if (ok && v > 0 && v <= maxzoom)
        zoomslider->setValue(v);
    else
        QMessageBox::information(this, "SnarkDisplay", "Error!\n"
            "Zoom value invalid or out of range.");
}

void displayprojection::intensity() {
    int l;

    l = windowname.length();
    if (intensitybutton->isChecked()) {
        windowname.append("_i");
        setCaption(windowname);
    } else {
        windowname.remove(l - 2, 2);
        setCaption(windowname);
    }
    resetImage();
}

/** obtain threshold value from the low threshold slider position
    update the text of the slider
    pre: low threshold slider not greater than high threshold slider
         Have fixed this
 */
void displayprojection::updateLowthresh() {
    char s[12];
    int v = lowthreshslider->value();
    int hv = highthreshslider->value();
    if (v > hv) {
        v = hv;
        lowthreshslider->setValue(hv);
    }
    double fv = ((double) v) / 1000;
    lowthreshold = fv;
    sprintf(s, "%7.4f", fv);
    lowthresh->setText(s);
    resetImage();
}

double displayprojection::getLowthresh() {
    return lowthreshold;
}

void displayprojection::updateLowthreshslider() {
    double fv;
    int v = 0;
    bool ok = false;
    fv = lowthresh->text().toDouble(&ok);
    if (ok) {
        if (fv > highthreshold) {
            QMessageBox::information(this, "SnarkDisplay", "Error!\n"
                    "Entered low threshold higher than high threshold.");
            return;
        }
        if (fv > 0)
            v = (int) (fv * 1000 + 0.5);
        else
            v = (int) (fv * 1000 - 0.5);
        if (v < lowthreshslider->minValue())
            lowthreshslider->setMinValue(v);
        lowthreshslider->setValue(v);
        lowthreshold = fv;
    }
}

/** have fixed this */
void displayprojection::updateHighthresh() {
    char s[12];
    int v = highthreshslider->value();
    int Lv = lowthreshslider->value();
    if (v < Lv) {
        v = Lv;
        highthreshslider->setValue(Lv);
    }
    double fv = ((double) v) / 1000;
    highthreshold = fv;
    sprintf(s, "%7.4f", fv);
    highthresh->setText(s);
    resetImage();
}

double displayprojection::getHighthresh() {
    return highthreshold;
}

void displayprojection::updateHighthreshslider() {
    double fv;
    int v = 0;
    bool ok = false;
    fv = highthresh->text().toDouble(&ok);
    if (ok) {
        if (fv < lowthreshold) {
            QMessageBox::information(this, "SnarkDisplay", "Error!\n"
                    "Entered high threshold lower than low threshold.");
            return;
        }
        if (fv > 0)
            v = (int) (fv * 1000 + 0.5);
        else
            v = (int) (fv * 1000 - 0.5);
        if (v > highthreshslider->maxValue())
            highthreshslider->setMaxValue(v);
        highthreshslider->setValue(v);
        highthreshold = fv;
    }
} // --displayprojection::updateHighthreshslider()

void displayprojection::resetImage() {
    /* adjust image to current thresholds and zoom */
    unsigned int i, j;
    int m, n, base, point;
    double t = 0;
    unsigned char tuc;
    QPixmap disp;

    //  if((20+(usrays*zoomval)+1)>530)
    //    resize(190+(prjnum*zoomval)+10, 20+(usrays*zoomval)+10);
    //  else
    //    resize(190+(prjnum*zoomval)+10, 530);

    projection = projection.smoothScale(prjnum*zoomval, usrays * zoomval);
    projection.setNumColors(256);
    for (i = 0; i < 256; i++)
        projection.setColor(i, qRgb(i, i, i));
    range = highthreshold - lowthreshold;

    for (i = 0; i < usrays; i++) {
        base = i*zoomval;
        for (j = 0; j < prjnum; j++) {
            point = j*zoomval;
            t = ((proj[j][i] - lowthreshold) / range);
            if (t == 0.0 || t < 0.0) {
                for (m = 0; m < zoomval; m++)
                    for (n = 0; n < zoomval; n++)
                        *(projection.scanLine(base + m) + point + n) = 0;
            } else {
                if (t > 1.0 || t == 1.0) {
                    for (m = 0; m < zoomval; m++)
                        for (n = 0; n < zoomval; n++)
                            *(projection.scanLine(base + m) + point + n) = 255;
                } else {
                    if (intensitybutton->isChecked()) {
                        tuc = (unsigned char) (sqrt(t)*255);
                        for (m = 0; m < zoomval; m++)
                            for (n = 0; n < zoomval; n++) {
                                *(projection.scanLine(base + m) + point + n) = tuc;
                            }
                    } else {
                        tuc = (unsigned char) (t * 255);
                        for (m = 0; m < zoomval; m++)
                            for (n = 0; n < zoomval; n++) {
                                *(projection.scanLine(base + m) + point + n) = tuc;
                            }
                    }
                }
            }
        }
    }
    convproj = disp.convertFromImage(projection, QPixmap::Auto);
    //  displaywidget->resize(prjnum*zoomval,usrays*zoomval);
    displaywidget->setBackgroundPixmap(disp);
    resetscroll();
} // --displayprojection::resetImage()

void displayprojection::resetImageRange() {
    /* adjust image to current thresholds */
    unsigned int i, j;
    double t = 0;
    QPixmap disp;

    range = highthreshold - lowthreshold;

    for (i = 0; i < usrays; i++)
        for (j = 0; j < prjnum; j++) {
            t = ((proj[i][j] - lowthreshold) / range);
            if (t <= 0)
                *(projection.scanLine(i) + j) = 0;
            else
                if (t >= 1)
                *(projection.scanLine(i) + j) = 255;
            else
                *(projection.scanLine(i) + j) = (unsigned char) (t * 255);
        }
    convproj = disp.convertFromImage(projection, QPixmap::Auto);
    displaywidget->setBackgroundPixmap(disp);
} // --displayprojection::resetImageRange()

/** Starts a lines window 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void displayprojection::callLines() {
    int i, j;
    //  unsigned int l[4];
    int l[4], linetype, c, x, y, sx = 400, sy = 300, xn, yn;
    int a1, minx, maxx, xrange;
    int topleftx, toplefty;
    double miny, maxy, yrange, t1, t2, a2;
    bool above = false, below = false;
    char s[11];
    QString title, xaxis, yaxis, temp, profilename;
    QPoint topleft;
    QString projname_qstr(projname);
    projname_qstr = projname_qstr.simplifyWhiteSpace();
    lineswindow* nw = new lineswindow(this, projname_qstr, lowthreshold, highthreshold, true, 0);
    c = nw->exec();
    if (c == QDialog::Accepted) {
        linetype = nw->getLinetype();
        QString rowOrColumn = ((linetype) ? ("Column ") : ("Row "));
        QString vs = ((linetype) ? ("Rows") : ("Columns"));
        //    std::cerr << "Plotting " << rowOrColumn << "vs " << vs << std::endl;
        //    std::cerr << "There are " << prjnum << " columns and " << usrays << " rows" << std::endl;
        int i_upperbnd = static_cast<int> ((linetype) ? (usrays) : (prjnum));
        int elljay_upperbnd = static_cast<int> ((linetype) ? (prjnum) : (usrays));
        //    std::cerr << i_upperbnd << " is upperbound of independent" << std::endl;
        l[0] = nw->getLine1();
        l[1] = nw->getLine2();
        l[2] = nw->getLine3();
        l[3] = nw->getLine4();
        title = nw->getTitle();
        xaxis = nw->getXAxis();
        yaxis = nw->getYAxis();
        minx = nw->getMinX();
        maxx = nw->getMaxX();
        miny = nw->getMinY();
        maxy = nw->getMaxY();
        topleft = this->pos();
        topleftx = topleft.x();
        toplefty = topleft.y();
        for (j = 0; j < 4; j++) { // loop through selection
            int&row = ((linetype) ? (i) : (l[j]));
            int&col = ((linetype) ? (l[j]) : (i));
            if ((l[j] >= 0) && (l[j] < elljay_upperbnd)) { // different plot for each valid j
                line_real_set_t myLRS(minx, maxx, miny, maxy); // create a brand new LRS for brand new plot
                myLRS.clear(); // and be sure of it!
                topleftx += 30;
                toplefty += 30;
                QPixmap* pixmap = new QPixmap(sd_line_t::WIDTH, sd_line_t::HEIGHT);
                pixmap->fill(Qt::white);
                QPainter picasso;
                picasso.begin(pixmap);
                picasso.setBackgroundColor(Qt::white);
                QString whichRowOrCol;
                whichRowOrCol.setNum(l[j]);
                QString plotSuffix = (rowOrColumn + whichRowOrCol);
                //QString plotName = projname_qstr.simplifyWhiteSpace();
                QString plotName = title.simplifyWhiteSpace(); // bug 227, wei, 6/13/2007
                plotName += (" (sinogram):: " + plotSuffix);
                //	std::cerr << "plotName : " << plotName << std::endl;
                line_real_t ourProjLine(projname_qstr);
                for (i = 0; i < i_upperbnd; i++) {
                    ourProjLine.add(i, proj[col][row]);
                }
                myLRS.add(ourProjLine);
                plot_t myPlot(myLRS);
                myPlot.setName(plotName);
                myPlot.setXAxisName(xaxis); // for all three of these names
                myPlot.setYAxisName(yaxis);
                // jk 1/13/2009 adding grayScale option for graphs
                if (nw->isGrayScaleChecked()) { // picasso paints on QPixmap pointed to by pmp
                    myPlot.plotGray(picasso);
                } else {
                    myPlot.plot(picasso);
                }
                displaylineswindow* dw = new displaylineswindow(this, plotName, pixmap, false, 0, topleftx, toplefty);
                dw->show();
            } // --if on l[j] being ok
        } // --loop through the j's [0,4)
    }
} // --displayprojection::callLines()

/** Saves the current image
@param void
@author Bruno M. Carvalho
@version 1.0 */
void displayprojection::saveImage() {
    int i, j;

    QString prefixForSaving = projname;
    prefixForSaving = prefixForSaving.simplifyWhiteSpace();
    prefixForSaving.replace(QRegExp("\\="), "_EQ_");
    prefixForSaving.replace(QRegExp("\\+"), "_PLUS_");
    prefixForSaving.replace(QRegExp("\\@"), "_AT_");
    //  prefixForSaving.replace( QRegExp("-"), "_MINUS_"); // maybe
    // convert everything non-alphanumeric to underscore
    prefixForSaving.replace(QRegExp("[^a-zA-Z0-9]"), "_");
    // allow maximum of 2 consecutive underscores
    prefixForSaving.replace(QRegExp("__+"), "__");
    // chuck underscore from beginning and end
    prefixForSaving.replace(QRegExp("^_+"), "");
    prefixForSaving.replace(QRegExp("_+$"), "");

    QString myExtensionFilter =
            "png (PNG file);;"
            "bmp (BMP file);;"
            "flt (ASCII float file);;"
            "pbm (PBM file);;"
            "pgm (PGM file);;"
            "ppm (PPM file);;"
            "xbm (XBM file);;"
            "xpm (XPM file)";

    QString mySaveAs = "Save Image as";
    QString startInDir = "./";
    QFileDialog fd(startInDir, myExtensionFilter, this, mySaveAs, true);
    fd.setMode(QFileDialog::AnyFile);
    fd.setSelection(prefixForSaving);
    if (QDialog::Accepted != fd.exec()) {
        if (std::verbose >= 2) std::cout << "--displaylineswindow::saveImage() in a hurry" << std::endl;
        return; // happens, say, when cancel is pressed.  just leave, no harm.
    }
    QString userFileSavePrefix = fd.selectedFile();
    QString userFileSaveExtension = fd.selectedFilter();
    if (userFileSavePrefix.isEmpty()) { // double-check for absurdity
        if (std::verbose >= 2) std::cout << "--displaylineswindow::saveImage() in a hurry" << std::endl;
        return; // happens, say, when cancel is pressed.  just leave, no harm.
    }
    if (std::verbose >= 2) std::cout << "Got file name as: \"" << userFileSavePrefix << "\"" << std::endl;
    if (std::verbose >= 2) std::cout << "Got selected ext: \"" << userFileSaveExtension << "\"" << std::endl;
    /* NB: userFileSavePrefix now contains directory info as well.
     * if user entered additional forward-slashes without
     * knowing that it signified directory, that's NOT our problem.
     * (the user MAY legitimately enter "a/myImage" as file name if there is a directory "a"
     *   within the currently open directory in the save dialog window)
     * do part only after last forward-slash (i.e., what user entered)...
     * don't want to burn ourselves if path in fact had legit non-alphanumericounderscore chars
     */
    QString justFilePrefix, justPath;
    // funnylooking because written to work with Qt BOTH 2.3 and 3.3
    int posLastSlash = userFileSavePrefix.findRev('/'); // returns pos if found, -1 if not.
    if (-1 != posLastSlash) {
        justPath = userFileSavePrefix.left(posLastSlash + 1); // +1 to include last '/'
        justFilePrefix = userFileSavePrefix;
        QRegExp fakeRegex("^" + justPath); // it just *looks* fake...
        justFilePrefix.replace(fakeRegex, ""); // chuck away including last '/' for justFilePrefix
        if (std::verbose >= 2) std::cout << "just after regex replace, justFilePrefix: \"" << justFilePrefix << "\"" << std::endl;
    } else { // if for some reason no forwardslash, handle that gracefully too.
        justFilePrefix = userFileSavePrefix;
        justPath = "";
    }
    if (std::verbose >= 2) std::cout << "justPath is: \"" << justPath << "\"" << std::endl;
    if (std::verbose >= 2) std::cout << "justFilePrefix is first: \"" << justFilePrefix << "\"" << std::endl;
    // remove justFilePrefix from ONLY the end of 'userFileSavePrefix'.
    justFilePrefix.replace(QRegExp("="), "_EQ_");
    justFilePrefix.replace(QRegExp("+"), "_PLUS_");
    justFilePrefix.replace(QRegExp("[^a-zA-Z0-9]"), "_"); // no forward slash in regex
    // allow maximum of 2 consecutive underscores
    justFilePrefix.replace(QRegExp("__+"), "__");
    // chuck underscore from beginning and end
    justFilePrefix.replace(QRegExp("^_+"), "");
    justFilePrefix.replace(QRegExp("_+$"), "");
    if (std::verbose >= 2) std::cout << "justFilePrefix is later: \"" << justFilePrefix << "\"" << std::endl;
    // reset userFileSavePrefix by putting back together justPath and the refined justFilePrefix
    userFileSavePrefix = justPath + justFilePrefix;
    userFileSaveExtension.truncate(3);
    if (std::verbose >= 2) std::cout << "My userFileSavePrefix is \"" << userFileSavePrefix << "\"" << std::endl;
    if (std::verbose >= 2) std::cout << "My userFileSaveExtension is \"" << userFileSaveExtension << "\"" << std::endl;
    QString extensionUpper = userFileSaveExtension.upper();
    QString saveFullNameWithPath = userFileSavePrefix + "." + extensionUpper;
    if (std::verbose >= 2) std::cout << "extensionUpper is now \"" << extensionUpper << "\"" << std::endl;
    if (std::verbose >= 2) std::cout << "saveFullNameWithPath is now \"" << saveFullNameWithPath << "\"" << std::endl;
    if (extensionUpper.compare(QString("FLT"))) { // if NOT "FLT" extension ("FLT" is Bruno's / Linkoping's bunchafloats format)
        QImageIO recio;
        recio.setImage(projection);
        recio.setFormat(extensionUpper);
        recio.setFileName(saveFullNameWithPath);
        if (recio.write()) {
            if (std::verbose >= 1) {
                std::cout << "successfully saved: \"" << saveFullNameWithPath << "\"" << std::endl;
                return;
            }
        } else {
            std::cerr << "Error opening " << saveFullNameWithPath << std::endl;
            return;
        }
    } else {
        FILE* fp;
        if ((fp = fopen(saveFullNameWithPath.latin1(), "w")) == NULL) {
            std::cerr << "UNsuccessfully tried to save as: \"" << saveFullNameWithPath << "\"" << std::endl;
        } else {
            for (i = 0; i < (static_cast<int> (usrays)); i++) {
                for (j = 0; j < (static_cast<int> (prjnum)); j++) {
                    fprintf(fp, "%f ", proj[i][j]);
                    if (!((j + 1) % 5))
                        fprintf(fp, "\n");
                }
                fprintf(fp, "\n");
            }
            fclose(fp);
            if (std::verbose >= 1) {
                std::cout << "(we think) successfully saved: \"" << saveFullNameWithPath << "\"" << std::endl;
            }
        }
    }
}
