/** @file displaywindow.hpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#ifndef DISPLAYWINDOW_H
#define DISPLAYWINDOW_H

#include "variables.hpp"
#include "displaywidget.hpp"
#include <qvariant.h>
#include <qdialog.h>
#include <qimage.h>
#include <qwidget.h>
class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QLabel;
class QLineEdit;
class QPushButton;
class QRadioButton;
class QSlider;
class QListBox;
class QImage;
class QPixmap;
class QScrollView;

// class displaywindow : public QDialog
class displaywindow : public QWidget
{ 
    Q_OBJECT

public:
    displaywindow( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 , DIGDataFormat fileformat = DIGDataFormat_BINARY, int imagenumber = 0, int x = 0 , int y = 0 );
    ~displaywindow();

    bool useglobal, usedifference, useintensity;
    int id;
    QString imagename;
    QImage* recp;
    bool converted;
    double **image;
    double **diffimage;
    bool status;
    double maxval,minval,diffmaxval,diffminval;
    double lowthreshold,highthreshold,difflowthreshold,diffhighthreshold;
    double range,Range;
    int zoomval;
    int maxzoom;
    displayWidget* displaywidget;
    QScrollView* scrollv;
    QLabel* zoomlabel;
    QSlider* zoomslider;
    QLineEdit* zoom;
    QPushButton* setzoombutton;
  //    QRadioButton* globalthresholdbutton;
  //    QRadioButton* differencebutton;
  //    QRadioButton* intensitybutton;
    QLabel* lowthreshlabel;
    QSlider* lowthreshslider;
    QLineEdit* lowthresh;
    QPushButton* setlowthreshbutton;
    QLabel* highthreshlabel;
    QSlider* highthreshslider;
    QLineEdit* highthresh;
    QPushButton* sethighthreshbutton;
    QLabel* pixelposlabel;
    QLineEdit* pixelpos;
    QLabel* pixelvaluelabel;
    QLineEdit* pixelvalue;
    QPushButton* profilebutton;
    QPushButton* linesbutton;
    QPushButton* savebutton;
    QPushButton* closebutton;
public slots:
    void updateClickedPixel(int cx,int cy);
    void updateZoom();
    void updateZoomslider();

    void setGlobal(bool on);
    void setDifference(bool on);
    void setIntensity(bool on);

    bool isGlobalthreshs();
    void updateGlobalthreshs();
  //    void setGlobalthreshs(bool value);
    void setGlobalthreshs();
  //    void differencePhantom();
  //    void intensity();
    void updateLowthresh();
    double getLowthresh();
    void updateLowthreshslider();
    void updateHighthresh();
    double getHighthresh();
    void updateHighthreshslider();
    void resetImage();
    void resetDiffImage();
    void resetImageRange();
    void callLines();
    void saveImage();
    void closeWindow();

 protected:
    virtual void closeEvent(QCloseEvent *e);
};

#endif // DISPLAYWINDOW_H
