/** @file range.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "range.hpp"
#include "variables.hpp"
#include "SnarkDisplay.hpp"

#include <stdio.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>


/* 
 *  Constructs a rangewindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
rangewindow::rangewindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{

    if ( !name )
	setName( "rangewindow" );
    resize( 460, 280 ); 
    setMinimumSize( QSize( 460, 280 ) );
    setMaximumSize( QSize( 460, 280 ) );
    setCaption( tr( "Global Features Window" ) );

    maxval=globalmax;
    minval=globalmin;
    Range=maxval-minval;
    lowglobalthresh=globalmin;
    highglobalthresh=globalmax;
    status=false;

    // need to connect these three checkboxes to meaningful things
    int cbsRowTop = 20;
    int cbsRowLeft = 50;
    int cbsRowDiff = 145;
    int cbsWidth = 130;
    int cbsHeight = 15;
    allGlobals = new QCheckBox( this, "allGlobalsCheckBox" );
    allGlobals->setGeometry( QRect ( cbsRowLeft, cbsRowTop, 
				     cbsWidth, cbsHeight ) );
    allGlobals->setText("Global Thresholds");
    // DO connect to both!... no, latter calls former.
    //    QObject::connect(allGlobals,SIGNAL(toggled(bool)),this,SLOT(signalAllGlobals(bool)));
    QObject::connect(allGlobals,SIGNAL(toggled(bool)),parent, SLOT(signalAllGlobals(bool)));

    allDifference = new QCheckBox( this, "allDifferenceCheckBox" );
    allDifference->setGeometry( QRect ( cbsRowLeft + cbsRowDiff, cbsRowTop, 
					cbsWidth, cbsHeight ) );
    allDifference->setText("Difference");
    // must also set global threshold extrema based on whether or not this is selected
    QObject::connect(allDifference,SIGNAL(toggled(bool)),this,SLOT(signalAllDifference(bool)));
    // first this, then the rest!
    QObject::connect(allDifference,SIGNAL(toggled(bool)),parent,SLOT(signalAllDifference(bool)));

    // removed per Gabor's suggestion in 2nd reopening of bug #146
    // don't go too crazy with commenting everything out
    //    allIntensity = new QCheckBox( this, "allIntensityCheckBox" );
    //    allIntensity->setGeometry( QRect ( cbsRowLeft + 2*cbsRowDiff, cbsRowTop,
    //				       cbsWidth, cbsHeight ) );
    //    allIntensity->setText("Intensity");
    //    QObject::connect(allIntensity,SIGNAL(toggled(bool)),parent,SLOT(signalAllIntensity(bool)));
    // for intensity, nothing to do in 'this' since it is orthogonal to difference

    lowtrheshframe = new QFrame( this, "lowtrheshframe" );
    lowtrheshframe->setGeometry( QRect( 15, 60, 430, 70 ) ); 
    lowtrheshframe->setFrameShape( QFrame::StyledPanel );
    lowtrheshframe->setFrameShadow( QFrame::Raised );

    lowthresh = new QLineEdit( lowtrheshframe, "lowthresh" );
    lowthresh->setGeometry( QRect( 185, 20, 140, 30 ) ); 
    char s[12];
    sprintf(s,"%10.7f",lowglobalthresh);
    lowthresh->setText(s);

    setlowbutton = new QPushButton( lowtrheshframe, "setlowbutton" );
    setlowbutton->setGeometry( QRect( 335, 15, 90, 40 ) ); 
    setlowbutton->setText( tr( "Set Value" ) );
    QObject::connect(setlowbutton,SIGNAL(clicked()),this,SLOT(updateLowthreshslider()));
    QObject::connect(setlowbutton,SIGNAL(clicked()),parent,SLOT(updateGlobalThreshs()));

    lowthreshslider = new QSlider( lowtrheshframe, "lowthreshslider" );
    lowthreshslider->setGeometry( QRect( 15, 30, 150, 20 ) ); 
    lowthreshslider->setOrientation( QSlider::Horizontal );
    lowthreshslider->setRange(0,1000000000);
    lowthreshslider->setValue(0);
    QObject::connect(lowthreshslider,SIGNAL(valueChanged(int)),this,SLOT(updateLowthresh()));
    QObject::connect(lowthreshslider,SIGNAL(valueChanged(int)),parent,SLOT(updateGlobalThreshs()));

    lowthershlabel = new QLabel( lowtrheshframe, "lowthershlabel" );
    lowthershlabel->setGeometry( QRect( 35, 10, 110, 20 ) ); 
    lowthershlabel->setText( tr( "Low Threshold" ) );

    highthreshframe = new QFrame( this, "highthreshframe" );
    highthreshframe->setGeometry( QRect( 15, 140, 430, 70 ) ); 
    highthreshframe->setFrameShape( QFrame::StyledPanel );
    highthreshframe->setFrameShadow( QFrame::Raised );

    highthresh = new QLineEdit( highthreshframe, "highthresh" );
    highthresh->setGeometry( QRect( 185, 20, 140, 30 ) ); 
    sprintf(s,"%10.7f",highglobalthresh);
    highthresh->setText(s);

    sethighbutton = new QPushButton( highthreshframe, "sethighbutton" );
    sethighbutton->setGeometry( QRect( 335, 15, 90, 40 ) ); 
    sethighbutton->setText( tr( "Set Value" ) );
    QObject::connect(sethighbutton,SIGNAL(clicked()),this,SLOT(updateHighthreshslider()));
    QObject::connect(sethighbutton,SIGNAL(clicked()),parent,SLOT(updateGlobalThreshs()));

    highthreshslider = new QSlider( highthreshframe, "highthreshslider" );
    highthreshslider->setGeometry( QRect( 15, 30, 150, 20 ) ); 
    highthreshslider->setOrientation( QSlider::Horizontal );
    highthreshslider->setRange(0,1000000000);
    highthreshslider->setValue(1000000000);
    QObject::connect(highthreshslider,SIGNAL(valueChanged(int)),this,SLOT(updateHighthresh()));
    QObject::connect(highthreshslider,SIGNAL(valueChanged(int)),parent,SLOT(updateGlobalThreshs()));

    highthreshlabel = new QLabel( highthreshframe, "highthreshlabel" );
    highthreshlabel->setGeometry( QRect( 35, 10, 110, 20 ) ); 
    highthreshlabel->setText( tr( "High Threshold" ) );

    Line2 = new QFrame( this, "Line2" );
    Line2->setGeometry( QRect( 0, 215, 460, 16 ) ); 
    Line2->setProperty( "frameShape", (int)QFrame::HLine );
    Line2->setFrameShadow( QFrame::Sunken );
    Line2->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    Line2->setProperty( "frameShape", (int)QFrame::HLine );

//     applybutton = new QPushButton( this, "applybutton" );
//     applybutton->setGeometry( QRect( 40, 230, 180, 40 ) ); 
//     applybutton->setText( tr( "Apply Global Thresholds" ) );
//     // a-ha!  clicked signal is connected to the PARENT's updateGlobalThreshs()... 
//     //      apply is not a toggle button so this makes sense
//     QObject::connect(applybutton,SIGNAL(clicked()),parent,SLOT(updateGlobalThreshs()));

    closebutton = new QPushButton( this, "closebutton" );
    //    closebutton->setGeometry( QRect( 300, 230, 100, 40 ) ); 
    closebutton->setGeometry( QRect( 180, 230, 100, 40 ) ); 
    closebutton->setText( tr( "Close" ) );
    QObject::connect(closebutton,SIGNAL(clicked()),this,SLOT(closeWindow()));

    signalAllGlobals( false );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
rangewindow::~rangewindow()
{
    // no need to delete child widgets, Qt does it all for us
}


/** update the text in lowthresh textbox based on the value of low threshold slider */
void rangewindow::updateLowthresh()
{
  int v = lowthreshslider->value();
  if(v > highthreshslider->value()) { // cap it
    lowthreshslider->setValue(highthreshslider->value());
    lowglobalthresh = highglobalthresh;
    lowthresh->setText(highthresh->text());
  } else {
    lowglobalthresh = (((double)v)/1000000000)*Range+minval;
    char s[12];
    sprintf(s,"%10.7f", lowglobalthresh);
    lowthresh->setText(s);
  }
}


void rangewindow::updateLowthreshslider()
{
  double fv; // floating-point value entered
  int v=0;   // integer position of slider
  bool ok=false;
  fv=lowthresh->text().toDouble(&ok); 
  if(ok) { // value in lowthresh text box parsed OK
    if((fv<minval) || (fv>maxval)) {
      QMessageBox::information(this,"SnarkDisplay","Error!\n"
			       "Entered threshold value must be within valid range");
      updateLowthresh();
      lowthresh->setFocus();
    } else {
      if(fv>0) {
	v=(int)(((fv-minval)/Range)*1000000000+0.5);
      } else {
	v=(int)(((fv-minval)/Range)*1000000000-0.5);
      }
      lowthreshslider->setValue(v);
    }
  } else {
    QMessageBox::information(this,"SnarkDisplay","Error!\n"
			     "Invalid Low Threshold value.");
  }
} // --rangewindow::updateLowthreshslider()


/** update the text in highthresh textbox based on the value of high threshold slider */
void rangewindow::updateHighthresh()
{
  int v = highthreshslider->value();
  if(v < lowthreshslider->value()) { // cap it
    highthreshslider->setValue(lowthreshslider->value());
    highglobalthresh = lowglobalthresh;
    highthresh->setText(highthresh->text());
  } else {
    highglobalthresh = (((double)v)/1000000000)*Range+minval;
    char s[12];
    sprintf(s,"%10.7f", highglobalthresh);
    highthresh->setText(s);
  }
} // --rangewindow::updateHighthresh()


void rangewindow::updateHighthreshslider()
{
  double fv;
  int v=0;
  bool ok=false;
  fv=highthresh->text().toDouble(&ok);
  if(ok) {
    if((fv>maxval) || (fv<minval)) {
      QMessageBox::information(this,"SnarkDisplay","Error!\n"
			       "Entered threshold value must be within valid range");
      updateHighthresh();
      highthresh->setFocus();
    } else {
      highglobalthresh=fv;
      //    status=true;
      //      if(fv>maxval) {
      //	maxval=fv;
      //	Range=maxval-minval;
      //      }
      if(fv>0) {
	v=(int)(((fv-minval)/Range)*1000000000+0.5);
      } else {
	v=(int)(((fv-minval)/Range)*1000000000-0.5);
      }
      //      if(v>highthreshslider->maxValue()) 
      //	highthreshslider->setMaxValue(v);
      highthreshslider->setValue(v);
      //    status=false;
    }
  } else {
    QMessageBox::information(this,"SnarkDisplay","Error!\n"
			     "Invalid High Threshold value.");
  }
} // --rangewindow::updateHighthreshslider()


/** en/disable fields in this window only 
    based on parameter indicating whether global thresholds should apply to all images
*/
void rangewindow::signalAllGlobals(bool on) {
  lowthresh->setEnabled(on);
  setlowbutton->setEnabled(on);
  lowthreshslider->setEnabled(on);
  lowthershlabel->setEnabled(on);
  highthreshframe->setEnabled(on);
  highthresh->setEnabled(on);
  sethighbutton->setEnabled(on);
  highthreshslider->setEnabled(on);
  highthreshlabel->setEnabled(on);
  // applybutton->setEnabled(on);
}

/** apply correct thresholds based on whether difference (for all) is selected or not*/
void rangewindow::signalAllDifference(bool on) {
  if(on) {
    maxval=globaldiffmax;
    minval=globaldiffmin;
  } else {
    maxval=globalmax;
    minval=globalmin;
  }
  Range=maxval-minval;
  lowglobalthresh=minval;
  highglobalthresh=maxval;
  updateLowthresh();
  updateHighthresh();
}  // --rangewindow::signalAllDifference()



void rangewindow::closeWindow() {
  if(allGlobals->isOn()) allGlobals->toggle(); // triggers the appropriate signal-slot mechanism
  if(allDifference->isOn()) allDifference->toggle();
  accept();
}


void rangewindow::closeEvent(QCloseEvent* e) {
  if(allGlobals->isOn()) allGlobals->toggle(); // triggers the appropriate signal-slot mechanism
  if(allDifference->isOn()) allDifference->toggle();
  e->accept();
}
