/** @file sd_line_t.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

/** Deniz, July 2004
    classes relating to each 'line' to be plotted
    includes conversion mechanisms from real coords to window coords
    handles 'actual' plotting too
*/

#include "sd_line_t.hpp"
#include <qpainter.h>


namespace sd_line_t {
  // number of pixels left blank as buffer outside plot area. 
  // X is both left and right, Y is both top and bot.
  //  MUST be natural number and not too big.
  int WIDTH  = 980;
  int HEIGHT = 720;
  int XWINPAD = 90;
  //  int XWINPAD = 0;
  int YWINPAD = 110;
  //  int YWINPAD = 0;
  int YAXISNOTCHLABELOFFSET = 38; // from y axis
  int YAXISLABELOFFSET = 58;
//    // in addition to the padding above...
  double Y_HALF_FUDGE_FACTOR = 0.03; // like 0.03
//    // fudge real Y-axis extrema on each side by this much.
//    // MUST be nonnegative.  can set to 0 if you don't want it to.
  int X_HALF_FUDGE_INCREMENT = 1; // like 1
//    // how much to fudge the real x-axis on both side: MUST be natural number
  
  void drawBoundary(QPainter& P) { // boundary JUST OUTSIDE the actual paint area
    // if PADDINGs are 0, should seem to do nothing
    int LEFT=  sd_line_t::XWINPAD - 1;
    int RIGHT= sd_line_t::WIDTH - sd_line_t::XWINPAD;
    int TOP=   sd_line_t::YWINPAD - 1;
    int BOT=   sd_line_t::HEIGHT - sd_line_t::YWINPAD;
    P.setPen( Qt::black );
    P.drawLine(LEFT , TOP, RIGHT, TOP); // (x1,y1,x2,y2)  >>inclusive<<
    P.drawLine(RIGHT, TOP, RIGHT, BOT);
    P.drawLine(RIGHT, BOT, LEFT , BOT);
    P.drawLine(LEFT , BOT, LEFT , TOP);
  } // --drawBoundary() in namespace sd_line_t
  
  /** don't wanna define it in QPainter to respect Qt library *
   * puts a big ugly smudge of paint at (cx, cy) without changing Pen */
  void drawBigPoint(QPainter& P, int cx, int cy) {
    int x, y;
    for(x = cx-1; x <= cx+1; x++) {
      for(y = cy-1; y <= cy+1; y++) {
	P.drawPoint(x, y);
      }
    }
    P.drawPoint(cx-2,cy-2);
    P.drawPoint(cx-2,cy+2);
    P.drawPoint(cx+2,cy-2);
    P.drawPoint(cx+2,cy+2);
    P.drawPoint(cx,cy-2);
    P.drawPoint(cx,cy+2);
    P.drawPoint(cx-2,cy);
    P.drawPoint(cx+2,cy);
  } // --drawBigPoint() in namespace sd_line_t

}; // -- namespace sd_line_t, for now
