/** @file projection.cpp
    @package snark14Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "projection.h"
#include "variables.h"
#include "misc.h"

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qstringlist.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>
#include <qtabwidget.h>

/** populates the tab inside the pseudo tab corresponding to create set 1 *
    @author DS
*/
void projectionwindow::populate_idheader() {
  idheaderlabel = new QLabel( idheaderWidget, "idheaderlabel" );
  idheaderlabel->setGeometry( QRect( 20, 30, 90, 20 ) ); 
  idheaderlabel->setText( tr( "ID HEADER" ) );
  idheader = new QLineEdit( idheaderWidget, "idheader" );
  idheader->setGeometry( QRect( 120, 20, 500, 28 ) ); 
}

bool projectionwindow::check_idheader(QString& errs) {
  errs = "";
  QString idText = idheader->text();
  QString simpleIdText = idText.simplifyWhiteSpace();
  if(simpleIdText.isEmpty()) {
    pseudoCommandsWidget->setCurrentPage(0);
    idheader->setFocus();
    errs = "Make sure that id header is not blank.";
    return false;
  } else {
    return true;
  }
}

QString projectionwindow::get_idheader() {
  QString idOut;
  idOut.append(idheader->text());
  return idOut;
}

// set 6
void projectionwindow::populate_geometry() {
    geometrylabel = new QLabel( geometryWidget, "geometrylabel" );
    geometrylabel->setGeometry( QRect( 10, 20, 100, 20 ) ); 
    geometrylabel->setText( tr( "GEOMETRY" ) );

    geometrybuttongroup = new QButtonGroup( geometryWidget, "geometrybuttongroup" );
    geometrybuttongroup->setGeometry( QRect( 120, 10, 570, 160 ) ); 
    geometrybuttongroup->setTitle( tr( "" ) );
    geometrybuttongroup->setEnabled(true);

    parallelbutton = new QRadioButton( geometrybuttongroup, "parallelbutton" );
    parallelbutton->setGeometry( QRect( 10, 20, 100, 24 ) ); 
    parallelbutton->setText( tr( "PARALLEL" ) );
    parallelbutton->setChecked(true);
    QObject::connect(parallelbutton,SIGNAL(clicked()),this,SLOT(activateParallelFields()));

    parallelbuttongroup = new QButtonGroup( geometrybuttongroup, "parallelbuttongroup" );
    parallelbuttongroup->setGeometry( QRect( 130, 10, 120, 70 ) ); 
    parallelbuttongroup->setTitle( tr( "" ) );
    parallelbuttongroup->setEnabled(true);

    uniformbutton = new QRadioButton( parallelbuttongroup, "uniformbutton" );
    uniformbutton->setGeometry( QRect( 10, 10, 100, 24 ) ); 
    uniformbutton->setText( tr( "UNIFORM" ) ); 
    uniformbutton->setChecked(true);

    variablebutton = new QRadioButton( parallelbuttongroup, "variablebutton" );
    variablebutton->setGeometry( QRect( 10, 40, 100, 24 ) ); 
    variablebutton->setText( tr( "VARIABLE" ) );

    parallelbuttongroup2 = new QButtonGroup( geometrybuttongroup, "parallelbuttongroup2" );
    parallelbuttongroup2->setGeometry( QRect( 260, 10, 90, 70 ) ); 
    parallelbuttongroup2->setTitle( tr( "" ) ); 
    parallelbuttongroup2->setEnabled(true);

    stripbutton = new QRadioButton( parallelbuttongroup2, "stripbutton" );
    stripbutton->setGeometry( QRect( 10, 10, 70, 24 ) ); 
    stripbutton->setText( tr( "STRIP" ) );

    linebutton = new QRadioButton( parallelbuttongroup2, "linebutton" );
    linebutton->setGeometry( QRect( 10, 40, 70, 24 ) ); 
    linebutton->setText( tr( "LINE" ) );
    linebutton->setChecked(true);

    divergentbutton = new QRadioButton( geometrybuttongroup, "divergentbutton" );
    divergentbutton->setGeometry( QRect( 10, 100, 120, 24 ) ); 
    divergentbutton->setText( tr( "DIVERGENT" ) );    
    QObject::connect(divergentbutton,SIGNAL(clicked()),this,SLOT(activateDivergentFields()));

    divergentbuttongroup = new QButtonGroup( geometrybuttongroup, "divergentbuttongroup" );
    divergentbuttongroup->setGeometry( QRect( 130, 80, 120, 70 ) ); 
    divergentbuttongroup->setTitle( tr( "" ) ); 
    divergentbuttongroup->setEnabled(false);

    arcbutton = new QRadioButton( divergentbuttongroup, "arcbutton" );
    arcbutton->setGeometry( QRect( 10, 10, 60, 24 ) ); 
    arcbutton->setText( tr( "ARC" ) );

    tangentbutton = new QRadioButton( divergentbuttongroup, "tangentbutton" );
    tangentbutton->setGeometry( QRect( 10, 40, 100, 20 ) ); 
    tangentbutton->setText( tr( "TANGENT" ) );

    sourcetoorigin = new QLineEdit( geometrybuttongroup, "sourcetoorigin" );
    sourcetoorigin->setGeometry( QRect( 260, 100, 140, 28 ) ); 
    sourcetoorigin->setEnabled(false);

    sourcetooriginlabel = new QLabel( geometrybuttongroup, "sourcetooriginlabel" );
    sourcetooriginlabel->setGeometry( QRect( 270, 80, 120, 20 ) ); 
    sourcetooriginlabel->setText( tr( "source_to_origin" ) );
    sourcetooriginlabel->setEnabled(false);

    sourcetodetector = new QLineEdit( geometrybuttongroup, "sourcetodetector" );
    sourcetodetector->setGeometry( QRect( 410, 100, 140, 28 ) ); 
    sourcetodetector->setEnabled(false);

    sourcetodetectorlabel = new QLabel( geometrybuttongroup, "sourcetodetectorlabel" );
    sourcetodetectorlabel->setGeometry( QRect( 410, 80, 140, 20 ) ); 
    sourcetodetectorlabel->setText( tr( "source_to_detector" ) );
    sourcetodetectorlabel->setEnabled(false);

    raysumslabel = new QLabel( geometryWidget, "raysumslabel" );
    raysumslabel->setGeometry( QRect( 10, 180, 90, 20 ) ); 
    raysumslabel->setText( tr( "RAYSUMS" ) );

    raysumsbuttongroup = new QButtonGroup( geometryWidget, "raysumsbuttongroup" );
    raysumsbuttongroup->setGeometry( QRect( 120, 180, 570, 120 ) ); 
    raysumsbuttongroup->setTitle( tr( "" ) );
    //    raysumsbuttongroup->setEnabled(false);

    userbutton = new QRadioButton( raysumsbuttongroup, "userbutton" );
    userbutton->setGeometry( QRect( 10, 40, 70, 24 ) ); 
    userbutton->setText( tr( "USER" ) ); 
    userbutton->setChecked(true);
    QObject::connect(userbutton,SIGNAL(clicked()),this,SLOT(activateUserFields()));

    userrays = new QLineEdit( raysumsbuttongroup, "userrays" );
    userrays->setGeometry( QRect( 130, 30, 90, 28 ) ); 
    userrays->setEnabled(true);

    userrayslabel = new QLabel( raysumsbuttongroup, "userrayslabel" );
    userrayslabel->setGeometry( QRect( 140, 10, 70, 20 ) ); 
    userrayslabel->setText( tr( "user_rays" ) );
    userrayslabel->setEnabled(true);

    programbutton = new QRadioButton( raysumsbuttongroup, "programbutton" );
    programbutton->setGeometry( QRect( 10, 90, 110, 24 ) ); 
    programbutton->setText( tr( "PROGRAM" ) );
    QObject::connect(programbutton,SIGNAL(clicked()),this,SLOT(activateProgramFields()));

    raysumsnelem = new QLineEdit( raysumsbuttongroup, "raysumsnelem" );
    raysumsnelem->setGeometry( QRect( 130, 80, 90, 28 ) ); 
    raysumsnelem->setEnabled(false);

    raysumsnelemlabel = new QLabel( raysumsbuttongroup, "raysumsnelemlabel" );
    raysumsnelemlabel->setGeometry( QRect( 150, 60, 50, 20 ) ); 
    raysumsnelemlabel->setText( tr( "nelem" ) );
    raysumsnelemlabel->setEnabled(false);

    raysumspixelsize = new QLineEdit( raysumsbuttongroup, "raysumspixelsize" );
    raysumspixelsize->setGeometry( QRect( 240, 80, 90, 28 ) ); 
    raysumspixelsize->setEnabled(false);

    raysumspixelsizelabel = new QLabel( raysumsbuttongroup, "raysumspixelsizelabel" );
    raysumspixelsizelabel->setGeometry( QRect( 250, 60, 70, 20 ) ); 
    raysumspixelsizelabel->setText( tr( "pixel_size" ) );
    raysumspixelsizelabel->setEnabled(false);

    detectorspacing = new QLineEdit( raysumsbuttongroup, "detectorspacing" );
    detectorspacing->setGeometry( QRect( 370, 50, 140, 28 ) ); 

    detectorspacinglabel = new QLabel( raysumsbuttongroup, "detectorspacinglabel" );
    detectorspacinglabel->setGeometry( QRect( 380, 30, 130, 20 ) ); 
    detectorspacinglabel->setText( tr( "detector_spacing" ) );

    angleslabel = new QLabel( geometryWidget, "angleslabel" );
    angleslabel->setGeometry( QRect( 8, 310, 70, 20 ) ); 
    angleslabel->setText( tr( "ANGLES" ) );

    anglesbuttongroup = new QButtonGroup( geometryWidget, "anglesbuttongroup" );
    anglesbuttongroup->setGeometry( QRect( 120, 310, 570, 100 ) ); 
    anglesbuttongroup->setTitle( tr( "" ) );

    prjnum = new QLineEdit( anglesbuttongroup, "prjnum" );
    prjnum->setGeometry( QRect( 17, 40, 120, 28 ) ); 

    prjnumlabel = new QLabel( anglesbuttongroup, "prjnumlabel" );
    prjnumlabel->setGeometry( QRect( 50, 20, 50, 20 ) ); 
    prjnumlabel->setText( tr( "prjnum" ) );

    linobutton = new QRadioButton( anglesbuttongroup, "linobutton" );
    linobutton->setGeometry( QRect( 160, 10, 60, 24 ) ); 
    linobutton->setText( tr( "LINO" ) );
    QObject::connect(linobutton,SIGNAL(clicked()),this,SLOT(deactivateEqualSpacingFields()));

    equalspacingbutton = new QRadioButton( anglesbuttongroup, "equalspacingbutton" );
    equalspacingbutton->setGeometry( QRect( 160, 40, 160, 24 ) ); 
    equalspacingbutton->setText( tr( "EQUAL SPACING" ) );
    QObject::connect(equalspacingbutton,SIGNAL(clicked()),this,SLOT(activateEqualSpacingFields()));

    nonebutton = new QRadioButton( anglesbuttongroup, "nonebutton" );
    nonebutton->setGeometry( QRect( 160, 70, 70, 24 ) ); 
    nonebutton->setText( tr( "NONE" ) );
    nonebutton->setChecked(true);
    QObject::connect(nonebutton,SIGNAL(clicked()),this,SLOT(deactivateEqualSpacingFields()));

    firstangle = new QLineEdit( anglesbuttongroup, "firstangle" );
    firstangle->setGeometry( QRect( 340, 40, 80, 28 ) ); 
    firstangle->setEnabled(false);

    firstanglelabel = new QLabel( anglesbuttongroup, "firstanglelabel" );
    firstanglelabel->setGeometry( QRect( 345, 20, 80, 20 ) ); 
    firstanglelabel->setText( tr( "first_angle" ) ); 
    firstanglelabel->setEnabled(false);

    lastangle = new QLineEdit( anglesbuttongroup, "lastangle" );
    lastangle->setGeometry( QRect( 440, 40, 80, 28 ) );
    lastangle->setEnabled(false); 

    lastanglelabel = new QLabel( anglesbuttongroup, "lastanglelabel" );
    lastanglelabel->setGeometry( QRect( 445, 20, 80, 20 ) ); 
    lastanglelabel->setText( tr( "last_angle" ) );
    lastanglelabel->setEnabled(false);

    angles = new QLineEdit( geometryWidget, "angles" );
    angles->setGeometry( QRect( 120, 415, 570, 105 ) ); 
    angles->setEnabled(true);

}

// bool projectionwindow::check_geometry(QString& errs) {
//   errs = "";
//   return true;
// }

/** Returns a string for set 6 of CREATE input sequence
i.e., geometry
@param void
@return projection geometry specification
@author Bruno M. Carvalho
@version 1.0 */
//QString createwindow::getOutput6()
QString projectionwindow::get_geometry() {

  QString output;

  output.sprintf("GEOMETRY\n");
  if(parallelbutton->isChecked()) {
    output.append("PARALLEL ");
    if(uniformbutton->isChecked()) {
      output.append("UNIFORM ");
    }
    else
      if(variablebutton->isChecked()) {
        output.append("VARIABLE ");
      }
    if(stripbutton->isChecked()) {
      output.append("STRIP ");
    }
    else
      if(linebutton->isChecked()) {
        output.append("LINE ");
      }
  } // -- if parallel
  else
    if(divergentbutton->isChecked()) {
      output.append("DIVERGENT ");
      if(arcbutton->isChecked()) {
        output.append("ARC ");
      }
      else
        if(tangentbutton->isChecked()) {
          output.append("TANGENT ");
        }
      output+=sourcetoorigin->text();
      output.append(" ");
      output+=sourcetodetector->text();
    }
  output.append("\n");
  output.append("RAYS ");
  if(userbutton->isChecked()) {
    output.append("USER ");
    output+=userrays->text();
    output.append(" ");
  }
  else
    if(programbutton->isChecked()) {
      output.append("PROGRAM ");
      output+=raysumsnelem->text();
      output.append(" ");
      output+=raysumspixelsize->text();
      output.append(" ");
    }
  output+=detectorspacing->text();
  output.append("\n");
  output.append("ANGLES ");
  output+=prjnum->text();
  output.append(" ");
  if(linobutton->isChecked()) {
    output.append("LINO ");
    //output.append("\n");
  }
  else
    if(equalspacingbutton->isChecked()) {
      output.append("EQUAL SPACING ");
      output.append("\n");
      output+=firstangle->text();
      output.append(" ");
      output+=lastangle->text();
      //output.append("\n");
    }
    else {
      output.append("\n");
      output+=angles->text();
      //output.append("\n");
    }
  return output;
}


// set 7
void projectionwindow::populate_measurement() {
  measurementlabel = new QLabel( measurementWidget, "measurementlabel" );
  measurementlabel->setGeometry( QRect( 10, 20, 130, 20 ) ); 
  measurementlabel->setText( tr( "MEASUREMENT" ) );
  
  measurementbuttongroup = new QButtonGroup( measurementWidget, "measurementbuttongroup" );
  measurementbuttongroup->setGeometry( QRect( 20, 50, 710, 410 ) ); 
  measurementbuttongroup->setTitle( tr( "" ) );
  measurementbuttongroup->setEnabled( true );
  
  perfectbutton = new QRadioButton( measurementbuttongroup, "perfectbutton" );
  perfectbutton->setGeometry( QRect( 10, 10, 100, 24 ) ); 
  perfectbutton->setText( tr( "PERFECT" ) ); 
  perfectbutton->setChecked(true);
  QObject::connect(perfectbutton,SIGNAL(clicked()),this,SLOT(deactivateNoisyFields()));
  
  noisybutton = new QRadioButton( measurementbuttongroup, "noisybutton" );
  noisybutton->setGeometry( QRect( 10, 40, 70, 24 ) ); 
  noisybutton->setText( tr( "NOISY" ) );
  QObject::connect(noisybutton,SIGNAL(clicked()),this,SLOT(activateNoisyFields()));
  
  noisyframe = new QFrame( measurementbuttongroup, "noisyframe" );
  noisyframe->setGeometry( QRect( 90, 40, 570, 250 ) ); 
  noisyframe->setFrameStyle( 49 );
  noisyframe->setEnabled(false);
  
  quantumbutton = new QCheckBox( noisyframe, "quantumbutton" );
  quantumbutton->setGeometry( QRect( 20, 30, 110, 24 ) ); 
  quantumbutton->setText( tr( "QUANTUM" ) );
  quantumbutton->setEnabled(false);
  QObject::connect(quantumbutton,SIGNAL(clicked()),this,SLOT(activateQuantumFields()));
  
  quanmn = new QLineEdit( noisyframe, "quanmn" );
  quanmn->setGeometry( QRect( 180, 30, 80, 28 ) ); 
  quanmn->setEnabled(false);
  
  quanmnlabel = new QLabel( noisyframe, "quanmnlabel" );
  quanmnlabel->setGeometry( QRect( 190, 10, 60, 20 ) ); 
  quanmnlabel->setText( tr( "quanmn" ) ); 
  quanmnlabel->setEnabled(false);
  
  quancm = new QLineEdit( noisyframe, "quancm" );
  quancm->setGeometry( QRect( 270, 30, 80, 28 ) ); 
  quancm->setEnabled(false);
  
  quancmlabel = new QLabel( noisyframe, "quancmlabel" );
  quancmlabel->setGeometry( QRect( 280, 10, 60, 20 ) ); 
  quancmlabel->setText( tr( "quancm" ) );
  quancmlabel->setEnabled(false);
  
  calibrationlabel = new QLabel( noisyframe, "calibrationlabel" );
  calibrationlabel->setGeometry( QRect( 360, 30, 110, 20 ) ); 
  calibrationlabel->setText( tr( "CALIBRATION" ) ); 
  calibrationlabel->setEnabled(false);
  
  quanin = new QComboBox( noisyframe, "quanin" );
  quanin->setGeometry( QRect( 480, 30, 80, 28 ) );
  quanin->insertItem("1",-1);
  quanin->insertItem("2",-1);
  quanin->insertItem("3",-1);
  quanin->insertItem("4",-1);
  quanin->setEnabled(false); 
  
  quaninlabel = new QLabel( noisyframe, "quaninlabel" );
  quaninlabel->setGeometry( QRect( 490, 10, 50, 20 ) ); 
  quaninlabel->setText( tr( "quanin" ) );
  quaninlabel->setEnabled(false);
  
  scatterbutton = new QCheckBox( noisyframe, "scatterbutton" );
  scatterbutton->setGeometry( QRect( 20, 90, 100, 24 ) ); 
  scatterbutton->setText( tr( "SCATTER" ) );
  QObject::connect(scatterbutton,SIGNAL(clicked()),this,SLOT(activateScatterFields()));
  
  sctnpk = new QLineEdit( noisyframe, "sctnpk" );
  sctnpk->setGeometry( QRect( 180, 90, 80, 28 ) );
  sctnpk->setEnabled(false); 
  
  sctnpklabel = new QLabel( noisyframe, "sctnpklabel" );
  sctnpklabel->setGeometry( QRect( 190, 70, 50, 20 ) ); 
  sctnpklabel->setText( tr( "sctnpk" ) ); 
  sctnpklabel->setEnabled(false);
  
  sctnwd = new QLineEdit( noisyframe, "sctnwd" );
  sctnwd->setGeometry( QRect( 270, 90, 80, 28 ) ); 
  sctnwd->setEnabled(false);

  sctnwdlabel = new QLabel( noisyframe, "sctnwdlabel" );
  sctnwdlabel->setGeometry( QRect( 280, 70, 60, 20 ) ); 
  sctnwdlabel->setText( tr( "sctnwd" ) );  
  sctnwdlabel->setEnabled(false);
  
  additivebutton = new QCheckBox( noisyframe, "additivebutton" );
  additivebutton->setGeometry( QRect( 20, 150, 90, 24 ) ); 
  additivebutton->setText( tr( "ADDITIVE" ) );
  additivebutton->setChecked(false);
  QObject::connect(additivebutton,SIGNAL(clicked()),this,SLOT(activateAdditiveFields()));
  
  addnmn = new QLineEdit( noisyframe, "addnmn" );
  addnmn->setGeometry( QRect( 180, 150, 80, 28 ) ); 
  addnmn->setEnabled(false);
  
  addnmnlabel = new QLabel( noisyframe, "addnmnlabel" );
  addnmnlabel->setGeometry( QRect( 190, 130, 60, 20 ) ); 
  addnmnlabel->setText( tr( "addnmn" ) ); 
  addnmnlabel->setEnabled(false);
  
  addnsd = new QLineEdit( noisyframe, "addnsd" );
  addnsd->setGeometry( QRect( 270, 150, 80, 28 ) ); 
  addnsd->setEnabled(false);
  
  addnsdlabel = new QLabel( noisyframe, "addnsdlabel" );
  addnsdlabel->setGeometry( QRect( 280, 130, 60, 20 ) ); 
  addnsdlabel->setText( tr( "addnsd" ) );
  addnsdlabel->setEnabled(false);
  
  multiplicativebutton = new QCheckBox( noisyframe, "multiplicativebutton" );
  multiplicativebutton->setGeometry( QRect( 20, 210, 150, 24 ) ); 
  multiplicativebutton->setText( tr( "MULTIPLICATIVE" ) );
  QObject::connect(multiplicativebutton,SIGNAL(clicked()),this,SLOT(activateMultiplicativeFields()));
  
  multnmn = new QLineEdit( noisyframe, "multnmn" );
  multnmn->setGeometry( QRect( 180, 210, 80, 28 ) );
  multnmn->setEnabled(false);
  
  multnmnlabel = new QLabel( noisyframe, "multnmnlabel" );
  multnmnlabel->setGeometry( QRect( 190, 190, 70, 20 ) ); 
  multnmnlabel->setText( tr( "multnmn" ) );
  multnmnlabel->setEnabled(false);
  
  multnsd = new QLineEdit( noisyframe, "multnsd" );
  multnsd->setGeometry( QRect( 270, 210, 80, 28 ) ); 
  multnsd->setEnabled(false);
  
  multnsdlabel = new QLabel( noisyframe, "multnsdlabel" );
  multnsdlabel->setGeometry( QRect( 280, 190, 70, 20 ) ); 
  multnsdlabel->setText( tr( "multnsd" ) ); 
  multnsdlabel->setEnabled(false);
  
  seedlabel = new QLabel( measurementbuttongroup, "seedlabel" );
  seedlabel->setGeometry( QRect( 10, 310, 110, 20 ) ); 
  seedlabel->setText( tr( "SEED" ) ); 
  seedlabel->setEnabled(false);
  
  seed = new QLineEdit( measurementbuttongroup, "seed" );
  seed->setGeometry( QRect( 140, 310, 70, 28 ) );  
  seed->setEnabled(false);
  
  seedlabel1 = new QLabel( measurementbuttongroup, "seedlabel1" );
  seedlabel1->setGeometry( QRect( 155, 290, 70, 20 ) ); 
  seedlabel1->setText( tr( "seed" ) );  
  seedlabel1->setEnabled(false);
  
  backgroundlabel = new QLabel( measurementbuttongroup, "backgroundlabel" );
  backgroundlabel->setGeometry( QRect( 10, 370, 120, 20 ) ); 
  backgroundlabel->setText( tr( "BACKGROUND" ) );
  
  backgr1 = new QLineEdit( measurementbuttongroup, "backgr1" );
  backgr1->setGeometry( QRect( 140, 370, 70, 28 ) ); 

  backgr1label = new QLabel( measurementbuttongroup, "backgr1label" );
  backgr1label->setGeometry( QRect( 145, 350, 70, 20 ) ); 
  backgr1label->setText( tr( "backgr(1)" ) ); 

}

// bool projectionwindow::check_measurement(QString& errs) {
//   errs = "";
//   return true;
// }


/** Returns a string for set 7 of CREATE input sequence
    i.e., measurement
@param void
@return measurement statistics
@author Bruno M. Carvalho
@version 1.0 */
// QString createwindow::getOutput7()
QString projectionwindow::get_measurement() {
  QString output,t;

  output.sprintf("MEASUREMENT ");
  if(perfectbutton->isChecked()) {
    output.append("PERFECT ");
    output.append("\n");
  }
  else
    if(noisybutton->isChecked()) {
      output.append("NOISY ");
      output.append("\n");
      if(quantumbutton->isChecked()) {
	output.append("QUANTUM ");
	output+=quanmn->text();
	output.append(" ");
	output+=quancm->text();
	output.append(" ");
        t.sprintf("CALIBRATION %d \n",quanin->currentItem()+1);
        output.append(t);
        output.append("SEED ");
	output+=seed->text();
        output.append("\n");
      }
      else 
	if(scatterbutton->isChecked()) {
  	  output.append("SCATTER ");
	  output+=sctnpk->text();
          output.append(" ");
	  output+=sctnwd->text();
	  output.append("\n");
	}
	else 
	  if(additivebutton->isChecked()) {
  	    output.append("ADDITIVE ");
	    output+=addnmn->text();
            output.append(" ");
	    output+=addnsd->text();
	    output.append("\n");
            output.append("SEED ");
	    output+=seed->text();
	    output.append("\n");
          }
	  else 
	    if(multiplicativebutton->isChecked()) {
    	      output.append("MULTIPLICATIVE ");
	      output+=multnmn->text();
              output.append(" ");
	      output+=multnsd->text();
	      output.append("\n");
              output.append("SEED ");
	      output+=seed->text();
	      output.append("\n");
	    }
    }
  output.append("BACKGROUND ");
  output+=backgr1->text();
  output.append(" ");
  // assume single energy with NERGY = 1 in pseudo projections
//   if(polychromaticbutton->isChecked()) {
//     output+=backgr2->text();
//     output.append(" ");
//     output+=backgr3->text();
//     output.append(" ");
//     output+=backgr4->text();
//     output.append(" ");
//     output+=backgr5->text();
//     output.append(" ");
//     output+=backgr6->text();
//     output.append(" ");
//     output+=backgr7->text();
//   }
  //output.append("\n");
  return output;
}

/** 
 *  Constructs a projectionwindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho / DS
@version 1.0 */
projectionwindow::projectionwindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "projectionwindow" );
    resize( 750, 630 );
    setMinimumSize( QSize( 730, 630 ) );
    setMaximumSize( QSize( 730, 630 ) );
    setCaption( tr( "Projection Window" ) );

    tabwidget1 = new QTabWidget( this, "tabwidget1" );
    tabwidget1->setGeometry( QRect( 0, 0, 750, 550 ) ); 
    tabwidget1->setMinimumSize( QSize( 750, 550 ) );
    tabwidget1->setMaximumSize( QSize( 750, 550 ) );
    tabwidget1->setTabPosition( QTabWidget::Top );
    tabwidget1->setTabShape( QTabWidget::Triangular );

    /* Create tab 1 -- projection type widget */

    projTypeWidget = new QWidget( tabwidget1, "projection type" );

    projTypeLabel = new QLabel( projTypeWidget, "projtypelabel" );
    projTypeLabel->setGeometry( QRect( 10, 20, 120, 20 ) ); 
    projTypeLabel->setText( tr( "Projection Type" ) );

    projectionbuttongroup = new QButtonGroup( projTypeWidget, "projectionbuttongroup" );
    projectionbuttongroup->setGeometry( QRect( 25, 40, 690, 480 ) ); 
    //    projectionbuttongroup->setTitle( tr( "Projection Type" ) );
    projectionbuttongroup->setAlignment( int( QButtonGroup::AlignHCenter ) );

    oldbutton = new QRadioButton( projectionbuttongroup, "oldbutton" );
    oldbutton->setGeometry( QRect( 10, 20, 95, 24 ) ); 
    oldbutton->setText( tr( "OLD" ) );
    oldbutton->toggle();
    QObject::connect(oldbutton,SIGNAL(clicked()),this,SLOT(deactivateFields()));
    
    tabwidget1->insertTab( projTypeWidget, "Projection Type" );

    /** Create tab 2 -- PseudoProjection Commands */
    pseudoCommandsWidget = new QTabWidget( tabwidget1, "pseudo commands" );

    pseudobutton = new QRadioButton( projectionbuttongroup, "pseudobutton" );
    pseudobutton->setGeometry( QRect( 10, 60, 95, 24 ) ); 
    pseudobutton->setText( tr( "PSEUDO" ) );
    QObject::connect(pseudobutton,SIGNAL(clicked()),this,SLOT(activatePseudoFields()));

    tabwidget1->insertTab( pseudoCommandsWidget, "Pseudo Commands" );
    tabwidget1->setTabEnabled(pseudoCommandsWidget, false);

    // Make the create-like command sets 1,6,7 under the pseudo commands tab!

    // set 1: idheader
    idheaderWidget = new QWidget( pseudoCommandsWidget, "ID Header" );
    populate_idheader();
    pseudoCommandsWidget->insertTab( idheaderWidget, "ID Header" );

    // set 6: geometry
    geometryWidget = new QWidget( pseudoCommandsWidget, "Geometry" );
    populate_geometry();
    pseudoCommandsWidget->insertTab( geometryWidget, "Geometry" );

    // set 7: measurement
    measurementWidget = new QWidget( pseudoCommandsWidget, "Measurement" );
    populate_measurement();
    pseudoCommandsWidget->insertTab( measurementWidget, "Measurement" );
    
    
    /** Create the options to real for beam hardening correction */
    realbutton = new QRadioButton( projectionbuttongroup, "realbutton" );
    realbutton->setGeometry( QRect( 10, 100, 95, 24 ) ); 
    realbutton->setText( tr( "REAL" ) );
    QObject::connect(realbutton,SIGNAL(clicked()),this,SLOT(deactivateFields()));
    
    bhcbutton = new QCheckBox( projectionbuttongroup, "bhcbutton");
    bhcbutton->setGeometry ( QRect( 130, 90, 500, 60) );
    bhcbutton->setText( tr( "BEAM HARDENING CORRECTION\n(Fill in the window below.)" ) );
    bhcbutton->setEnabled(false);
    QObject::connect(realbutton,SIGNAL(clicked()),this,SLOT(activateRealOptions()));

    bhcwindow = new QMultiLineEdit( projectionbuttongroup, "bhcwindow");
    bhcwindow->setGeometry( QRect( 130, 155, 500, 300 ) );
    bhcwindow->setEnabled(false);
    QObject::connect(bhcbutton,SIGNAL(clicked()),this,SLOT(activateRealOptions()));

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 100, 560, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 510, 560, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/**  
 *  Destroys the object and frees any allocated resources
 */
projectionwindow::~projectionwindow()
{
    // no need to delete child widgets, Qt does it all for us
}

/** Deactivates pseudo and rebin fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::deactivateFields()
{
  if(oldbutton->isChecked() || realbutton->isChecked()) {
    //    tabwidget1->deleteTab( 1 );
    tabwidget1->setTabEnabled(pseudoCommandsWidget, false);
  }
  if(oldbutton->isChecked() || pseudobutton->isChecked()) {
    bhcwindow->setEnabled(false);
    bhcbutton->setEnabled(false);
    //bhcbutton->setOn(false);
  }
  if( ! bhcbutton->isChecked() ) {
    bhcwindow->setEnabled(false);
  }
}

/** Activates real options
@param void
@author Joanna Klukowska
@version 1.0 */
void projectionwindow::activateRealOptions()
{
  deactivateFields();
  if(realbutton->isChecked()) {
    bhcbutton->setEnabled(true);
    if (bhcbutton->isChecked() ) {
        bhcwindow->setEnabled(true);
    }
  }
  else {
  }
}


/** Activates pseudo fields "recursively"
@param void
@author Bruno M. Carvalho / DS
@version 1.0 */
void projectionwindow::activatePseudoFields()
{
  deactivateFields();
  if(pseudobutton->isChecked()) {
    //bhcwindow->setEnabled(false);
    //bhcbutton->setEnabled(false);
    tabwidget1->setTabEnabled(pseudoCommandsWidget, true);
    tabwidget1->setCurrentPage( 1 );
    pseudoCommandsWidget->setTabEnabled(idheaderWidget, true);
    pseudoCommandsWidget->setTabEnabled(geometryWidget, true);
    pseudoCommandsWidget->setTabEnabled(measurementWidget, true);
  }
  else {
    //    tabwidget1->setTabEnabled(pseudoCommandsWidget, false);
    //    pseudoCommandsWidget->setEnabled(FALSE);
  }
}


/** Activates parallel fields and deactivates divergent fields
@param void
@author Bruno M. Carvalho (copied over by DS from create.cpp)
@version 1.0 */
void projectionwindow::activateParallelFields()
{
  if(parallelbutton->isChecked()) {
    parallelbuttongroup->setEnabled(true);
    parallelbuttongroup2->setEnabled(true);
    divergentbuttongroup->setEnabled(false);
    sourcetoorigin->setEnabled(false);
    sourcetooriginlabel->setEnabled(false);
    sourcetodetector->setEnabled(false);
    sourcetodetectorlabel->setEnabled(false);
    //    activateOKButton();
  }
  else {
    parallelbuttongroup->setEnabled(false);
    parallelbuttongroup2->setEnabled(false);
    //    activateOKButton();
  }
}

/** Activates divergent fields and deactivates parallel fields
@param void
@author Bruno M. Carvalho (copied over by DS from create.cpp)
@version 1.0 */
void projectionwindow::activateDivergentFields()
{
  if(divergentbutton->isChecked()) {
    divergentbuttongroup->setEnabled(true);
    sourcetoorigin->setEnabled(true);
    sourcetooriginlabel->setEnabled(true);
    sourcetodetector->setEnabled(true);
    sourcetodetectorlabel->setEnabled(true);
    parallelbuttongroup->setEnabled(false);
    parallelbuttongroup2->setEnabled(false);
    //    activateOKButton();
  }
  else {
    divergentbuttongroup->setEnabled(false);
    sourcetoorigin->setEnabled(false);
    sourcetooriginlabel->setEnabled(false);
    sourcetodetector->setEnabled(false);
    sourcetodetectorlabel->setEnabled(false);
    //    activateOKButton();
  }
}


/** Activates user fields and deactivates program fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateUserFields()
{
  if(userbutton->isChecked()) {
    userrays->setEnabled(true);
    userrayslabel->setEnabled(true);
    raysumsnelem->setEnabled(false);
    raysumsnelemlabel->setEnabled(false);
    raysumspixelsize->setEnabled(false);
    raysumspixelsizelabel->setEnabled(false);
    //    activateOKButton();
  }
  else {
    userrays->setEnabled(false);
    userrayslabel->setEnabled(false);
    //    activateOKButton();
  }
}

/** Activates program fields and deactivates user fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateProgramFields()
{
  if(programbutton->isChecked()) {
    raysumsnelem->setEnabled(true);
    raysumsnelemlabel->setEnabled(true);
    raysumspixelsize->setEnabled(true);
    raysumspixelsizelabel->setEnabled(true);
    userrays->setEnabled(false);
    userrayslabel->setEnabled(false);
    //    activateOKButton();
  }
  else {
    raysumsnelem->setEnabled(false);
    raysumsnelemlabel->setEnabled(false);
    raysumspixelsize->setEnabled(false);
    raysumspixelsizelabel->setEnabled(false);
    //    activateOKButton();
  }
}

/** Activates equal spacing fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateEqualSpacingFields()
{
  if(equalspacingbutton->isChecked()) {
    firstangle->setEnabled(true);
    firstanglelabel->setEnabled(true);
    lastangle->setEnabled(true);
    lastanglelabel->setEnabled(true);
    angles->setEnabled(false);
    //    activateOKButton();
  }
  else {
    firstangle->setEnabled(false);
    firstanglelabel->setEnabled(false);
    lastangle->setEnabled(false);
    lastanglelabel->setEnabled(false);
    angles->setEnabled(true);
    //    activateOKButton();
  }
}


/** Deactivates equal spacing fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::deactivateEqualSpacingFields()
{
  firstangle->setEnabled(false);
  firstanglelabel->setEnabled(false);
  lastangle->setEnabled(false);
  lastanglelabel->setEnabled(false);
  if(linobutton->isChecked()) 
    angles->setEnabled(false);
  else
    if(nonebutton->isChecked()) 
      angles->setEnabled(true);
  //  activateOKButton();
}



/** Activates noisy fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateNoisyFields()
{
  if(noisybutton->isChecked()) {
    noisyframe->setEnabled(true);
    quantumbutton->setChecked(true);
    quantumbutton->setEnabled(true);
    quanmn->setEnabled(true);
    quanmnlabel->setEnabled(true);
    quancm->setEnabled(true);
    quancmlabel->setEnabled(true);
    calibrationlabel->setEnabled(true);
    quanin->setEnabled(true); 
    quaninlabel->setEnabled(true);
    seedlabel->setEnabled(true);
    seed->setEnabled(true);
    seedlabel1->setEnabled(true);
    //    activateOKButton();
  }
  else {
    noisyframe->setEnabled(false);
    //    activateOKButton();
  }
}

/** Deactivates noisy fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::deactivateNoisyFields()
{
  noisyframe->setEnabled(false);
  seedlabel->setEnabled(false);
  seed->setEnabled(false);
  seedlabel1->setEnabled(false);
  //  activateOKButton();
}

/** Activates quantum fields and deactivates scatter, additive and 
multiplicative fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateQuantumFields()
{
  if(quantumbutton->isChecked()) {
    quanmn->setEnabled(true);
    quanmnlabel->setEnabled(true);
    quancm->setEnabled(true);
    quancmlabel->setEnabled(true);
    calibrationlabel->setEnabled(true);
    quanin->setEnabled(true);
    quaninlabel->setEnabled(true);
    seed->setEnabled(true);
    seedlabel->setEnabled(true);
    seedlabel1->setEnabled(true);
    //    activateOKButton();
  }
  else {
    quanmn->setEnabled(false);
    quanmnlabel->setEnabled(false);
    quancm->setEnabled(false);
    quancmlabel->setEnabled(false);
    calibrationlabel->setEnabled(false);
    quanin->setEnabled(false);
    quaninlabel->setEnabled(false);
    if(!additivebutton->isChecked() && !multiplicativebutton->isChecked()) {
      seed->setEnabled(false);
      seedlabel->setEnabled(false);
      seedlabel1->setEnabled(false);
    }
    //    activateOKButton();
  }
}

/** Activates scatter fields and deactivates quantum, additive and 
multiplicative fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateScatterFields()
{
  if(scatterbutton->isChecked()) {
    sctnpk->setEnabled(true);
    sctnpklabel->setEnabled(true);
    sctnwd->setEnabled(true);
    sctnwdlabel->setEnabled(true);
    //    activateOKButton();
  }
  else {
    sctnpk->setEnabled(false);
    sctnpklabel->setEnabled(false);
    sctnwd->setEnabled(false);
    sctnwdlabel->setEnabled(false);
    //    activateOKButton();
  }
}

/** Activates additive fields and deactivates quantum, scatter and 
multiplicative fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateAdditiveFields()
{
  if(additivebutton->isChecked()) {
    addnmn->setEnabled(true);
    addnmnlabel->setEnabled(true);
    addnsd->setEnabled(true);
    addnsdlabel->setEnabled(true);
    seed->setEnabled(true);
    seedlabel->setEnabled(true);
    seedlabel1->setEnabled(true);
    //    activateOKButton();
  }
  else {
    addnmn->setEnabled(false);
    addnmnlabel->setEnabled(false);
    addnsd->setEnabled(false);
    addnsdlabel->setEnabled(false);
    if(!quantumbutton->isChecked() && !multiplicativebutton->isChecked()) {
      seed->setEnabled(false);
      seedlabel->setEnabled(false);
      seedlabel1->setEnabled(false);
    }
    //    activateOKButton();
  }
}

/** Activates multiplicative fields and deactivates quantum, scatter and 
additive fields
@param void
@author Bruno M. Carvalho
@version 1.0 */
void projectionwindow::activateMultiplicativeFields()
{
  if(multiplicativebutton->isChecked()) {
    multnmn->setEnabled(true);
    multnmnlabel->setEnabled(true);
    multnsd->setEnabled(true);
    multnsdlabel->setEnabled(true);
    seed->setEnabled(true);
    seedlabel->setEnabled(true);
    seedlabel1->setEnabled(true);
    //    activateOKButton();
  }
  else {
    multnmn->setEnabled(false);
    multnmnlabel->setEnabled(false);
    multnsd->setEnabled(false);
    multnsdlabel->setEnabled(false);
    if(!quantumbutton->isChecked() && !additivebutton->isChecked()) {
      seed->setEnabled(false);
      seedlabel->setEnabled(false);
      seedlabel1->setEnabled(false);
    }
    //    activateOKButton();
  }
}


/** Returns a string for PROJECTION input sequence
@param void
@return projection
@author Bruno M. Carvalho / DS
@version 1.0 */
QString projectionwindow::getOutput()
{
  QString output;

  output.sprintf("PROJECTION ");
  if(oldbutton->isChecked()) {
    output.append("OLD ");
  }
  if(realbutton->isChecked()) {
    output.append("REAL ");
    if (bhcbutton->isChecked() ) {
      output += "BEAM HARDENING CORRECTION\n";
      output += bhcwindow->text();
    }
  }
  if(pseudobutton->isChecked()) {
    output += "PSEUDO\n";
    output += get_idheader();
    output += "\n";
    output += get_geometry();
    output += "\n";
    output += get_measurement();
  }
  return output;
} // --projectionwindow::getOutput()

void projectionwindow::checkValues()
{
  QString err;// contains the error string
  if(!pseudobutton->isChecked()) {
    accept(); // that was easy.
  } else {
    if (!check_idheader(err)) {   //  don't accept, show why
      QMessageBox::information(this,"SnarkInput",err);
    } else {
      // i was going to do this more elegantly using
      // check_geometry(err) and check_measurement(err) but realized
      // how much code would have to be rewritten...
      // just stole the part below from bruno's create.cpp  --deniz
      int f=0;
      int usr=0,pnlm,pn,sed;
      float stoo,stod=0,ppxls,decs,fa,la,qm,qc,sw,sp,am,as,mm,ms;
      float b1;

      /* Consistency check for geometry section */
      if(divergentbutton->isChecked()) {
	stoo=strtof((const char *)sourcetoorigin->text());
	if(errnum) 
	  f=21;
	stod=strtof((const char *)sourcetodetector->text());
	if(errnum) 
	  f=22;
	if(stoo<0)
	  f=30;
	if(stod<0)
	  f=31;
      }
      if(userbutton->isChecked()) {
	usr=strtoi((const char *)userrays->text());
	if(errnum) 
	  f=23;
	if(usr<1 || !(usr%2))
	  f=34;
      }
      if(programbutton->isChecked()) {
	pnlm=strtoi((const char *)raysumsnelem->text());
	if(errnum) 
	  f=24;
	if(pnlm<1 || !(pnlm%2))
	  f=35;
	ppxls=strtof((const char *)raysumspixelsize->text());
	if(errnum) 
	  f=25;
	if(ppxls<=0)
	  f=32;
	else 
	  if(divergentbutton->isChecked() && pnlm*ppxls*M_SQRT1_2 >= stoo)
	    f=37;
      }
      decs=strtof((const char *)detectorspacing->text());
      if(errnum) 
	f=26;
      if(decs<0)
	f=33;
      if(divergentbutton->isChecked() && userbutton->isChecked()) 
	if((usr-1)*decs>=stod*M_PI)
	  f=38;
      pn=strtoi((const char *)prjnum->text());
      if(errnum) 
	f=27;
      if(pn<1)
	f=36;
      if(linobutton->isChecked())
	if(pn<2)
	  f=36;
      if(equalspacingbutton->isChecked()) {
	fa=strtof((const char *)firstangle->text());
	if(errnum) 
	  f=28;
	la=strtof((const char *)lastangle->text());
	if(errnum) 
	  f=29;
	if(pn<2)
	  f=36;
      }
      if(nonebutton->isChecked()) {
	QString angst = angles->text();
	//Splits angst into strings wherever the regular expression sep occurs, and return the list of those strings.
	// If allowEmptyEntries is TRUE, a null string is inserted in the list wherever the separator matches twice without intervening text.
	QRegExp sep("\\s+");
	// whitespace.  the first '\' is for escaping '\'
	QStringList angleStringList = QStringList::split(sep, angst);
	int angleCount = 0;
	bool ok;
	for ( QStringList::Iterator it = angleStringList.begin(); 
	      it != angleStringList.end(); ++it ) {
	  // check (*it) is really a float
	  // double QString::toDouble ( bool * ok = 0 ) const
	  ok = true;
	(*it).toDouble(&ok);
	if(!ok) f=39;
	angleCount++;
	}
	// pn is already numerified value in prjnum field
	if(angleCount != pn) f=39;
      }
      
      /* Consistency check for measurement section */
      
      if(noisybutton->isChecked()) {
	if(scatterbutton->isChecked()) {
	  sp=strtof((const char *)sctnpk->text());
	  if(errnum) 
	    f=42;
	  if(sp<=0)
	    f=51;
	  sw=strtof((const char *)sctnwd->text());
	  if(errnum) 
	    f=43;
	  if(sw<decs)
	    f=53;
	}
	else {
	  if(quantumbutton->isChecked() || additivebutton->isChecked() || multiplicativebutton->isChecked()) {
	    sed=strtoi((const char *)seed->text());
	    if(errnum) 
	      f=48;
	  }
	  if(quantumbutton->isChecked()) {
	    qm=strtof((const char *)quanmn->text());
	    if(errnum) 
	      f=40;
	    qc=strtof((const char *)quancm->text());
	    if(errnum) 
	      f=41;
	    if(qc<=0)
	      f=50;
	  }
	  if(additivebutton->isChecked()) {
	    am=strtof((const char *)addnmn->text());
	    if(errnum) 
	      f=44;
	    as=strtof((const char *)addnsd->text());
	    if(errnum) 
	      f=45;
	  }
	  if(multiplicativebutton->isChecked()) {
	    mm=strtof((const char *)multnmn->text());
	    if(errnum) 
	      f=46;
	    if(mm==0)
	      f=52;
	    ms=strtof((const char *)multnsd->text());
	    if(errnum) 
	      f=47;
	  }
	}
	if(!quantumbutton->isChecked() && !scatterbutton->isChecked() && !additivebutton->isChecked() && !multiplicativebutton->isChecked())
	  f=54;
      }
      
      b1=strtof((const char *)backgr1->text());
      if(errnum) 
	f=49;

      // and then do something based on value of error flag f
      switch(f) {
      case 0: accept();
	break;
	
      case 21: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid source_to_origin format.");
	pseudoCommandsWidget->setCurrentPage(1);
	sourcetoorigin->setFocus();
	break;
      case 22: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid source_to_detector format.");
	pseudoCommandsWidget->setCurrentPage(1);
	sourcetodetector->setFocus();
	break;
      case 23: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid user_rays format.");
	pseudoCommandsWidget->setCurrentPage(1);
	userrays->setFocus();
	break;
      case 24: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid nelem format.");
	pseudoCommandsWidget->setCurrentPage(1);
	raysumsnelem->setFocus();
	break;
      case 25: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid pixel_size format.");
	pseudoCommandsWidget->setCurrentPage(1);
	raysumspixelsize->setFocus();
	break;
      case 26: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid detector_spacing format.");
	pseudoCommandsWidget->setCurrentPage(1);
	detectorspacing->setFocus();
	break;
      case 27: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid prjnum format.");
	pseudoCommandsWidget->setCurrentPage(1);
	prjnum->setFocus();
	break;
      case 28: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid first_angle format.");
	pseudoCommandsWidget->setCurrentPage(1);
	firstangle->setFocus();
	break;
      case 29: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid last_angle format.");
	pseudoCommandsWidget->setCurrentPage(1);
	lastangle->setFocus();
	break;
      case 30: QMessageBox::information(this,"SnarkInput","Error!\n"
					"source_to_origin must be positive.");
	pseudoCommandsWidget->setCurrentPage(1);
	sourcetoorigin->setFocus();
	break;
      case 31: QMessageBox::information(this,"SnarkInput","Error!\n"
					"source_to_detector must be positive.");
	pseudoCommandsWidget->setCurrentPage(1);
	sourcetodetector->setFocus();
	break;
      case 32: QMessageBox::information(this,"SnarkInput","Error!\n"
					"pixel_size must be positive.");
	pseudoCommandsWidget->setCurrentPage(1);
	raysumspixelsize->setFocus();
	break;
      case 33: QMessageBox::information(this,"SnarkInput","Error!\n"
					"detector_spacing must be equal or greater than 0.");
	pseudoCommandsWidget->setCurrentPage(1);
	detectorspacing->setFocus();
	break;
      case 34: QMessageBox::information(this,"SnarkInput","Error!\n"
					"user_rays must be positive and odd.");
	pseudoCommandsWidget->setCurrentPage(1);
	userrays->setFocus();
	break;
      case 35: QMessageBox::information(this,"SnarkInput","Error!\n"
					"nelem must be positive and odd.");
	pseudoCommandsWidget->setCurrentPage(1);
	raysumsnelem->setFocus();
	break;
      case 36: QMessageBox::information(this,"SnarkInput","Error!\n"
					"projnum must be positive, and greater than 1\n"
					"if LINO or EQUAL_SPACING is selected.");
	pseudoCommandsWidget->setCurrentPage(1);
	prjnum->setFocus();
	break;
      case 37: QMessageBox::information(this,"SnarkInput","Error!\n"
					"nelem * pixel_size/sqrt(2) must be smaller than source_to_detector.");
	pseudoCommandsWidget->setCurrentPage(1);
	raysumsnelem->setFocus();
	break;
      case 38: QMessageBox::information(this,"SnarkInput","Error!\n"
					"(user_rays-1) * detector_spacing must be smaller than source_to_detector * PI.");
	pseudoCommandsWidget->setCurrentPage(1);
	detectorspacing->setFocus();
	break;
	// case 39 is never reached, since this was only half implemented.
      case 39: QMessageBox::information(this,"SnarkInput","Error!\n"
					"number of angles must be equal to prjnum and each angle must be a floating-point number.");
	pseudoCommandsWidget->setCurrentPage(1);
	angles->setFocus();
	break;
      case 40: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid quanmn format.");
	pseudoCommandsWidget->setCurrentPage(2);
	quanmn->setFocus();
	break;
      case 41: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid quancm format.");
	pseudoCommandsWidget->setCurrentPage(2);
	quancm->setFocus();
	break;
      case 42: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid sctnpk format.");
	pseudoCommandsWidget->setCurrentPage(2);
	sctnpk->setFocus();
	break;
      case 43: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid sctnwd format.");
	pseudoCommandsWidget->setCurrentPage(2);
	sctnwd->setFocus();
	break;
      case 44: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid addnmn format.");
	pseudoCommandsWidget->setCurrentPage(2);
	addnmn->setFocus();
	break;
      case 45: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid addnsd format.");
	pseudoCommandsWidget->setCurrentPage(2);
	addnsd->setFocus();
	break;
      case 46: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid multnmn format.");
	pseudoCommandsWidget->setCurrentPage(2);
	multnmn->setFocus();
	break;
      case 47: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid multnsd format.");
	pseudoCommandsWidget->setCurrentPage(2);
	multnsd->setFocus();
	break;
      case 48: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid seed format.");
	pseudoCommandsWidget->setCurrentPage(2);
	seed->setFocus();
	break;
      case 49: QMessageBox::information(this,"SnarkInput","Error!\n"
					"Invalid backgr(i) format.");
	pseudoCommandsWidget->setCurrentPage(2);
	backgr1->setFocus();
	break;
      case 50: QMessageBox::information(this,"SnarkInput","Error!\n"
					"quancm must be positive.");
	pseudoCommandsWidget->setCurrentPage(2);
	quancm->setFocus();
	break;
      case 51: QMessageBox::information(this,"SnarkInput","Error!\n"
					"sctnpk must be positive");
	pseudoCommandsWidget->setCurrentPage(2);
	sctnpk->setFocus();
	break;
      case 52: QMessageBox::information(this,"SnarkInput","Error!\n"
					"multnmn must be nonzero.");
	pseudoCommandsWidget->setCurrentPage(2);
	multnmn->setFocus();
	break;
      case 53: QMessageBox::information(this,"SnarkInput","Error!\n"
					"sctnwd must be greater than detector_spacing.");
	pseudoCommandsWidget->setCurrentPage(2);
	sctnwd->setFocus();
	break;
      case 54: QMessageBox::information(this,"SnarkInput","Error!\n"
					"if NOISY is selected at least one of the options\n"                                   
					" QUANTUM, SCATTER, ADDITIVE OR MULTIPLICATIVE must be selected too.");
	pseudoCommandsWidget->setCurrentPage(2);
	quantumbutton->setFocus();
	break;
      } // -- switch(f)
    } // -- if didn't fail at the idheader    
  } // -- if pseudo /was/ checked
} // --projectionwindow::checkValues()
