/*  
 **********************************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/congeo.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 **********************************************************************
 
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

 THIS SUBROUTINE TRANSFORMS ONE DIVERGENT BEAM OF
 PROJECTION DATA
 PROJEC(NR),NR=1,2,...,NRAYS,
 WITH DETECTOR SPACING PINC ( AND EITHER ARC OR
 TANGENT ) INTO A COMPLETE FAN-BEAM OF PROJECTION
 DATA
 NUPROJ(L),L=1,NURAYS,
 AT DETECTOR SPACING NUPINC ( AS IF WE HAD THE ARC
 CASE ), WHERE NURAYS IS EVEN, AND NURAYS*NUPINC
 EQUALS PI*STOD, SUCH THAT NUPROJ(L) IS THE LINE INTEGRAL
 FOR THE RAY THROUGH THE SOURCE, THAT MAKES ANGLE
 L*(NUPINC/RADIUS) RADIANS
 WITH THE LINE ORTHOGONAL TO THE CENTRAL RAY IN THE
 DIVERGENT BEAM.THE CENTRAL RAYS ARE PROJEC(NRAYS/2+1)
 AND PROJEC(NURAYS/2+1).
 NUPNTS HAS TO BE INITIALIZED AS FOLLOWS:
 NUPNTS(NURAYS/2-I)=I*NUPINC/PINC, I=1,...,NURAYS/2-1,
 SO NUPNTS(NURAYS/2-I) IS THE SCALED POSITION OF THE
 I-TH RAY ON THE CIRCLE OF DETECTORS, MEASURED RELATIVE
 TO THE CENTRAL RAY.

 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 */

#include <cstdio>

#include "blkdta.h"
#include "uiod.h"

#include "congeo.h"

void congeo(REAL* projec, INTEGER nrays, REAL* nuproj, INTEGER nurays, REAL* nupnts)
{

	INTEGER nur2;
	INTEGER nra2;
	INTEGER indist;
	INTEGER j;
	INTEGER indnu;
	INTEGER indpro;
	INTEGER i;
	REAL space;
	INTEGER irefl;
	INTEGER indref;
	INTEGER nr2;
	INTEGER l;

	nur2 = nurays / 2;
	nra2 = nrays / 2;

	// THE FIRST (TANGENT) RAY

	nuproj[0] = 0.0;

	// THE CENTER RAY

	nuproj[nur2] = projec[nra2];

	// FILL IN POINTS OUTSIDE ORIGINAL TABLE

	indist = nra2;
	j = 1;
	for (;;)
	{
		j++;
		if (nupnts[j - 1] < float(indist))
			break;
		nuproj[j] = 0.0;
		indnu = nurays - j + 2;
		nuproj[indnu] = 0.0;
	}

//      THE POINTS INSIDE THE TABLE

	indist = indist - 1;
	indpro = nra2 - indist + 1;

	for (i = j; i < nur2; i++)
	{
		for (;;)
		{
			space = nupnts[i - 1] - ((REAL) (indist));
			if (space >= 0)
				break;
			indist--;
			indpro++;
		}
		nuproj[i] = space * projec[indpro - 1] + ((REAL) 1.0 - space) * projec[indpro];

		// THE CORRESPONDING POINTS ON THE NEGATIVE AXIS

		irefl = nurays - i + 2;
		indref = nrays - indpro + 1;
		nuproj[irefl] = ((REAL) 1.0 - space) * projec[indref]
				+ space * projec[indref + 1];

	}

	if (trace < 5)
		return;

	nr2 = nurays / 2 - 1;

	fprintf(output, "\n         interpolation performed at points:");
	for (l = 0; l < nr2; l++)
	{
		if ((l % 5) == 0)
		{
			fprintf(output, "\n          ");
		}
		fprintf(output, "%13.6e", nupnts[l]);
	}

	fprintf(output, "\n         projection data");
	for (l = 0; l < nrays; l++)
	{
		if ((l % 5) == 0)
		{
			fprintf(output, "\n          ");
		}
		fprintf(output, "%13.6e", projec[l]);
	}

	fprintf(output, "\n         interpolated projection data");
	for (l = 0; l < nurays; l++)
	{
		if ((l % 5) == 0)
		{
			fprintf(output, "\n          ");
		}
		fprintf(output, " %13.6e", nuproj[l]);
	}

	trace--;

	return;
}
